#!/usr/bin/env python2
import sys
import os
from optparse import OptionParser

def setParam(option, opt, value, parser):
	setattr(parser.values, option.dest, value)

COMPILE=""
def setTool(option, opt, value, parser):
	global COMPILE
	COMPILE="TOOL"
	setParam(option, opt, value, parser)

def setLib(option, opt, value, parser):
	global COMPILE
	COMPILE="LIB"
	setParam(option, opt, value, parser)

def setConfig(option, opt, value, parser):
	if value != "":
		os.putenv("USER_CONFIG_FILE", value)
	
def checkBuildType(parser):
	if (parser.values.BUILD == "") or (parser.values.BUILD == "Debug") or (parser.values.BUILD == "Release") or (parser.values.BUILD == "RelWithDebInfo") or (parser.values.BUILD == "MinSizeRel"):
		return True
	else:
		return False

def checkParams(parser):
	if bool(parser.values.TOOL) ^ bool(parser.values.LIB):
		return checkBuildType(parser)
	else:
		print "ERROR :: You must set lib OR tool param to compile."
		return False

#-- main()

def runCmd( command ):
	print command
	result = os.system(command)
	if not result == 0:
		sys.exit(1)

def setOptions(parser):
	parser.add_option("-t", "--tool", dest="TOOL", action="callback", callback=setTool, type="string", help="Specify the TOOL to compile.", metavar="TOOL")
	parser.add_option("-l", "--lib", dest="LIB", action="callback", callback=setLib, type="string", help="Specify the LIB to compile.", metavar="LIB")
	parser.add_option("-c", "--config", dest="CONFIG", action="callback", callback=setConfig, type="string", help="Specify the USER_CONFIG_FILE to use.", default="", metavar="CONFIG")
	parser.add_option("-o", "--options", dest="SHOW_OPTIONS", action="store_true", default=False, help="Get the options defined in the LIB/TOOL. Also is needed to set -l or -t param.", metavar="SHOW_OPTIONS")
	parser.add_option("-d", "--deps", dest="SHOW_DEPS", action="store_true", default=False, help="Get target dependencies. Also is needed to set -l or -t param.", metavar="SHOW_DEPS")
	parser.add_option("-p", "--platform", dest="PLATFORM", action="callback", callback=setParam, type="string", help="Set the PLATFORM to compile.", default="", metavar="PLATFORM")
	parser.add_option("-k", "--pack", dest="PACKAGE", action="store_true", help="Pack the project?", default=False, metavar="PACKAGE")
	parser.add_option("-b", "--build", dest="BUILD", action="callback", callback=setParam, type="string", default="", help="Set the build type for make based generators.")
	parser.add_option("-v", "--verbose", dest="VERBOSE", action="store", type="int", default=0, help="Put cmake debug mode")

def setupConfigure(parser):
	DEPOT = os.path.dirname(os.path.dirname(os.path.abspath( __file__ )))
	
	if os.getenv("DEPOT") == None:
		os.putenv("DEPOT",DEPOT)
	else:
		DEPOT = os.getenv("DEPOT")

	if parser.values.PLATFORM == "":
		PLATFORM=""
	else:
		PLATFORM="-DCMAKE_TOOLCHAIN_FILE=\"%s/config/target/Platform_%s.cmake\"" % (DEPOT, parser.values.PLATFORM)

	if not parser.values.BUILD == "":
		BUILD="-DCMAKE_BUILD_TYPE=\""+parser.values.BUILD+"\""
	else:
		BUILD=""

	if not parser.values.VERBOSE == 0:
		VERBOSITY="-DVERBOSITY=%d" % parser.values.VERBOSE
	else:
		VERBOSITY=""

	if COMPILE == "TOOL":
		IS_TOOL="-DIS_TOOL=1"
		COMPILE_PROJECT="-DCOMPILE_PROJECT=\""+parser.values.TOOL+"\""
	else:
		IS_TOOL="-DIS_TOOL=0"
		COMPILE_PROJECT="-DCOMPILE_PROJECT=\""+parser.values.LIB+"\""

	return "cmake %s %s %s %s %s %s/config" % (VERBOSITY, BUILD, IS_TOOL, COMPILE_PROJECT, PLATFORM, DEPOT)


parser = OptionParser()
setOptions(parser)

parser.parse_args()

#print parser.values
if checkParams(parser):
	# Configure the project
	runCmd( setupConfigure(parser) )

	if parser.values.SHOW_OPTIONS:
		runCmd( "cmake --build . --target options" )
	elif parser.values.SHOW_DEPS:
		runCmd( "cmake --build . --target deps" )
	else:
		if not parser.values.BUILD == "":
			cfg = "--config %s " % parser.values.BUILD
		else:
			cfg = ""

		# build them
		runCmd( "cmake --build . %s" % cfg )

		if(parser.values.PACKAGE):
			runCmd( "cmake --build . --target pack %s" % cfg )

else:
	parser.print_help()

