#Make the prebuild for each project of the lib/tool compilation.
MACRO (DoBuild type)
	SET(VERSION ${PRE_BUILD_PROJECT_VERSION})

	# To avoid problems when using the name in #define
	STRING(TOUPPER ${PRE_BUILD_PROJECT_NAME} PROJECT_NAME_UPPERCASE )
	STRING(REPLACE "-" "_" PROJECT_NAME_UPPERCASE_ESCAPED ${PROJECT_NAME_UPPERCASE} )
	LIST(APPEND CMAKE_MODULE_PATH ${PROJECT_SOURCE_DIR})

	# Process project
	PROCESS_PROJECT_AUX( ${PRE_BUILD_PROJECT_NAME} ${PROJECT_SOURCE_DIR} ${PRE_BUILD_PROJECT_NAME}_info 1 )

	# Define source groups
	define_source_groups( ${${PRE_BUILD_PROJECT_NAME}_info_SOURCE_FILES} )

	# Setup output name and returns the name
	IF(${type} STREQUAL "lib")
		# Set output name and destination
		SET_LIBNAME( out_name ${PRE_BUILD_PROJECT_NAME} )
		set( out_dir "lib" )

		# Setup library build type
		IF(BUILD_SHARED)
			ADD_LIBRARY( ${PRE_BUILD_PROJECT_NAME} SHARED ${${PRE_BUILD_PROJECT_NAME}_info_SOURCE_FILES} )
			stripBinary( ${PRE_BUILD_PROJECT_NAME} )
		ELSE(BUILD_SHARED)
			ADD_LIBRARY( ${PRE_BUILD_PROJECT_NAME} STATIC ${${PRE_BUILD_PROJECT_NAME}_info_SOURCE_FILES} )
		ENDIF(BUILD_SHARED)
	ELSE()
		# Get revision
		GET_GIT_REVISION( ${PRE_BUILD_PROJECT_NAME}_TAG_VERSION ${PRE_BUILD_PROJECT_NAME}_BUILD_VERSION )

		# Setup executable
		add_system_executable( ${PRE_BUILD_PROJECT_NAME} ${PRE_BUILD_PROJECT_NAME}_info_SOURCE_FILES )

		# Set output name and destination
		SET_TOOLNAME( out_name ${PRE_BUILD_PROJECT_NAME} )
		set( out_dir ${PRE_BUILD_BIN_DESTINATION} )

		# Add strip if necesary
		stripBinary( ${PRE_BUILD_PROJECT_NAME} )

		# Add package construction
		DO_PACKAGE()
	ENDIF(${type} STREQUAL "lib")
	set_target_properties( ${PRE_BUILD_PROJECT_NAME} PROPERTIES OUTPUT_NAME ${out_name} )

	# Add project dependencies
	GET_SOURCE_DIR( ${type} ${PRE_BUILD_PROJECT_NAME} src )
	BUILD_PROJECT_DEPS( ${type} ${PRE_BUILD_PROJECT_NAME} ${src} build )

	# Add project definitions
	LIST( APPEND build_DEFINITIONS "-DPROJECT_NAME=${PRE_BUILD_PROJECT_NAME}" )
	IF(${CMAKE_BUILD_TYPE} STREQUAL "Debug")
		LIST( APPEND build_DEFINITIONS "-D_DEBUG" )
	ELSE()
		LIST( APPEND build_DEFINITIONS "-DNDEBUG" )
		IF(${CMAKE_BUILD_TYPE} STREQUAL "Release")	# TODO: Check this!
			SET(CMAKE_SHARED_LIBRARY_LINK_C_FLAGS "")
			SET(CMAKE_SHARED_LIBRARY_LINK_CXX_FLAGS "")
		ENDIF(${CMAKE_BUILD_TYPE} STREQUAL "Release")
	ENDIF()
	CLEAN_LIST( build_DEFINITIONS )
	add_definitions( ${build_DEFINITIONS} )

	# Add project include directories
	LIST( APPEND build_INCLUDE_DIRECTORIES ${PROJECT_BINARY_DIR} )
	CLEAN_LIST( build_INCLUDE_DIRECTORIES )
	INCLUDE_DIRECTORIES( ${build_INCLUDE_DIRECTORIES} )

	# Add project link directories
	CLEAN_LIST( build_LINK_DIRECTORIES )
	link_directories( ${build_LINK_DIRECTORIES} )

	# Add project link libraries
	CLEAN_LIST( build_LINK_LIBRARIES )
	TARGET_LINK_LIBRARIES( ${PRE_BUILD_PROJECT_NAME} ${build_LINK_LIBRARIES} )

	# Show status
	SHOW_STATUS()

	# Write generated/config.h file
	Write_Config( ${type} )

	# Compile and run tests
	IF(COMPILE_TESTS AND PRE_BUILD_PROJECT_TESTS)
		DO_TESTS( ${PRE_BUILD_PROJECT_TESTS} )
	ENDIF(COMPILE_TESTS AND PRE_BUILD_PROJECT_TESTS)

	# Install project
	installProject( ${PRE_BUILD_PROJECT_NAME} ${out_dir} )

	# Add additional targets
	add_extra_targets( ${PRE_BUILD_PROJECT_NAME} ${type} "" )
ENDMACRO (DoBuild)

# Do a package target with cpack
MACRO( DO_PACKAGE )
	# Setup some package variables
	INITVAR( PACKAGE_DESCRIPTION  "${PRE_BUILD_PROJECT_DESCRIPTION}" "Generic description" )
	INITVAR( PACKAGE_VERSION      "${PRE_BUILD_PROJECT_VERSION}" "1.0.0" )
	INITVAR( PACKAGE_MAINTAINER   "${PRE_BUILD_PACKAGE_MAINTAINER}" "ginga@lifia.info.unlp.edu.ar" )
	INITVAR( PACKAGE_NAME         "${PRE_BUILD_PACKAGE_NAME}" ${PRE_BUILD_PROJECT_NAME} )
	INITVAR( PACKAGE_LICENSE_FILE "${PRE_BUILD_LICENSE_FILE}" "${CONFIG_PATH}/package/COPYING" )

    # Setup cpack module path (To find NSIS.template.in)
	set( CPACK_MODULE_PATH "${CONFIG_PATH}/package" ${CPACK_MODULE_PATH} )

	# Generate the targets to build packages later
	INCLUDE_COMPONENTS( "${CONFIG_PATH}/package/CPackConfig" )
	INCLUDE_COMPONENTS( "${PROJECT_SOURCE_DIR}/cpack" )
	INCLUDE(CPack)
ENDMACRO( DO_PACKAGE )

# Get all project 
MACRO( BUILD_PROJECT_DEPS type name src out )
	IF (BUILD_SHARED)
		# Add direct internal dependencies (cflags and libs)
		get_internal_components( ${name}_info_INTERNAL_COMPONENTS internals )
		add_dependents_list( internals 1 1 ${out} )

		# Add direct external dependencies (cflags and libs)
		add_dependents_list( ${name}_info_EXTERNAL_COMPONENTS 1 1 ${out} )
	ELSE()
		# Add direct internal dependencies (only cflags)
		get_internal_components( ${name}_info_INTERNAL_COMPONENTS internals )
		add_dependents_list( internals 1 0 ${out} )

		# Add direct external dependencies (only cflags)
		add_dependents_list( ${name}_info_EXTERNAL_COMPONENTS 1 0 ${out} )

		# Add all internal dependencies if tool
		IF(${type} STREQUAL "tool")
			# Get all dependencies, in order
			GET_PROJECT_BUILD_ORDER( ${type} ${name} ${src} build_list 0 )

			# Reverse list
			IF (build_list)
			  LIST( REVERSE build_list )
			ENDIF (build_list)

			# Add all internal components (only libs)
			get_internal_components( build_list comp_internals )
			add_dependents_list( comp_internals 0 1 ${out} )

			# Join all external components
			FOREACH( dep ${build_list} )
			  LIST( APPEND externals ${${dep}_info_EXTERNAL_COMPONENTS} )
			ENDFOREACH( dep ${build_list} )
			LIST( APPEND externals ${${name}_info_EXTERNAL_COMPONENTS} )
			CLEAN_LIST( externals )

			# Add all externals (only libs)
			add_dependents_list( externals 0 1 ${out} )
		ENDIF(${type} STREQUAL "tool")
	ENDIF (BUILD_SHARED)
ENDMACRO( BUILD_PROJECT_DEPS type name out )

MACRO(SetPlatformName)
	IF (NOT PLATFORM)
		IF (UNIX)
			set( PLATFORM UNIX )
		ELSE(MSVC EQUAL 1)
			set( PLATFORM VS )
		ENDIF (UNIX)
	ENDIF (NOT PLATFORM)
ENDMACRO(SetPlatformName)

# Returns project full name
macro(GET_PROJECT_FULL_NAME type name full)
	IF(EXISTS "${DEPOT}/${type}/${name}")
		SET( ${full} "${name}")
	ELSE(EXISTS "${DEPOT}/${type}/${name}")
		IF(EXISTS "${DEPOT}/${type}/${PROJECT_PREFIX}-${name}")
			SET( ${full} "${PROJECT_PREFIX}-${name}")
		ELSE()
			message(FATAL_ERROR "\n -- \t ERROR: Path for the project \"${name}\" not found.\n")
		ENDIF(EXISTS "${DEPOT}/${type}/${PROJECT_PREFIX}-${name}")
	ENDIF(EXISTS "${DEPOT}/${type}/${name}")
endmacro(GET_PROJECT_FULL_NAME)

# Returns project source dir
macro(GET_SOURCE_DIR type name src_dir )
	GET_PROJECT_FULL_NAME( ${type} ${name} proj_name )
	SET( ${src_dir} "${DEPOT}/${type}/${proj_name}")
endmacro(GET_SOURCE_DIR)

# Returns project directories: src and build
macro(GET_PROJECT_DIR type name src_dir build_dir)
	GET_PROJECT_FULL_NAME( ${type} ${name} proj_name )
	SET( ${src_dir} "${DEPOT}/${type}/${proj_name}")
	SET( ${build_dir} "${PROJECT_BINARY_DIR}/${proj_name}")
endmacro(GET_PROJECT_DIR)

# Pass internal component to component names
macro( get_internal_components names comps )
	FOREACH( name ${${names}} )
		STRING( TOUPPER ${name} comp )
		LIST( APPEND ${comps} "GINGALIB-${comp}" )
	ENDFOREACH( name ${${names}} )
endmacro( get_internal_components names comps )

# Parse each package and copy the cflags and libs necesary to build
macro(add_dependents_list comps cflags libs out)
	FOREACH( comp ${${comps}} )
		IF (${cflags} EQUAL 1)
			LIST( APPEND ${out}_DEFINITIONS         ${${comp}_DEFINITIONS} )
			LIST( APPEND ${out}_INCLUDE_DIRECTORIES ${${comp}_INCLUDE_DIRS} ${${comp}_INCLUDE_DIR} )
		ENDIF (${cflags} EQUAL 1)

		IF (${libs} EQUAL 1)
			LIST( APPEND ${out}_LINK_DIRECTORIES ${${comp}_LIBRARY_DIRS} )
			EXTRACT_LIBS( ${out}_LINK_LIBRARIES ${comp} )
		ENDIF (${libs} EQUAL 1)
	ENDFOREACH( comp ${comps} )
endmacro(add_dependents_list comps out)

# Strip binary to discards all symbols from the current compilation.
macro (stripBinary name)
	IF(${CMAKE_BUILD_TYPE} STREQUAL "Release" AND STRIP_TOOL)
		ADD_CUSTOM_COMMAND( TARGET ${name} POST_BUILD COMMAND ${STRIP_TOOL} $<TARGET_FILE:${name}> )
	ENDIF()
endmacro(stripBinary)

# Add all tests
MACRO(DO_TESTS tests)
	FOREACH( test ${tests} )
		# Set test name		
		set( name "${PRE_BUILD_PROJECT_NAME}${test}" )
		set( src "${PROJECT_SOURCE_DIR}/${test}" )

		# Process project
		PROCESS_PROJECT_AUX( ${name} ${src} ${name}_info 1 )

		# Add components from project name
		LIST( APPEND ${name}_info_INTERNAL_COMPONENTS ${${PRE_BUILD_PROJECT_NAME}_info_INTERNAL_COMPONENTS} )
		LIST( APPEND ${name}_info_EXTERNAL_COMPONENTS ${${PRE_BUILD_PROJECT_NAME}_info_EXTERNAL_COMPONENTS} )

		# Build test dependencies
		BUILD_PROJECT_DEPS( "tool" ${name} ${src} test_build )

		# Add test exe
		add_system_executable( ${name} ${name}_info_SOURCE_FILES )

		# Link
		TARGET_LINK_LIBRARIES( ${name} ${PRE_BUILD_PROJECT_NAME} ${test_build_LINK_LIBRARIES} )

		IF(RUN_TESTS)
			ENABLE_TESTING()
			IF (EXISTS ${PROJECT_SOURCE_DIR}/test/test.sh)
				ADD_TEST( ${name} ${PROJECT_SOURCE_DIR}/test/test.sh ${CMAKE_BINARY_DIR}/${name} ${PROJECT_SOURCE_DIR})
			ELSE (EXISTS ${PROJECT_SOURCE_DIR}/test/test.sh)
				ADD_TEST( ${name} ${CMAKE_BINARY_DIR}/${name} )
			ENDIF(EXISTS ${PROJECT_SOURCE_DIR}/test/test.sh)
		ENDIF(RUN_TESTS)
	ENDFOREACH( test ${tests} )
ENDMACRO(DO_TESTS tests)

# Install the project into its corresponding directory
MACRO(installProject PROJECT_NAME DESTINATION)
	INSTALL( TARGETS ${PRE_BUILD_PROJECT_NAME}
		 DESTINATION ${DESTINATION}
		 CONFIGURATIONS ${CMAKE_BUILD_TYPE}
		 PERMISSIONS OWNER_READ OWNER_WRITE OWNER_EXECUTE
			     GROUP_READ 	    GROUP_EXECUTE
			     WORLD_READ 	    WORLD_EXECUTE )
ENDMACRO(installProject)

# Show log of the compiled project.
macro(SHOW_STATUS)
	# display status message for important variables
	MESSAGE( STATUS )
	MESSAGE( STATUS "-------------------------------------------------------------------------------" )
	MESSAGE( STATUS "PROJECT_NAME = ${PRE_BUILD_PROJECT_NAME}" )
	MESSAGE( STATUS "PROJECT_DESCRIPTION = ${PRE_BUILD_PROJECT_DESCRIPTION}" )
	MESSAGE( STATUS "PROJECT_VERSION = ${PRE_BUILD_PROJECT_VERSION}" )
	MESSAGE( STATUS "PLATFORM = ${PLATFORM}" )
	MESSAGE( STATUS "SYSTEM = ${CMAKE_SYSTEM_NAME}" )
	MESSAGE( STATUS "USER CONFIG FILE: ${SETUP_USER}" )
	MESSAGE( STATUS "PLATFORM_FILE = ${CMAKE_TOOLCHAIN_FILE}" )
	MESSAGE( STATUS "BUILD_SHARED = ${BUILD_SHARED}" )
	MESSAGE( STATUS "BUILD_TYPE = ${CMAKE_BUILD_TYPE}" )
	MESSAGE( STATUS "INSTALL_PREFIX = ${CMAKE_INSTALL_PREFIX}" )
	MESSAGE( STATUS "CONFIG_INSTALL_PREFIX = ${CMAKE_CONFIG_INSTALL_PREFIX}" )
	MESSAGE( STATUS "INTERNAL DEPENDENCIES = ${${PRE_BUILD_PROJECT_NAME}_info_INTERNAL_COMPONENTS}")
	MESSAGE( STATUS "EXTERNAL DEPENDENCIES = ${${PRE_BUILD_PROJECT_NAME}_info_EXTERNAL_COMPONENTS}")
	MESSAGE( STATUS "OPTIONS = ${${PRE_BUILD_PROJECT_NAME}_info_OPTIONS_NAMES}")
	if(${VERBOSITY} GREATER 0)
		MESSAGE( STATUS "CMAKE_MODULE_PATH = ${CMAKE_MODULE_PATH}" )
		MESSAGE( STATUS "CMAKE_FIND_ROOT_PATH = ${CMAKE_FIND_ROOT_PATH}" )
		MESSAGE( STATUS "DEFINITIONS = ${build_DEFINITIONS}" )
		MESSAGE( STATUS "INCLUDE_DIRECTORIES = ${build_INCLUDE_DIRECTORIES}" )
		MESSAGE( STATUS "LINK_DIRECTORIES = ${build_LINK_DIRECTORIES}" )
		MESSAGE( STATUS "LINK_LIBRARIES = ${build_LINK_LIBRARIES}")
	endif(${VERBOSITY} GREATER 0)
	IF(${VERBOSITY} GREATER 1)
		MESSAGE( STATUS "-------------------------------------------------------------------------------" )
		MESSAGE( STATUS "HEADER FILES = ${${PRE_BUILD_PROJECT_NAME}_info_HEADER_FILES}" )
		MESSAGE( STATUS "SOURCE FILES = ${${PRE_BUILD_PROJECT_NAME}_info_SOURCE_FILES}" )
	ENDIF(${VERBOSITY} GREATER 1)
	MESSAGE( STATUS "-------------------------------------------------------------------------------" )
endmacro(SHOW_STATUS)

MACRO (Write_Custom_Options file)
	if(EXISTS ${PROJECT_SOURCE_DIR}/${file})
		FILE( APPEND ${OUTPUT_CONFIG_FILE} "\n//\tOptions from ${file}\n")
		FILE( READ ${PROJECT_SOURCE_DIR}/${file} CONFIG_PROJECT )
		FILE( APPEND ${OUTPUT_CONFIG_FILE} "${CONFIG_PROJECT}" )
	endif(EXISTS ${PROJECT_SOURCE_DIR}/${file})
ENDMACRO (Write_Custom_Options)

# Save all configs from the current lib/tool compilation into a config.h for the compiled project.
MACRO (Write_Config buildType)
	SET( OUTPUT_CONFIG_FILE "${PROJECT_BINARY_DIR}/CMakeFiles/CMakeTmp/${PRE_BUILD_PROJECT_NAME}_config.h.cin" )

	# Create build options to use in the help
	FOREACH(group ${${PRE_BUILD_PROJECT_NAME}_info_OPTIONS_NAMES})
		LIST(APPEND BUILD_OPTS "${group}=${${group}}")
	ENDFOREACH(group)

	# Define macros foreach values are appended into tmp config of current project.
	FILE(WRITE ${OUTPUT_CONFIG_FILE} "#pragma once\n\n")
	FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_NAME " "\"${PRE_BUILD_PROJECT_NAME}\"\n")
	FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_VERSION " "\"${VERSION}\"\n")
	FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_BUILD_TYPE " "\"${CMAKE_BUILD_TYPE}\"\n")
	if(${buildType} STREQUAL "tool")
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_BUILD_VERSION " "\"${${PRE_BUILD_PROJECT_NAME}_BUILD_VERSION}\"\n")
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_DESCRIPTION " "\"${PRE_BUILD_PROJECT_DESCRIPTION}\"\n")
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_TAG_VERSION " "\"${${PRE_BUILD_PROJECT_NAME}_TAG_VERSION}\"\n")
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_BUILD_OPTS " "\"TYPE=${CMAKE_BUILD_TYPE};SHARED=${BUILD_SHARED};${BUILD_OPTS}\"\n")
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "#define ${PROJECT_NAME_UPPERCASE_ESCAPED}_PREFIX " "\"${CMAKE_CONFIG_INSTALL_PREFIX}\"\n")
	endif(${buildType} STREQUAL "tool")

	# Append autogenerated options
	FILE(APPEND ${OUTPUT_CONFIG_FILE} "\n")
	FOREACH(group ${${PRE_BUILD_PROJECT_NAME}_info_OPTIONS_NAMES})
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "//\t${group} OPTIONS\n")
		#define all groups options with their values
		FOREACH(opt ${${group}_OPTIONS})
			FILE(APPEND ${OUTPUT_CONFIG_FILE}"#define ${group}_USE_${opt} ${${group}_USE_${opt}}\n")
			IF(${group}_USE_${opt})
				STRING(TOUPPER ${opt} OPT_UPPER)
				WRITE_CUSTOM_OPTIONS( "config.${group}_${OPT_UPPER}.h.cin" )
			ENDIF(${group}_USE_${opt})
		ENDFOREACH(opt ${${group}_OPTIONS})
		FILE(APPEND ${OUTPUT_CONFIG_FILE} "\n")
	ENDFOREACH(group ${${PRE_BUILD_PROJECT_NAME}_info_OPTIONS_NAMES})

	# Write custom project config data
	WRITE_CUSTOM_OPTIONS( config.h.cin )

	CONFIGURE_FILE( ${OUTPUT_CONFIG_FILE} ${PROJECT_BINARY_DIR}/generated/config.h )
ENDMACRO (Write_Config)

# Check all project dependencies and returns the ordered build list of internal components
MACRO(GET_PROJECT_BUILD_ORDER type project src build_list only_ints)
	set( all_comps "" )

	# Set variable only if we are getting internals dependencies
	if (${only_ints})
		set( GET_ONLY_INTERNALS 1 )
	endif (${only_ints})

	# Process main project information	
	PROCESS_PROJECT_AUX( ${project} ${src} ${project}_info 0 )
	SET( comps ${${project}_info_INTERNAL_COMPONENTS} )
	CLEAN_LIST( comps )

	# Get all components that project depend
	WHILE (comps)
		# Pop first item
		LIST( GET comps 0 comp )
		LIST( REMOVE_AT comps 0 )

		# Add component to visited list
		LIST( APPEND all_comps ${comp} )

		# Process component
		PROCESS_PROJECT( "lib" ${comp} ${comp}_info 0 )
			
		# For each component in internal components
		FOREACH( int ${${comp}_info_INTERNAL_COMPONENTS})
			# Add int component if not in visited list
			LIST_CONTAINS( contains ${int} ${all_comps} )
			IF (NOT contains)
				LIST( APPEND comps ${int} )
			ENDIF (NOT contains)
		ENDFOREACH( int ${${comp}_info_INTERNAL_COMPONENTS})

		LIST(REMOVE_DUPLICATES comps)
	ENDWHILE (comps)

	# Reset internal dependency variable
	if (${only_ints})
		unset( GET_ONLY_INTERNALS )
	endif (${only_ints})

	# Remove duplicates, and the result is all internal components to build
	LIST(REMOVE_DUPLICATES all_comps)

	# Order the build list 
	set( build_comps "" )
	set( build_complete 0 )
	WHILE (${build_complete} EQUAL 0)
		set( builds 0 )

		# Try build at least one component
		FOREACH( comp ${all_comps})
			# IF all comp dependecies are build, add to list the build
			set( need_build 1 )
			FOREACH( int ${${comp}_info_INTERNAL_COMPONENTS} )
				LIST(FIND build_comps ${int} pos)
				IF (${pos} EQUAL -1)
					set( need_build 0 )
				ENDIF (${pos} EQUAL -1)
			ENDFOREACH( int ${${comp}_info_INTERNAL_COMPONENTS} )

			IF (${need_build} EQUAL 1)
				LIST( APPEND build_comps ${comp} )
				LIST( REMOVE_ITEM all_comps ${comp} )
				SET( builds 1 )
			ENDIF (${need_build} EQUAL 1)
		ENDFOREACH( comp ${all_comps})

		# Check if we can finish
		IF( ${builds} EQUAL 0)
			set( build_complete 1 )
		ENDIF (${builds} EQUAL 0)
	ENDWHILE (${build_complete} EQUAL 0)

	SET( ${build_list} ${build_comps} )
	IF(${VERBOSITY} GREATER 0)
		message(STATUS "Build list: ${build_comps}")
	ENDIF(${VERBOSITY} GREATER 0)
ENDMACRO(GET_PROJECT_BUILD_ORDER)

# Define groups from the source code files
macro(define_source_groups)
	set(CURRENT_GROUP "")
	set(CURRENT_GROUP_FILES "")
	foreach(file ${ARGN})
		#Get, by regex, the substring between the first "/" and the last "/" to define a source group
		#Ex: src/impl/.../aFile.h, with regex, obtains "/impl/.../"
		STRING(REGEX MATCH "[/\\](.*)[/\\]" SOURCE_GROUP_NAME ${file})
		string(LENGTH "${SOURCE_GROUP_NAME}" length )

		if(${length} GREATER 0)
			#If SOURCE_GROUP_NAME is a valid string, that needs remove the first "/" and the last "/" to define a valid source group.
			#Ex: regex obtains "/impl/.../", now remove the first "/" and the last "/" => "impl/..."
			MATH(EXPR length "${length}-2" )
			string(SUBSTRING ${SOURCE_GROUP_NAME} 1 ${length} SOURCE_GROUP_NAME)
			IF ( WIN32 )
				STRING( REGEX REPLACE "/" "\\\\" SOURCE_GROUP_NAME ${SOURCE_GROUP_NAME} )
			ENDIF ( WIN32 )
		endif(${length} GREATER 0)

		if("${SOURCE_GROUP_NAME}" EQUAL "${CURRENT_GROUP}")
			LIST(APPEND CURRENT_GROUP_FILES ${file})
		else("${SOURCE_GROUP_NAME}" EQUAL "${CURRENT_GROUP}")
			source_group("${CURRENT_GROUP}" FILES ${CURRENT_GROUP_FILES})
			set(CURRENT_GROUP ${SOURCE_GROUP_NAME})
			set(CURRENT_GROUP_FILES "")
			LIST(APPEND CURRENT_GROUP_FILES ${file})
		endif("${SOURCE_GROUP_NAME}" EQUAL "${CURRENT_GROUP}")
	endforeach(file ${PRE_BUILD_PROJECT_NAME}_SOURCE_FILES)
endmacro(define_source_groups)

# Process a project, get all information and puts result into proj_data
MACRO( PROCESS_PROJECT type project_name proj_data allComps )
	GET_SOURCE_DIR( ${type} ${project_name} dir )
	PROCESS_PROJECT_AUX( ${project_name} ${dir} ${proj_data} ${allComps} )
ENDMACRO( PROCESS_PROJECT )

# Process a project, get information and puts result into project_vars
# If allComps is
#     0 -> deps
#     1 -> deps,sources,install
MACRO( PROCESS_PROJECT_AUX project_name project_dir project_vars allComps )
	message( STATUS "Begin parse project: ${project_name} ${project_dir}")

	# Initialize project variables
	INIT_PROCESS_VARIABLES()

	# Process project files
	INCLUDE_PROJECT_COMPONENTS( ${project_dir} "deps" )
	IF(${allComps} EQUAL 1)
		INCLUDE_PROJECT_COMPONENTS( ${project_dir} "sources" )
		INCLUDE_PROJECT_COMPONENTS( ${project_dir} "install" )
	ENDIF()

	# Copy project variables
	COPY_PROCESS_VARIABLES( ${project_vars} )

	# Add header to sources
	LIST( APPEND ${project_vars}_SOURCE_FILES ${${project_vars}_HEADER_FILES} )

	message( STATUS "End parse project: ${project_name}")
ENDMACRO( PROCESS_PROJECT_AUX )

# Cleanup project variables. Used by PROCESS_PROJECT macro
MACRO( INIT_PROCESS_VARIABLES )
	SET( OPTIONS_NAMES "")
	SET( HEADER_FILES "")
	SET( SOURCE_FILES "")
	SET( INTERNAL_COMPONENTS "")
	SET( EXTERNAL_COMPONENTS "")
ENDMACRO( INIT_PROCESS_VARIABLES )

# Copy and unset project variables. Used by PROCESS_PROJECT macro
MACRO( COPY_PROCESS_VARIABLES var )
	COPY_AND_CLEAN_VARIABLE( ${var} OPTIONS_NAMES )
	COPY_AND_CLEAN_VARIABLE( ${var} HEADER_FILES )
	COPY_AND_CLEAN_VARIABLE( ${var} SOURCE_FILES )
	COPY_AND_CLEAN_VARIABLE( ${var} INTERNAL_COMPONENTS )
	COPY_AND_CLEAN_VARIABLE( ${var} EXTERNAL_COMPONENTS )
ENDMACRO( COPY_PROCESS_VARIABLES )

# Include components
MACRO(INCLUDE_COMPONENTS prefix)
	INCLUDE_IF_EXIST( ${prefix}.cmake )
	INCLUDE_IF_EXIST( ${prefix}.PLATFORM_${PLATFORM}.cmake )
	INCLUDE_IF_EXIST( ${prefix}.SYSTEM_${CMAKE_SYSTEM_NAME}.cmake )
ENDMACRO(INCLUDE_COMPONENTS prefix)

# Include components (deps/sources) and optional files of the current project compilation.
MACRO(INCLUDE_PROJECT_COMPONENTS path includePrefix)
	INCLUDE_COMPONENTS( "${path}/${includePrefix}" )

	# Process options
	FOREACH(group ${OPTIONS_NAMES})
		FOREACH(opt ${${group}_OPTIONS})
			IF(${group}_USE_${opt})
				STRING(TOUPPER ${opt} OPT_UPPER)
				SET(CURRENT_GROUP_OPTION ${group}_USE_${opt})
				INCLUDE_IF_EXIST(${path}/${includePrefix}.${group}_${OPT_UPPER}.cmake)
				UNSET(CURRENT_GROUP_OPTION)
			ENDIF()
		ENDFOREACH(opt)
	ENDFOREACH(group)
ENDMACRO(INCLUDE_PROJECT_COMPONENTS path includePrefix)

# Get the library file referenced by each member of the list
MACRO(EXTRACT_LIBS outputvar component)
	set(tmp_list_DEBUG "")
	Set(tmp_list_RELEASE "")
	Set(tmp_list_GENERAL "")

	SET(_list_suffix _GENERAL)
	FOREACH(lib ${${component}_LIBRARIES})
		IF("${lib}" STREQUAL "optimized")
				SET(_list_suffix _RELEASE)
		ELSE("${lib}" STREQUAL "optimized")
			IF("${lib}" STREQUAL "debug")
				SET(_list_suffix _DEBUG)
			ELSE("${lib}" STREQUAL "debug")
				IF("${lib}" STREQUAL "general")
					SET(_list_suffix _GENERAL)
				ELSE("${lib}" STREQUAL "general")
					LIST(APPEND tmp_list${_list_suffix} ${lib})
				ENDIF("${lib}" STREQUAL "general")
			ENDIF("${lib}" STREQUAL "debug")
		ENDIF("${lib}" STREQUAL "optimized")
	ENDFOREACH(lib ${${component}_LIBRARIES}})

	if(${CMAKE_BUILD_TYPE} STREQUAL "Release" OR ${CMAKE_BUILD_TYPE} STREQUAL "MinSizeRel" )
		LIST(APPEND ${outputvar} ${tmp_list_GENERAL} ${tmp_list_RELEASE} )
	else()
		LIST(APPEND ${outputvar} ${tmp_list_GENERAL} ${tmp_list_DEBUG} )
	endif(${CMAKE_BUILD_TYPE} STREQUAL "Release" OR ${CMAKE_BUILD_TYPE} STREQUAL "MinSizeRel" )
	LIST( REMOVE_DUPLICATES ${outputvar} )
ENDMACRO(EXTRACT_LIBS outputvar component)

# Add packages from the Boost libraries, returning their names in "where"
# This macro is used to avoid calling find_package for packages that are already added
MACRO(ADD_BOOST_COMPONENTS_IMPL prefix )
	find_package( Boost 1.46.0 REQUIRED ${ARGN} )
	ADD_DEPENDENTS_IMPL( Boost )
ENDMACRO(ADD_BOOST_COMPONENTS_IMPL)

MACRO(ADD_INTERNAL_COMPONENTS_IMPL prefix)
	IF(GET_ONLY_INTERNALS)
		LIST(APPEND ${prefix}INTERNAL_COMPONENTS ${ARGN})
	ELSE()
		find_package( GingaLib 0.11.0 REQUIRED ${ARGN} )
		LIST(APPEND ${prefix}INTERNAL_COMPONENTS ${GINGALIB_NAMES_COMPONENTS})
	ENDIF()
ENDMACRO(ADD_INTERNAL_COMPONENTS_IMPL)

# Add external dependencies to the current project processed
macro(ADD_DEPENDENTS_IMPL)
	LIST(APPEND EXTERNAL_COMPONENTS ${ARGN})
	if(${VERBOSITY} GREATER 0)
		message(STATUS "Appended [${ARGN}] into EXTERNAL_COMPONENTS")
	endif(${VERBOSITY} GREATER 0)
endmacro(ADD_DEPENDENTS_IMPL)

# Add cmake command
macro( add_cmake_build_cmd out_dir tgt opts result)
	set( ${result} cmake --build . --config ${CMAKE_BUILD_TYPE} )
	if (NOT "${tgt}" STREQUAL "")
		set( ${result} ${${result}} --target ${tgt} )
	endif (NOT "${tgt}" STREQUAL "")

	if (NOT "${opts}" STREQUAL "")
		set( ${result} ${${result}} -- "${opts}" )
	endif (NOT "${opts}" STREQUAL "")
endmacro( add_cmake_build_cmd )

# Add a new target
macro( add_target name type is_default dependencies )
	# Get directories
	GET_PROJECT_DIR( ${type} ${name} src_dir build_dir )

	# Create directory
	execute_process( COMMAND cmake -E make_directory ${build_dir} )

	# Setup build cmd (from platform/system)
	SET_BUILD_TARGET( ${build_dir} build_cmd install_cmd )

	if(${is_default})
		SET(_all ALL)
	endif()
	
	# obtain the debug param passed to this cmake.
	PARSE_VERBOSITY( ${VERBOSITY} verbose_param )

	# set cmake cmd
	set( cmake_cmd cmake ${verbose_param} "-DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE}" -DCMAKE_PROJECT_NAME=${name} )
	IF (CMAKE_TOOLCHAIN_FILE)
		set( cmake_cmd ${cmake_cmd} -DCMAKE_TOOLCHAIN_FILE=${CMAKE_TOOLCHAIN_FILE} )
	ENDIF (CMAKE_TOOLCHAIN_FILE)
	IF (${VERBOSITY} GREATER 1)
		set( cmake_cmd ${cmake_cmd} -DVERBOSITY=${VERBOSITY} -DCMAKE_VERBOSE_MAKEFILE=ON )
	ENDIF (${VERBOSITY} GREATER 1)
	set( cmake_cmd ${cmake_cmd} ${src_dir} )

	# Add target to build
	ADD_CUSTOM_TARGET(${name} ${_all}
		COMMAND cmake -E echo ${cmake_cmd}
		COMMAND ${cmake_cmd}
		COMMAND cmake -E echo ${build_cmd}
		COMMAND ${build_cmd}
		COMMAND cmake -E echo "test! ..."
		COMMAND ctest -V  # run tests
		COMMAND cmake -E echo ${install_cmd}
		COMMAND ${install_cmd}
		COMMAND cmake echo -E ${pack_cmd}
		COMMAND ${pack_cmd}                             # and pack if requested
		WORKING_DIRECTORY "${build_dir}"
		VERBATIM
	)

	# Add pack to default target
	if(${is_default})
		add_cmake_build_cmd( ${build_dir} package "" pack2_cmd)
		ADD_CUSTOM_TARGET( 
			"pack"
			COMMAND ${pack2_cmd}
			WORKING_DIRECTORY "${build_dir}"
		)
	endif()

	# Add dependencies
	FOREACH(dep ${dependencies})
		ADD_DEPENDENCIES( ${name} ${dep} )
	ENDFOREACH(dep ${dependencies})
endmacro( add_target )

# Extra targets
macro( ADD_EXTRA_TARGETS name type components )
	ADD_EXTRA_TARGET( ${name} ${type} eclipse -G "Eclipse CDT4 - Unix Makefiles" "${components}" )
	ADD_EXTRA_TARGET( ${name} ${type} KDevelop3 -G "KDevelop3" "${components}" )
	ADD_OPTIONS_TARGET( ${name} "${components}" )
	ADD_DEPS_TARGET( ${name} "${components}" )
endmacro( ADD_EXTRA_TARGETS )

# Aux targets functions
macro( ADD_EXTRA_TARGET name type target option param components )
	ADD_CUSTOM_TARGET( "${target}" )
	ADD_CMD_TO_TARGET( ${name} ${type} ${target} ${option} ${param}  )
	FOREACH(component ${components})
		ADD_CMD_TO_TARGET( ${component} "lib" ${target} ${option} ${param}  )
	ENDFOREACH(component)
endmacro( ADD_EXTRA_TARGET )

macro( ADD_CMD_TO_TARGET name type target option param )
	GET_PROJECT_DIR( ${type} ${name} src_dir build_dir )
	IF(NOT EXISTS "${build_dir}")
		SET( build_dir ${PROJECT_BINARY_DIR} )
	ENDIF()
	ADD_CUSTOM_COMMAND(
		TARGET "${target}"
		PRE_BUILD
		COMMAND cmake -E chdir ${build_dir} cmake ${option} ${param} -DCMAKE_TOOLCHAIN_FILE="${CMAKE_TOOLCHAIN_FILE}" ${src_dir} )
endmacro( ADD_CMD_TO_TARGET )

macro( ADD_ECHO_TO_TARGET target msg )
	ADD_CUSTOM_COMMAND( TARGET "${target}" PRE_BUILD COMMAND cmake -E echo ${msg} )
endmacro( ADD_ECHO_TO_TARGET )

# Options target
macro( ADD_OPTIONS_TARGET name components )
	set( target "options" )
	ADD_CUSTOM_TARGET( "${target}" )
	ADD_ECHO_TO_TARGET( ${target} "\n" )
	ADD_OPTIONS_TO_TARGET( ${name} ${target} )
	FOREACH(component ${components})
		ADD_OPTIONS_TO_TARGET( ${component} ${target} )
	ENDFOREACH(component)
	ADD_ECHO_TO_TARGET( ${target} "\n" )
endmacro( ADD_OPTIONS_TARGET )

macro( ADD_OPTIONS_TO_TARGET name target  )
	ADD_ECHO_TO_TARGET( ${target} ${name} )
	FOREACH(group ${${name}_info_OPTIONS_NAMES})
		ADD_ECHO_TO_TARGET( ${target} "\t \t${group}:" )
		FOREACH(opt ${${group}_OPTIONS})
			ADD_ECHO_TO_TARGET( ${target} "\t \t \t \t${opt} ${${group}_USE_${opt}}" )
		ENDFOREACH(opt)
	ENDFOREACH(group)
endmacro( ADD_OPTIONS_TO_TARGET )

# Deps target
macro( ADD_DEPS_TARGET name components )
	set( target "deps" )
	ADD_CUSTOM_TARGET( "${target}" )

	# Make internal list
	LIST( APPEND internals ${${name}_info_INTERNAL_COMPONENTS} )
	FOREACH(component ${components})
		LIST( APPEND internals ${${component}_info_INTERNAL_COMPONENTS} )
	ENDFOREACH(component)
	CLEAN_LIST(internals)

	# Make external list
	LIST( APPEND externals ${${name}_info_EXTERNAL_COMPONENTS} )
	FOREACH(component ${components})
		LIST( APPEND externals ${${component}_info_EXTERNAL_COMPONENTS} )
	ENDFOREACH(component)
	CLEAN_LIST(externals)

	# Show
	LIST(LENGTH internals len_i)
	LIST(LENGTH externals len_e)
	ADD_ECHO_TO_TARGET( ${target} "\n" )
	ADD_ECHO_TO_TARGET( ${target} "\tInternals '('${len_i}')': ${internals}" )
	ADD_ECHO_TO_TARGET( ${target} "\tExternals '('${len_e}')': ${externals}" )
	ADD_ECHO_TO_TARGET( ${target} "\n" )
endmacro( ADD_DEPS_TARGET )


