/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "font.h"
#include <boost/algorithm/string.hpp>
#include <vector>
#include <algorithm>

namespace canvas {

Font::Font()
{
	_families.push_back(DEFAULT_FONT);
	_size = 10;
	_bold = false;
	_italic = false;
	_smallCaps = false;
}

Font::Font( const std::vector<std::string> &families, size_t size, bool bold, bool italic, bool smallCaps )
	: _families( families )
{
	_size = size;
	_bold = bold;
	_italic = italic;
	_smallCaps = smallCaps;

	//	Add default font
	addDefault();	
}

Font::Font( const char *faces, size_t size, bool bold, bool italic, bool smallCaps )
{
	_size = size;
	_bold = bold;
	_italic = italic;
	_smallCaps = smallCaps;

	//	Add families
	boost::split( _families, faces, boost::is_any_of( "," ) );
	addDefault();
}

Font::~Font()
{
}

//	SETTERS
void Font::families( const std::vector<std::string> &families ) {
	_families = families;
	addDefault();
}

void Font::size( size_t size ) {
	_size = size;
}

void Font::bold( bool enable ) {
	_bold = enable;
}

void Font::italic( bool enable ) {
	_italic = enable;
}

void Font::smallCaps( bool enable ) {
	_smallCaps = enable;
}

//	GETTERS
const std::vector<std::string>& Font::families() const {
	return _families;
}

std::string Font::familiesAsString() const {
	std::string tmp;
	for (size_t i=0; i<_families.size(); i++) {
		if (i) {
			tmp += ",";
		}
		tmp += _families[i];
	}
	return tmp;
}

size_t Font::size() const {
	return _size;
}

bool Font::bold() const {
	return _bold;
}

bool Font::italic() const {
	return _italic;
}

bool Font::smallCaps() const {
	return _smallCaps;
}

void Font::addDefault() {
	std::vector<std::string>::iterator it = std::find( _families.begin(), _families.end(), DEFAULT_FONT );
	if (it == _families.end()) {
		_families.push_back(DEFAULT_FONT);
	}
}

bool Font::operator==(const Font &font) const {
	bool check = true;
	check &= (familiesAsString() == font.familiesAsString());
	check &= (_size == font.size());
	check &= (_bold == font.bold());
	check &= (_italic == font.italic());
	check &= (_smallCaps == font.smallCaps());
	return check;
}

bool Font::operator!=(const Font &font) const {
	return !((*this) == font);
}

bool Font::operator<(const Font &font) const {
	return _size < font.size();
}

}
