/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "group.h"
#include "surface.h"
#include <util/mcr.h>
#include <util/log.h>
#include <boost/foreach.hpp>

namespace canvas {

Group::Group( int id )
	: _id(id)
{
	_zIndex = 1;
}

Group::~Group()
{
	if (_surfaces.size()) {
		LWARN("Group", "Not all surfaces removed from group %d: size=%d", _id, _surfaces.size());
	}
}

int Group::id() const {
	return _id;
}

void Group::zIndex( int zIndex ) {
	if (_zIndex!=zIndex) {
		_zIndex = zIndex;
		BOOST_FOREACH( Surface* surface, _surfaces ) {
			surface->markDirty();
		}
	}
}

int Group::zIndex() const {
	return _zIndex;
}

void Group::add( Surface *surface ) {
	std::vector<Surface *>::iterator it=std::find(
		_surfaces.begin(), _surfaces.end(), surface );
	if (it == _surfaces.end()) {
		_surfaces.push_back( surface );
		LTRACE("Group", "Surface[%p] joined group %d", surface, _id);
		surface->group( this );
	}
}

void Group::remove( Surface *surface ) {
	std::vector<Surface *>::iterator it=std::find(
		_surfaces.begin(), _surfaces.end(), surface );
	if (it != _surfaces.end()) {
		LTRACE("Group", "Surface[%p] left group %d", surface, _id);
		(*it)->group(NULL);
		_surfaces.erase(it);
	}
}

}
