/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../../system.h"
#include <vector>

struct _GIOChannel;
typedef struct _GIOChannel GIOChannel;

namespace util {
namespace task {
	class DispatcherImpl;
}
}

namespace canvas {
namespace gtk {

class System : public canvas::System {
public:
	System();
	virtual ~System();

	//	Dispatcher
	virtual util::task::Dispatcher *dispatcher() const;
	
	//	exit system loop
	virtual void exit();

	//	IO events
	virtual util::id::Ident addIO( util::DESCRIPTOR fd, const EventCallback &callback );
	virtual void stopIO( util::id::Ident &id );

	//	Timers
	virtual util::id::Ident registerTimer( util::DWORD ms, const EventCallback &callback );
	virtual void unregisterTimer( util::id::Ident &id );

	//	Socket events
	virtual util::id::Ident addSocket( util::SOCKET sock, const EventCallback &callback );
	virtual void stopSocket( util::id::Ident &id );

	//	Instance creation
	virtual WebViewer *createWebViewer( Surface * );

	//	Aux
	typedef Event<int> Timer;
	typedef std::vector<Timer *> TimerList;
	typedef Event<int> IOEvent;
	typedef std::vector<IOEvent *> IOList;
	void onTaskEvent();
	void onDispatchTimer( Timer *timer );
	void onDispatchIO( IOEvent *io );	
	void onWindowDestroyed( void );

protected:
	//	run system loop
	virtual void runImpl();

	//	Initialization
	virtual bool init();
	virtual void fin();

	//	Instance creation
	virtual canvas::Window *createWindow() const;
	virtual canvas::Canvas *createCanvas() const;

	//	Aux tasks
	void onPost();
	void doExit();

private:
	util::task::DispatcherImpl *_tasks;
	util::id::Pool *_pool;
	int _idleFunc;
	TimerList _timerList;
	IOList _ioList;
	bool _mainLoopRunning;
};

}
}

