/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../../rect.h"
#include "../../mediaplayerimpl.h"

struct libvlc_instance_t;
struct libvlc_media_player_t;
struct libvlc_media_t;

namespace util {
	class Url;
}

namespace canvas {
namespace vlc {

typedef boost::function<void (bool)> OnResizeCallback;
class Player;

class MediaPlayer : public canvas::MediaPlayerImpl {
public:
	MediaPlayer( System *sys, Player *player );
	virtual ~MediaPlayer();

	//	Initialization
	virtual bool initialize( const std::string &url );
	virtual void finalize();

	//	Media operations
	virtual void play();
	virtual void stop();
	virtual void pause( bool needPause );

	//	Volume
	virtual void mute( bool needMute );

	//	Aux
	void renderFrame( void *id );
	void *allocFrame( void **pixels );
	void freeFrame( void *id );

	unsigned getFormat( char *chroma, unsigned *width, unsigned *height, unsigned *pitches, unsigned *lines );
	void cleanup();
	
	void onStopped();

protected:
	//	Aux
	libvlc_instance_t *instance() const;
	canvas::Window *win() const;
	bool getParam( util::Url *url, const std::string &param1, const std::string &param2, std::string &result );
	void onWindowResize( bool start );

	//	No overlay methods
	virtual void onMove( const Point &point );
	virtual void onResize( const Size &size );

	//	Aux volume
	virtual void setVolume( Volume vol );
	virtual Volume maxVol();
	virtual Volume minVol();

private:
	Player *_player;	
	libvlc_media_player_t *_mp;
	OnResizeCallback _originalResizeCallback;
	bool _needsMute;
	Volume _volume;
};

}
}

