/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "mediaplayer.h"
#include "point.h"
#include "types.h"
#include <boost/math/special_functions/round.hpp>

namespace canvas {

MediaPlayer::MediaPlayer()
{
}

MediaPlayer::~MediaPlayer()
{
}

void MediaPlayer::onStop( const OnStopCallback &callback ) {
	_onStop = callback;
}

void MediaPlayer::onMediaStopped() {
	if (!_onStop.empty()) {
		_onStop();
	}
}

void MediaPlayer::bounds( const Rect &rect ) {
	{
		canvas::Size size(rect);
		resize(size);
	}

	{
		canvas::Point point(rect);
		move(point);
	}
}

void MediaPlayer::volume( Volume vol ) {
	Volume normalizeVol=vol;
	if (normalizeVol>0) {
		int relation = boost::math::iround((float)(maxVol() - minVol()) / (float)(MAX_VOLUME - MIN_VOLUME));
		normalizeVol = vol * relation + minVol();
	}
	setVolume( normalizeVol );
}

Volume MediaPlayer::maxVol() {
	return MAX_VOLUME;
}

Volume MediaPlayer::minVol() {
	return MIN_VOLUME;
}

}

