/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "videooverlay.h"
#include "window.h"
#include "rect.h"
#include "point.h"
#include <util/log.h>

namespace canvas {

VideoOverlay::VideoOverlay( int zIndex )
	: _zIndex(zIndex)
{
}

VideoOverlay::~VideoOverlay()
{
}

//	Initialization
bool VideoOverlay::initialize( const Rect &rect ) {
	LDEBUG( "VideoOverlay", "Initialize: rect=(%d,%d,%d,%d)", rect.x, rect.y, rect.w, rect.h );

	//	Setup
	_rect = rect;

	if(!win()) {
		LWARN( "VideoOverlay", "Cannot create video overlay without a window" );
		return false;
	}

	//	Create component
	if (!create()) {
		LWARN( "VideoOverlay", "Cannot create video overlay component" );
		return false;
	}

	//	Setup
	if (!setup()) {
		return false;
	}

	show();

	win()->restackOverlays();
	
	return true;
}

void VideoOverlay::finalize() {
	LDEBUG( "VideoOverlay", "Finalize" );
	hide();
	destroy();
}

bool VideoOverlay::setup() {
	LDEBUG( "VideoOverlay", "Setup overlay" );

	if (!move()) {
		LWARN( "VideoOverlay", "Cannot move overlay" );		
		return false;
	}

	if (!resize()) {
		LWARN( "VideoOverlay", "Cannot resize overlay" );
		return false;
	}

	return true;
}

bool VideoOverlay::move( const Point &point ) {
	LDEBUG( "VideoOverlay", "Move: point=(%d,%d)", point.x, point.y );
	_rect=point;
	return move();
}

bool VideoOverlay::move() {
	Point point = win()->translateToWindowCoordenate( Point(_rect) );
	return moveImpl(point);
}

bool VideoOverlay::resize( const Size &size ) {
	LDEBUG( "VideoOverlay", "Resize: size=(%d,%d)", size.w, size.h );
	_rect=size;
	return resize();
}

bool VideoOverlay::resize() {
	Size size = win()->translateToWindowSize( Size(_rect) );
	return resizeImpl(size);
}

void VideoOverlay::iconify( bool enable ) {
	if (enable) {
		hide();
	} else {
		show();
	}
}

void VideoOverlay::updateBounds() {
	move();
	resize();
}

const Rect &VideoOverlay::bounds() const {
	return _rect;
}

int VideoOverlay::zIndex() const {
	return _zIndex;
}

bool VideoOverlay::getDescription( VideoDescription &desc ) {
#ifdef _WIN32
	desc = NULL;
#else
	desc.display = NULL;
	desc.screenID = -1;
	desc.winID = -1;
#endif
	return true;
}

}
