/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <gtest/gtest.h>
#include "../../../src/font.h"
#include <algorithm>

TEST( font, default_constructor ) {
    canvas::Font f;
    std::vector<std::string> families = f.families();
    ASSERT_TRUE( std::find(families.begin(), families.end(), DEFAULT_FONT) != families.end() );
    ASSERT_TRUE( f.size() == 10);
    ASSERT_TRUE( f.bold() == false );
    ASSERT_TRUE( f.italic() == false);
    ASSERT_TRUE( f.smallCaps() == false);
  
}

TEST( font, constructor_from_string ) {
    canvas::Font f("DejaVuSans", 20);
    ASSERT_TRUE( f.size() == 20);
    ASSERT_TRUE( f.bold() == false );
    ASSERT_TRUE( f.italic() == false);
    ASSERT_TRUE( f.smallCaps() == false);
    ASSERT_TRUE( f.familiesAsString() == "DejaVuSans,Tiresias");
}

TEST( font, constructor_family_size ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Tiresias");
    canvas::Font f(familiesSet, 20);

    std::vector<std::string> familiesGet = f.families();
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Tiresias") != familiesGet.end() );
    ASSERT_TRUE( f.size() == 20);
    ASSERT_TRUE( f.bold() == false );
    ASSERT_TRUE( f.italic() == false);
    ASSERT_TRUE( f.smallCaps() == false);
  
}

TEST( font, set_bold_italic_small ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    canvas::Font f(familiesSet, 20);
    f.bold(true);
    f.italic(true);
    f.smallCaps(true);

    std::vector<std::string> familiesGet = f.families();
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Ubuntu-R") != familiesGet.end() );
    ASSERT_TRUE( f.size() == 20);
    ASSERT_TRUE( f.bold() == true );
    ASSERT_TRUE( f.italic() == true);
    ASSERT_TRUE( f.smallCaps() == true);
  
}

TEST( font, change_size ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    canvas::Font f(familiesSet, 20);
    f.size(25);

    std::vector<std::string> familiesGet = f.families();
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Ubuntu-R") != familiesGet.end() );
    ASSERT_TRUE( f.size() == 25);
  
}

TEST( font, change_family ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    canvas::Font f(familiesSet, 20);

    std::vector<std::string> newFamilies;
    newFamilies.push_back("Tiresias");
    f.families(newFamilies);
    
    std::vector<std::string> familiesGet = f.families();
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Tiresias") != familiesGet.end() );
    ASSERT_TRUE( f.size() == 20);

}

TEST( font, multiples_families ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    familiesSet.push_back("DejaVuSans");
    familiesSet.push_back("Tiresias");
    canvas::Font f(familiesSet, 20);

    std::vector<std::string> familiesGet = f.families();
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Ubuntu-R") != familiesGet.end() );
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "DejaVuSans") != familiesGet.end() );
    ASSERT_TRUE( std::find(familiesGet.begin(), familiesGet.end(), "Tiresias") != familiesGet.end() );
    ASSERT_TRUE( f.size() == 20);
}

TEST( font, familiesAsString ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    familiesSet.push_back("DejaVuSans");
    familiesSet.push_back("Tiresias");
    canvas::Font f(familiesSet, 20);

    ASSERT_TRUE( f.familiesAsString() == "Ubuntu-R,DejaVuSans,Tiresias");
}

TEST( font, familiesAsStringWithoutDefault ) {
    std::vector<std::string> familiesSet;
    familiesSet.push_back("Ubuntu-R");
    familiesSet.push_back("DejaVuSans");
    canvas::Font f(familiesSet, 20);

    ASSERT_TRUE( f.familiesAsString() == "Ubuntu-R,DejaVuSans,Tiresias");
}
