/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "surface.h"
#include "../util.h"
#include "../../../src/canvas.h"
#include "../../../src/surface.h"
#include "../../../src/group.h"

class SurfaceGroup: public Surface {
protected:
    std::string getSubDirectory() const { return "Group"; }
};

TEST_F( SurfaceGroup, one_group ) {
	canvas::Surface *s1 = getCanvas()->createSurfaceFromPath( util::getImageName( "blue.jpg" ) );
	canvas::Surface *s2 = getCanvas()->createSurfaceFromPath( util::getImageName( "red.jpg" ) );
	canvas::Surface *s3 = getCanvas()->createSurfaceFromPath( util::getImageName( "yellow.jpg" ) );

	ASSERT_TRUE( s1 != NULL );
	ASSERT_TRUE( s2 != NULL );
	ASSERT_TRUE( s3 != NULL );

	s1->autoFlush(true);
	s2->autoFlush(true);
	s3->autoFlush(true);

	s2->setLocation( canvas::Point(70,35) );
	s3->setLocation( canvas::Point(35,70) );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "one_group_before_grouping" ) ) );
	
	canvas::Group *group = getCanvas()->createGroup();
	group->add( s1 );
	group->add( s2 );
	group->add( s3 );

	s1->setZIndex(3);
	s2->setZIndex(1);
	s3->setZIndex(2);

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "one_group_after_grouping" ) ) );

	group->remove( s1 );
	group->remove( s2 );
	group->remove( s3 );

	getCanvas()->destroy( s1 );
	getCanvas()->destroy( s2 );
	getCanvas()->destroy( s3 );

	getCanvas()->destroy( group );
}

TEST_F( SurfaceGroup, surface_and_one_group ) {
	canvas::Surface *s1 = getCanvas()->createSurfaceFromPath( util::getImageName( "blue.jpg" ) );
	canvas::Surface *s2 = getCanvas()->createSurfaceFromPath( util::getImageName( "red.jpg" ) );
	canvas::Surface *s = getCanvas()->createSurfaceFromPath( util::getImageName( "green.jpg" ) );
	canvas::Surface *s3 = getCanvas()->createSurfaceFromPath( util::getImageName( "yellow.jpg" ) );

	ASSERT_TRUE( s != NULL );
	ASSERT_TRUE( s1 != NULL );
	ASSERT_TRUE( s2 != NULL );
	ASSERT_TRUE( s3 != NULL );

	s->autoFlush(true);
	s1->autoFlush(true);
	s2->autoFlush(true);
	s3->autoFlush(true);

	s->setLocation( canvas::Point(100,100) );
	s->resize( canvas::Size(30,100), true );
	s1->setLocation( canvas::Point(115,115) );
	s1->resize( canvas::Size(30,100), true );
	s2->setLocation( canvas::Point(65,135) );
	s2->resize( canvas::Size(100,30), true );
	s3->setLocation( canvas::Point(80,150) );
	s3->resize( canvas::Size(100,30), true );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_one_group_before_grouping" ) ) );
	
	canvas::Group *group = getCanvas()->createGroup();
	group->add( s1 );
	group->add( s2 );
	group->add( s3 );

	s1->setZIndex(3);
	s2->setZIndex(1);
	s3->setZIndex(2);

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_one_group_after_grouping" ) ) );

	group->remove( s1 );
	group->remove( s2 );
	group->remove( s3 );

	getCanvas()->destroy( s );
	getCanvas()->destroy( s1 );
	getCanvas()->destroy( s2 );
	getCanvas()->destroy( s3 );

	getCanvas()->destroy( group );
}

TEST_F( SurfaceGroup, surface_and_one_group_change_surface_zindex ) {
	canvas::Surface *s1 = getCanvas()->createSurfaceFromPath( util::getImageName( "blue.jpg" ) );
	canvas::Surface *s2 = getCanvas()->createSurfaceFromPath( util::getImageName( "red.jpg" ) );
	canvas::Surface *s = getCanvas()->createSurfaceFromPath( util::getImageName( "green.jpg" ) );
	canvas::Surface *s3 = getCanvas()->createSurfaceFromPath( util::getImageName( "yellow.jpg" ) );

	ASSERT_TRUE( s != NULL );
	ASSERT_TRUE( s1 != NULL );
	ASSERT_TRUE( s2 != NULL );
	ASSERT_TRUE( s3 != NULL );

	s->autoFlush(true);
	s1->autoFlush(true);
	s2->autoFlush(true);
	s3->autoFlush(true);

	s->setLocation( canvas::Point(100,100) );
	s->resize( canvas::Size(30,100), true );
	s1->setLocation( canvas::Point(115,115) );
	s1->resize( canvas::Size(30,100), true );
	s2->setLocation( canvas::Point(65,135) );
	s2->resize( canvas::Size(100,30), true );
	s3->setLocation( canvas::Point(80,150) );
	s3->resize( canvas::Size(100,30), true );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_one_group_before_grouping" ) ) );
	
	canvas::Group *group = getCanvas()->createGroup();
	group->add( s1 );
	group->add( s2 );
	group->add( s3 );

	s1->setZIndex(3);
	s2->setZIndex(1);
	s3->setZIndex(2);

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_one_group_after_grouping" ) ) );

	s->setZIndex(2);

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_one_group_after_changing_zindex" ) ) );

	group->remove( s1 );
	group->remove( s2 );
	group->remove( s3 );

	getCanvas()->destroy( s );
	getCanvas()->destroy( s1 );
	getCanvas()->destroy( s2 );
	getCanvas()->destroy( s3 );

	getCanvas()->destroy( group );
}

TEST_F( SurfaceGroup, surface_and_two_groups ) {
	canvas::Surface* s1 = getCanvas()->createSurfaceFromPath( util::getImageName( "blue.jpg" ) );
	canvas::Surface* s2 = getCanvas()->createSurfaceFromPath( util::getImageName( "red.jpg" ) );
	canvas::Surface* s3 = getCanvas()->createSurfaceFromPath( util::getImageName( "yellow.jpg" ) );
	canvas::Surface* s = getCanvas()->createSurfaceFromPath( util::getImageName( "image.png" ) );
	canvas::Surface* s4 = getCanvas()->createSurfaceFromPath( util::getImageName( "green.jpg" ) );

	ASSERT_TRUE( s != NULL );
	ASSERT_TRUE( s1 != NULL );
	ASSERT_TRUE( s2 != NULL );
	ASSERT_TRUE( s3 != NULL );
	ASSERT_TRUE( s4 != NULL );

	s->autoFlush(true);
	s1->autoFlush(true);
	s2->autoFlush(true);
	s3->autoFlush(true);
	s4->autoFlush(true);

	s->setOpacity(128);
	s1->setLocation( canvas::Point(100,100) );
	s1->resize( canvas::Size(30,100), true );
	s2->setLocation( canvas::Point(115,115) );
	s2->resize( canvas::Size(30,100), true );
	s3->setLocation( canvas::Point(65,135) );
	s3->resize( canvas::Size(100,30), true );
	s4->setLocation( canvas::Point(80,150) );
	s4->resize( canvas::Size(100,30), true );
	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_two_groups_before_grouping" ) ) );
	
	canvas::Group *group1 = getCanvas()->createGroup();
	group1->add( s1 );
	group1->add( s3 );

	canvas::Group *group2 = getCanvas()->createGroup();
	group2->add( s2 );
	group2->add( s4 );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_two_groups_after_grouping" ) ) );

	group1->remove( s1 );
	group1->remove( s3 );
	group2->remove( s2 );
	group2->remove( s4 );
	
	getCanvas()->destroy( s );
	getCanvas()->destroy( s1 );
	getCanvas()->destroy( s2 );
	getCanvas()->destroy( s3 );
	getCanvas()->destroy( s4 );

	getCanvas()->destroy( group1 );
	getCanvas()->destroy( group2 );
}

TEST_F( SurfaceGroup, surface_and_two_groups_change_zindex ) {
	canvas::Surface *s1 = getCanvas()->createSurfaceFromPath( util::getImageName( "blue.jpg" ) );
	canvas::Surface *s2 = getCanvas()->createSurfaceFromPath( util::getImageName( "red.jpg" ) );
	canvas::Surface *s3 = getCanvas()->createSurfaceFromPath( util::getImageName( "yellow.jpg" ) );
	canvas::Surface *s = getCanvas()->createSurfaceFromPath( util::getImageName( "image.png" ) );
	canvas::Surface *s4 = getCanvas()->createSurfaceFromPath( util::getImageName( "green.jpg" ) );

	ASSERT_TRUE( s != NULL );
	ASSERT_TRUE( s1 != NULL );
	ASSERT_TRUE( s2 != NULL );
	ASSERT_TRUE( s3 != NULL );
	ASSERT_TRUE( s4 != NULL );

	s->autoFlush(true);
	s1->autoFlush(true);
	s2->autoFlush(true);
	s3->autoFlush(true);
	s4->autoFlush(true);

	s->setOpacity(128);
	s1->setLocation( canvas::Point(100,100) );
	s1->resize( canvas::Size(30,100), true );
	s2->setLocation( canvas::Point(115,115) );
	s2->resize( canvas::Size(30,100), true );
	s3->setLocation( canvas::Point(65,135) );
	s3->resize( canvas::Size(100,30), true );
	s4->setLocation( canvas::Point(80,150) );
	s4->resize( canvas::Size(100,30), true );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_two_groups_before_grouping" ) ) );
	
	canvas::Group *group1 = getCanvas()->createGroup();
	group1->add( s1 );
	group1->add( s3 );

	canvas::Group *group2 = getCanvas()->createGroup();
	group2->add( s2 );
	group2->add( s4 );

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_two_groups_after_grouping" ) ) );

	s->setZIndex(2);
	group1->zIndex(3);
	s1->setZIndex(2);
	s2->setZIndex(2);

	ASSERT_TRUE( util::compareImages( getCanvas(), getExpectedPath( "surface_and_two_groups_after_grouping_and_changing_zindex" ) ) );

	group1->remove( s1 );
	group1->remove( s3 );
	group2->remove( s2 );
	group2->remove( s4 );

	getCanvas()->destroy( s );
	getCanvas()->destroy( s1 );
	getCanvas()->destroy( s2 );
	getCanvas()->destroy( s3 );
	getCanvas()->destroy( s4 );

	getCanvas()->destroy( group1 );
	getCanvas()->destroy( group2 );
}
