/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-canvas implementation.

    DTV-canvas is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-canvas is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-canvas.

    DTV-canvas es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-canvas se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "window.h"
#include "../../src/window.h"
#include "../../src/system.h"
#include "../../src/dummy/window.h"
#include <util/mcr.h>
#include <util/cfg/cfg.h>

void Window::SetUp() {
	System::SetUp();
	win()->fullScreen(false);
	util::cfg::setValue<bool>("gui.window.fullscreen", false);
}

void Window::TearDown() {
	System::TearDown();
}

canvas::Window *Window::win() {
	return System::getSystem()->window();
}

TEST_F( Window, constructor ) {
	ASSERT_FALSE( win()->isFullScreen() );
	ASSERT_FALSE( win()->isIconified() );

	canvas::Size size;
	size.h = util::cfg::getValue<int>("gui.window.size.height");
	size.w = util::cfg::getValue<int>("gui.window.size.width");
	ASSERT_TRUE( win()->size() == size );
}

TEST_F( Window, constructor_in_fullScreen ) {
	util::cfg::setValue<bool>("gui.window.fullscreen", true);
	canvas::dummy::Window *win = new canvas::dummy::Window();
	ASSERT_TRUE( win->initialize() );

	ASSERT_TRUE( win->isFullScreen() );
	ASSERT_FALSE( win->isIconified() );

	win->finalize();
	DEL( win );
}

TEST_F( Window, set_fullScreen_dinamicaly ) {
	ASSERT_FALSE( win()->isFullScreen() );
	ASSERT_FALSE( win()->isIconified() );

	canvas::Size size;
	size.h = util::cfg::getValue<int>("gui.window.size.height");
	size.w = util::cfg::getValue<int>("gui.window.size.width");
	ASSERT_TRUE( win()->size() == size );

	if (win()->supportFullScreen()) {
		win()->fullScreen( true );
		ASSERT_TRUE( win()->isFullScreen() );
	}
}

TEST_F( Window, set_unset_fullScreen_dinamicaly ) {
	ASSERT_FALSE( win()->isFullScreen() );
	ASSERT_FALSE( win()->isIconified() );

	canvas::Size size;
	size.h = util::cfg::getValue<int>("gui.window.size.height");
	size.w = util::cfg::getValue<int>("gui.window.size.width");
	ASSERT_TRUE( win()->size() == size );

	if (win()->supportFullScreen()) {
		win()->fullScreen( true );
		ASSERT_TRUE( win()->isFullScreen() );

		win()->fullScreen( false );
		ASSERT_FALSE( win()->isFullScreen() );
	}
}
