/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "channel.h"
#include "../lua.h"
#include <zapper/plugin/pluginmanager.h>
#include <zapper/channelplayer/channelservice.h>
#include <zapper/channelplayer/channelplayer.h>
#include <zapper/display/displayservice.h>

namespace luaz {
namespace channel {

namespace impl {
static zapper::channel::ChannelPlayer *player = NULL;
}

static void onChannelBlocked( lua_State *L, bool blocked ) {
	lua_getfield( L, LUA_GLOBALSINDEX, "zapperCurrentShowBlocked" );
	lua_pushboolean( L, blocked );
	lua_call( L, 1, 0 );
}

static int l_blocked( lua_State *L ) {
	bool blocked = impl::player->isBlocked();
	lua_pushboolean( L, blocked );
	return 1;
}

static int l_current( lua_State *L ) {
	int ch=impl::player->current();
	lua_pushnumber( L, ch );
	return 1;
}

static int l_change( lua_State *L ) {
	impl::player->change( luaL_checkint(L, 1) );
	return 0;
}

static int l_setFullScreen( lua_State * /*L*/ ) {
	impl::player->setFullScreen();
	return 0;
}

static int l_resize( lua_State *L ) {
	impl::player->resize(
		luaL_checkint(L, 1),
		luaL_checkint(L, 2),
		luaL_checkint(L, 3),
		luaL_checkint(L, 4) );
	return 0;
}

static int l_mute( lua_State *L ) {
	int toggle = luaL_checkint(L, 1);
	impl::player->mute( toggle ? true : false );
	return 0;
}

static int l_volume( lua_State *L ) {
	int vol = luaL_checkint(L, 1);
	impl::player->volume( vol );
	return 0;
}

static int l_nextVideo( lua_State *L ) {
	int cur=impl::player->nextVideo();
	lua_pushnumber( L, cur );	
	return 1;
}

static int l_nextAudio( lua_State *L ) {
	int cur=impl::player->nextAudio();
	lua_pushnumber( L, cur );	
	return 1;
}

static int l_nextSubtitle( lua_State *L ) {
	int cur=impl::player->nextSubtitle();
	lua_pushnumber( L, cur );	
	return 1;
}

static int l_haveSubtitle( lua_State *L ) {
	int cur=impl::player->subtitleCount();
	if (cur > 0) {
		lua_pushstring( L, "CC" );
	}
	else {
		lua_pushstring( L, "" );
	}
	return 1;
}

static int l_getResolution( lua_State *L ) {
	zapper::display::mode::type id=impl::player->getResolution();
	zapper::display::Mode &mode = zapper::display::getMode( id );
	lua_pushstring( L, mode.name );
	return 1;
}

static int l_getAspect( lua_State *L ) {
	zapper::display::aspect::type id=impl::player->getAspect();
	zapper::display::Aspect &aspect = zapper::display::getAspect( id );
	lua_pushstring( L, aspect.name );
	return 1;
}

static int l_getAudio( lua_State *L ) {
	zapper::audio::channel::type id=impl::player->getAudio();
	zapper::audio::Channel &ch = zapper::audio::getChannelInfo( id );
	lua_pushstring( L, ch.name );
	return 1;
}

static const struct luaL_Reg channel_methods[] = {
	{ "current",        l_current        },
	{ "change",         l_change         },
	{ "setFullScreen",  l_setFullScreen  },
	{ "resize",         l_resize         },
	{ "mute",           l_mute           },
	{ "volume",         l_volume         },
	{ "nextVideo",      l_nextVideo      },
	{ "nextAudio",      l_nextAudio      },
	{ "nextSubtitle",   l_nextSubtitle   },
	{ "haveSubtitle",   l_haveSubtitle   },
	{ "getResolution",  l_getResolution  },
	{ "getAspect",      l_getAspect      },
	{ "getAudio",       l_getAudio       },
	{ "isBlocked",      l_blocked        },
	{ NULL,             NULL             }
};

//	Public methods
void attach( std::vector<std::string> &deps ) {
	deps.push_back( "ChannelService" );
}

void initialize( zapper::plugin::PluginManager *mgr ) {
	zapper::channel::ChannelService *srv = mgr->findService<zapper::channel::ChannelService>( "ChannelService" );
	assert(srv);
	impl::player = srv->master();
}

void finalize( void ) {
}

void start( lua_State *st ) {
	impl::player->onChannelBlocked().connect( boost::bind(&onChannelBlocked,st,_1) );
	luaL_register( st, "channel", channel_methods );
}

void stop( void ) {
}

}
}

