/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "control.h"
#include "../lua.h"
#include <zapper/plugin/pluginmanager.h>
#include <zapper/control/service.h>

namespace luaz {
namespace control {

namespace impl {
	static zapper::control::Service *control = NULL;
}

static int l_setPIN( lua_State *L ) {
	const char *pin = luaL_checkstring(L, 1);
	if (pin) {
		impl::control->setPIN( pin );
	}
	return 0;
}

static int l_resetPIN( lua_State * /*L*/ ) {
	impl::control->setPIN( "" );
	return 0;
}

static int l_checkPIN( lua_State *L ) {
	const char *pin = luaL_checkstring(L, 1);
	lua_pushboolean( L, impl::control->checkPIN( pin ) );
	return 1;
}

static int l_isSet( lua_State *L ) {
	bool isSet=impl::control->enabled();
	lua_pushnumber( L, isSet ? 1 : 0 );
	return 1;
}

static int l_isSessionOpened( lua_State *L ) {
	bool isOpened=impl::control->sessionIsOpen();
	lua_pushnumber( L, isOpened ? 1 : 0 );
	return 1;
}

static int l_getParentalAge( lua_State *L ) {
	lua_pushnumber( L, impl::control->parentalAge() );
	return 1;
}

static int l_setParentalAge( lua_State *L ) {
	impl::control->setParentalAge( (zapper::parental::age::type)luaL_checkint(L, 1) );
	return 0;
}

static int l_getParentalContent( lua_State *L ) {
	lua_pushnumber( L, impl::control->parentalContent() );	
	return 1;
}

static int l_setParentalContent( lua_State *L ) {
	impl::control->setParentalContent( (util::BYTE) luaL_checkint(L, 1) );
	return 0;
}

static const struct luaL_Reg control_methods[] = {
	{ "setPIN",             l_setPIN             },
	{ "resetPIN",           l_resetPIN           },
	{ "checkPIN",           l_checkPIN           },
	{ "isSet",              l_isSet              },
	{ "isSessionOpened",    l_isSessionOpened    },
	{ "setParentalAge",     l_setParentalAge     },
	{ "getParentalAge",     l_getParentalAge     },
	{ "getParentalContent", l_getParentalContent },
	{ "setParentalContent", l_setParentalContent },
	{ NULL,                 NULL                 }
};

//	Public methods
void attach( std::vector<std::string> &deps ) {
	deps.push_back( "Control" );
}

void initialize( zapper::plugin::PluginManager *mgr ) {
	impl::control = mgr->addRef<zapper::control::Service>( "Control" );
}

void finalize( void ) {
	RELEASE_SERVICE(impl::control);
}

void start( lua_State *st ) {
	luaL_register( st, "control", control_methods );
}

void stop( void ) {
}

}
}

