/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include <zapper/input/input.h>
#include <zapper/plugin/service.h>
#include <util/id/ident.h>
#include <util/keydefs.h>
#include <boost/function.hpp>
#include <string>
#include <map>
#include <queue>

typedef struct lua_State lua_State;

namespace util {
	class Task;
}

namespace canvas {
	class System;
	class Timer;
	class Surface;
}

namespace zapper {
	class Zapper;

namespace input {
	class InputService;
}

}	//	namespace zapper


namespace luaz {

class MainWindow : public zapper::plugin::ServiceImpl<MainWindow> {
public:
	MainWindow( zapper::Zapper *zapper, lua_State *st, bool supportWidgets );
	virtual ~MainWindow( void );

	//	Run main lua loop
	void run();
	void stop( int ret );

	//	Keys
	void lockKeys( bool needKeys );

	//	Background
	void showBackground( bool show );

	//	Reset config
	void reset();

	//	Time
	int getTime( const std::string &format );
	int getDate( const std::string &format, int daysOffset );

	//	Timers
	void registerTimer( int tid, int ms );
	void cancelTimer( int tID );

	//	Web browser
	void launchBrowser();

	//	Window functions
	void fullscreen( bool enable );
	bool isFullscreen() const;
	void iconify( bool enable );

protected:
	//	Types
	typedef std::map<int, util::id::Ident>  MapOfTimers;

	//	Services virtual methods
	virtual void onAttach();
	virtual bool onOnline();
	virtual void onOffline();	
	virtual bool onStart();
	virtual void onStop();

	//	Aux
	void registerKeys();
	void onKey( util::key::type key, bool isUp );
	void onButtonEvent( ::canvas::event::ButtonEvent *evt );	
	void registerModules();	
	void processTimer( util::id::Ident &id, int uiTimer );
	void initBgSurface();

	canvas::System *sys() const;

private:
	zapper::Zapper *_zapper;
	lua_State *_lua;
	bool _supportWidgets;
	MapOfTimers _timers;
	zapper::input::InputService *_input;
	canvas::Surface *_bgSurface;
};

}
