/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "module.h"
#include "channels.h"
#include "channel.h"
#include "player.h"
#include "display.h"
#include "middleware.h"
#include "tuner.h"
#include "update.h"
#include "control.h"
#include <zapper/plugin/dependency.h>
#include <zapper/plugin/pluginmanager.h>
#include <boost/foreach.hpp>
#include <boost/signals.hpp>
#include <boost/algorithm/string/predicate.hpp>

namespace luaz {
namespace module {

struct Module {
	const char *name;
	void (*attach)( std::vector<std::string> &deps );
	void (*initialize)( zapper::plugin::PluginManager *mgr );
	void (*finalize)( void );
	void (*start)( lua_State *st );
	void (*stop)( void );
	void (*onConnectionChanged)( bool isRegistered );
};

#define DO_MODULE(name)	\
	{ #name, &name::attach, &name::initialize, &name::finalize, &name::start, &name::stop, NULL }

#define DO_MODULE_P(name) \
	{ #name, &name::attach, &name::initialize, &name::finalize, &name::start, &name::stop, &name::onConnectionChanged }

static const struct Module modules[] = {
	DO_MODULE(tuner),
	DO_MODULE(control),
	DO_MODULE(channels),
	DO_MODULE(channel),
	DO_MODULE(player),
	DO_MODULE(display),
	DO_MODULE_P(middleware),
	DO_MODULE_P(update),
	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL }
};

//	Implementation
namespace impl {
	static boost::signals::connection onRegister;
}

static void onRegisterChanged( zapper::plugin::Service *srv, bool isRegistered ) {
	int i=0;
	while (modules[i].name) {
		if (boost::iequals(srv->name(),modules[i].name) && modules[i].onConnectionChanged) {
			(*modules[i].onConnectionChanged)( isRegistered );
			break;
		}
		i++;		
	}
}

//	Services virtual methods
void attach( std::vector<std::string> &deps ) {
	int i=0;
	while (modules[i].name) {
		if (modules[i].attach) {
			(*modules[i].attach)( deps );
		}
		i++;
	}
}

void initialize( zapper::plugin::PluginManager *mgr ) {
	int i=0;
	impl::onRegister=mgr->onRegisterChanged().connect( boost::bind(&onRegisterChanged,_1,_2) );
	while (modules[i].name) {
		if (modules[i].initialize) {
			(*modules[i].initialize)( mgr );
		}
		i++;
	}
}

void finalize( void ) {
	int i=0;
	impl::onRegister.disconnect();
	while (modules[i].name) {
		if (modules[i].finalize) {
			(*modules[i].finalize)();
		}
		i++;
	}
}

void start( lua_State *st ) {
	int i=0;
	while (modules[i].name) {
		if (modules[i].start) {
			(*modules[i].start)( st );
		}
		i++;
	}
}

void stop( void ) {
	int i=0;
	while (modules[i].name) {
		if (modules[i].stop) {
			(*modules[i].stop)();
		}
		i++;
	}
}

}
}

