/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "tuner.h"
#include "../lua.h"
#include <zapper/plugin/pluginmanager.h>
#include <zapper/tuner/tuner.h>
#include <zapper/tuner/tunerlistener.h>

namespace luaz {
namespace tuner {

namespace impl {
	static zapper::Tuner *tuner = NULL;
	static ::tuner::signal::type signal = ::tuner::signal::invalid;
}

class LuaListener : public zapper::TunerListener {
public:
	LuaListener( lua_State *st ) : _st(st) {}
	virtual ~LuaListener() {}

	virtual void onBeginScan( size_t networks ) {
		lua::call( _st, "zapperBeginScan", networks );
	}

	virtual void onNetwork( const std::string &net ) {
		lua::call( _st, "zapperScanNetwork", net );
	}

	virtual void onEndScan() {
		lua::call( _st, "zapperEndScan" );
	}

	virtual void onStateChanged( ::tuner::state::type /*st*/, ::tuner::signal::type signal ) {
		impl::signal = signal;
	}

private:
	lua_State *_st;
};

static int l_startScan( lua_State * /*L*/ ) {
	impl::tuner->startScan();
	return 0;
}

static int l_stopScan( lua_State * /*L*/ ) {
	impl::tuner->stopScan();
	return 0;
}

static int l_getSignal( lua_State *L ) {
	lua_pushnumber( L,  impl::signal );
	return 1;
}

static const struct luaL_Reg tuner_methods[] = {
	{ "startScan", l_startScan },
	{ "stopScan",  l_stopScan  },
	{ "getSignal", l_getSignal },	
	{ NULL,        NULL        }
};

//	Public methods
void attach( std::vector<std::string> &deps ) {
	deps.push_back( "Tuner0" );
}

void initialize( zapper::plugin::PluginManager *mgr ) {
	impl::tuner = mgr->addRef<zapper::Tuner>( "Tuner0" );
	impl::signal = ::tuner::signal::invalid;
}

void finalize() {
	RELEASE_SERVICE(impl::tuner);
}

void start( lua_State *st ) {
	impl::tuner->addListener( new LuaListener(st) );
	luaL_register( st, "tuner", tuner_methods );
}

void stop() {
}

}
}

