/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "middleware.h"
#include "../lua.h"
#include <zapper/plugin/pluginmanager.h>
#include <zapper/update/updateservice.h>

namespace luaz {
namespace update {

namespace impl {
	static zapper::plugin::PluginManager *mgr = NULL;
	static lua_State *st=NULL;
	static bool connected=false;
}

static zapper::update::UpdateService *getDownloader() {
	return impl::mgr->findService<zapper::update::UpdateService>( "Update" );	
}

static void applyUpdate( int id, bool apply ) {
	zapper::update::UpdateService *srv = getDownloader();
	if (srv) {
		srv->apply( id, apply );
	}
}
	
static void processUpdate( int id, bool compulsory ) {
	if (compulsory) {
		applyUpdate( id, true );
	}
	else {
		lua::call( impl::st, "zapperUpdateReceived", id );
	}
}

static void downloadProgress( int step, int total ) {
	lua::call( impl::st, "zapperUpdateDownloadProgress", step, total );	
}

static void connectNotifications() {
	if (!impl::connected) {
		zapper::update::UpdateService *srv = getDownloader();
		if (srv) {
			//	Get notifications
			srv->onUpdateReceived( boost::bind( &processUpdate, _1, _2 ) );
			srv->onDownloadProgress( boost::bind( &downloadProgress, _1, _2 ) );			
			impl::connected = true;
		}
	}
}

//	LUA API
static int l_isEnabled( lua_State *L ) {
	return lua::isPluginEnabled( L, impl::mgr, "Update" );
}

static int l_enable( lua_State *L ) {
	return lua::enablePlugin( L, impl::mgr, "Update" );
}

static int l_apply( lua_State *L ) {
	applyUpdate( luaL_checkint(L, 1), luaL_checkint(L, 2) == 1 );
	return 0;
}

static const struct luaL_Reg update_methods[] = {
	{ "isEnabled", l_isEnabled },	
	{ "enable",    l_enable    },
	{ "apply",     l_apply     },	
	{ NULL,        NULL        }
};

//	Public methods
void attach( std::vector<std::string> &/*deps*/ ) {
}

void initialize( zapper::plugin::PluginManager *mgr ) {
	impl::mgr = mgr;
	connectNotifications();
}

void finalize( void ) {
	impl::mgr = NULL;
}

void start( lua_State *st ) {
	impl::st = st;	
	luaL_register( st, "update", update_methods );
}

void stop( void ) {
	impl::st = NULL;	
}

void onConnectionChanged( bool isRegistered ) {
	if (isRegistered) {
		connectNotifications();
	}
	else {
		impl::connected=false;
	}	
}

}
}

