/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "ui.h"
#include "lua.h"
#include "modules/mainwindow.h"
#include "generated/config.h"
#include <zapper/plugin/pluginmanager.h>
#include <zapper/zapper.h>
#include <util/log.h>
#include <util/fs.h>
#include <util/cfg/cfg.h>
#include <boost/filesystem.hpp>
#include <assert.h>

namespace fs = boost::filesystem;

namespace luaz {

UI::UI( const std::string &name, bool supportWidgets, const std::string &icon/*=std::string("")*/ )
	: _name(name), _icon(icon), _supportWidgets(supportWidgets)
{
	_zapper = NULL;
	_lState = NULL;
}

UI::~UI()
{
}

int UI::run( const std::string &script ) {
	if (initialize()) {
		//	Start zapper
		_zapper->start();

		loadEnvironmentVars();

		{//	Run lua script
			fs::path tmp( lua::scriptsPath() );
			tmp /= script;
			runScript( tmp.string() );
		}

		//	Stop zapper
		_zapper->stop();

		finalize();
	}

	return _zapper->exitCode();
}

bool UI::initialize() {
	LDEBUG( "UI", "Initialize" );

	//	Setup icon
	if (!_icon.empty()) {
		util::cfg::PropertyTree &widnowProps = util::cfg::get("gui.window");
		if (widnowProps.get<std::string>("icon").empty()) {
			fs::path tmp( util::fs::makeName( util::fs::dataRootDir(), _name, _icon ) );
			widnowProps.set("icon", tmp.string() );
		}
	}

	_lState = lua::init();

	if (!_lState) {
		return false;
	}

	//	Create zapper
	_zapper = zapper::Zapper::create( _name );
	if (!_zapper) {
		LERROR( "UI", "Cannot create zapper object" );
		return false;
	}

	//	Initialize zapper
	if (!_zapper->initialize()) {
		return false;
	}

	//	Add services to zapper
	addServices();
	return true;
}

void UI::finalize() {
	LDEBUG( "UI", "Finalize" );
	//	Finalize zapper!
	_zapper->finalize();
	
	//	Close lua
	lua_close( _lState );
	_lState = NULL;

	delete _zapper;
}

void UI::addServices() {
	//	Add main window
	zapper::plugin::PluginManager *mgr = _zapper->mgr();
	_mainWindow = new MainWindow(_zapper,_lState,_supportWidgets);
	mgr->registerService(_mainWindow);
}

void UI::setEnvironmentVar( const std::string &name, const std::string &value ) {
	//	Setup global lua variable
	_environmentVars.push_back(std::make_pair(name, value));
}

void UI::loadEnvironmentVars() {
	// Common tool vars
	lua::setGlobal(_lState , "zapperVersion", _zapper->getVersion() );
	lua::setGlobal(_lState , "platformVersion", _zapper->getPlatformVersion() );
	lua::setGlobal(_lState , "toolBuildVersion", util::cfg::getValue<std::string>("tool.buildVersion") );

	// Specific tool vars
	BOOST_FOREACH( variable var, _environmentVars ) {
		lua::setGlobal(_lState , var.first, var.second );
	}
}

void UI::runScript( const std::string &script ) {
	//	Start script of lua
	LDEBUG( "UI", "Run script: %s", script.c_str() );
	if (luaL_dofile(_lState, script.c_str() )){
		int i=lua_gettop(_lState);
		printf( "[UI] %s\n",  lua_tostring(_lState, i));
	}
	LDEBUG( "UI", "Close script!" );
}

}

