/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * button.cpp
 *
 *  Created on: Jun 17, 2011
 *      Author: gonzalo
 */

#include "luawgt.h"

#include "wgt.h"
#include "../widget/button.h"
#include "../widget/buttonset.h"
#include "../widget/extendedbuttonset.h"

#include <boost/make_shared.hpp>

#include <iostream>

namespace wgt {

namespace button {

std::map<WidgetId, ButtonPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const ColorId bgColor       = luaL_checkint(L, narg++);
	const ColorId activeBgColor = luaL_checkint(L, narg++);

	ButtonPtr ww = boost::make_shared<Button>(x, y, w, h, Wgt::view()->getColorById(bgColor), Wgt::view()->getColorById(activeBgColor));
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int label(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);

	ButtonPtr button = _widgets[id];
	button->label(text);
	if (lua_gettop(L) == narg) {
		const int fsize = luaL_checkint(L, narg++);
		button->label()->fontSize(fsize);
	}

	return 0;
}

int icon(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* path  = luaL_checkstring(L, narg++);
	const char* aPath = luaL_checkstring(L, narg++);
	const int imgw    = luaL_checkint(L, narg++);
	const int imgh    = luaL_checkint(L, narg++);

	ButtonPtr button = _widgets[id];
	button->icon(path, aPath, imgw, imgh);
	return 0;
}

int textColor(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const ColorId color       = luaL_checkint(L, narg++);
	const ColorId activeColor = luaL_checkint(L, narg++);

	ButtonPtr button = _widgets[id];
	button->textColor(Wgt::view()->getColorById(color), Wgt::view()->getColorById(activeColor));
	return 0;
}

int fontSize(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int size    = luaL_checkint(L, narg++);

	ButtonPtr button = _widgets[id];
	button->label()->fontSize(size);
	return 0;
}

static int enable(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int b    = luaL_checkint(L, narg++);

	ButtonPtr button = _widgets[id];
	button->enable(b!=0);
	button->needsRepaint(true);

	return 0;
}

int action(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* f     = luaL_checkstring(L, narg++);

	ButtonPtr button = _widgets[id];
	button->action(std::string(f));
	return 0;
}

int selectedCallback(lua_State* L) {
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* f     = luaL_checkstring(L, narg++);

	ButtonPtr button = _widgets[id];
	button->selectedCallback(std::string(f));
	return 0;
}

void init(lua_State *L){
	std::cout << "[lua] Button::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",       create},
			{"label",     label},
			{"icon",      icon},
			{"textColor", textColor},
			{"fontSize",  fontSize},
			{"action",    action},
			{"enable",    enable},
			{"selectedCallback", selectedCallback},
			{NULL,  NULL}
	};

	luaL_register(L, "button", methods);
}

} // button

namespace buttonset {

std::map<WidgetId, ButtonSetPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int color = luaL_checkint(L, narg++);

	ButtonSetPtr ww = boost::make_shared<ButtonSet>(x, y, w, h, Wgt::view()->getColorById(color));
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int createExtended(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int color = luaL_checkint(L, narg++);

	ExtendedButtonSetPtr ww = boost::make_shared<ExtendedButtonSet>(x, y, w, h, Wgt::view()->getColorById(color));
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int addButton(lua_State* L)
{
	// Por ahora no usa nada específico de los botones, se podría reemplazar por addChild de Window
	int narg = 2;
	const WidgetId setid = luaL_checkint(L, narg++);
	const WidgetId btnid = luaL_checkint(L, narg++);

	ButtonSetPtr set = _widgets[setid];
	ButtonPtr btn = button::_widgets[btnid] ;//Wgt::view()->getWidgetById(btnid);

	set->addButton(btn);

	return 0;
}

int setActiveButton(lua_State* L)
{
	int narg = 2;
	const WidgetId setid = luaL_checkint(L, narg++);
	const int active  = luaL_checkint(L, narg++);

	ButtonSetPtr set = _widgets[setid];
	if (active >= 0) {
		set->selectChild(active);
	} else {
		set->deactivateChild();
	}
	set->needsRepaint(true);

	return 0;
}

int setCircular(lua_State* L)
{
	int narg = 2;
	const WidgetId setid = luaL_checkint(L, narg++);
	const int circular   = luaL_checkint(L, narg++);

	ButtonSetPtr set = _widgets[setid];
	set->circular(circular != 0);

	return 0;
}

void init(lua_State *L){
	std::cout << "[lua] ButtonSet::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",             create},
			{"newExtended",     createExtended},
			{"addButton",       addButton},
			{"setActiveButton", setActiveButton},
			{"circular",        setCircular},
			{NULL,  NULL}
	};

	luaL_register(L, "buttonset", methods);
}

} // buttonset


} // wgt
