/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * dialogs.cpp
 *
 *  Created on: Jun 17, 2011
 *      Author: gonzalo
 */

#include "luawgt.h"

#include "wgt.h"
#include "../widget/pindialog.h"

#include <boost/make_shared.hpp>

#include <iostream>

namespace wgt {

namespace yesnodialog {

std::map<WidgetId, YesNoDialogPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int zIndex = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const int borderWidth   = luaL_checkint(L, narg++);
	const int bw = luaL_checkint(L, narg++);
	const int bh = luaL_checkint(L, narg++);

	YesNoDialogPtr ww = boost::make_shared<YesNoDialog>(x,y, w,h, bgColor, borderColor, borderWidth, bw, bh);
	ww->zIndex(zIndex);
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int yesAction(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* fname = luaL_checkstring(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->yesAction(std::string(fname));

	return 0;
}

int noAction(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* fname = luaL_checkstring(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->noAction(std::string(fname));

	return 0;
}

int title(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* title = luaL_checkstring(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->title(title);

	return 0;
}

int icon(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* path = luaL_checkstring(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];

	if (lua_gettop(L) == 4) {
		const char* pathDisabled = luaL_checkstring(L, narg++);
		dialog->iconPath(path, w,h, pathDisabled);
	} else {
		dialog->iconPath(path, w,h);
	}

	return 0;
}

int text(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);
	const int maxLines = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->text()->text(text);
	dialog->text()->lines(maxLines);

	return 0;
}

int titleColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->titleColor(Wgt::view()->getColorById(color));

	return 0;
}

int textColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->textColor(Wgt::view()->getColorById(color));

	return 0;
}

int buttonColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color       = luaL_checkint(L, narg++);
	const ColorId activeColor = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->buttonColor(Wgt::view()->getColorById(color), Wgt::view()->getColorById(activeColor));

	return 0;
}

int buttonTextColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color       = luaL_checkint(L, narg++);
	const ColorId activeColor = luaL_checkint(L, narg++);

	YesNoDialogPtr dialog = _widgets[id];
	dialog->buttonTextColor(Wgt::view()->getColorById(color), Wgt::view()->getColorById(activeColor));

	return 0;
}

void init(lua_State *L)
{
	std::cout << "[lua] yesnodialog::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",         create},
			{"yesAction",   yesAction},
			{"noAction",    noAction},

			{"title",    title},
			{"icon",     icon},
			{"text",     text},

			{"titleColor",      titleColor},
			{"textColor",       textColor},
			{"buttonColor",     buttonColor},
			{"buttonTextColor", buttonTextColor},

			{NULL,  NULL}
	};

	luaL_register(L, "yesnodialog", methods);
}
} // yesnodialog

//--------------------------------------------------------------------------//

namespace alertdialog {

std::map<WidgetId, AlertDialogPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int zIndex = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const int borderWidth   = luaL_checkint(L, narg++);

	AlertDialogPtr ww = boost::make_shared<AlertDialog>(x,y, w,h, bgColor, borderColor, borderWidth);
	ww->zIndex(zIndex);
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int callback(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* fname = luaL_checkstring(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->callback(std::string(fname));

	return 0;
}

int title(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* title = luaL_checkstring(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->title(title);

	return 0;
}

int icon(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* path = luaL_checkstring(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->iconPath(path, w,h);

	return 0;
}

int text(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);
	const int maxLines = luaL_checkint(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->text()->text(text);
	dialog->text()->lines(maxLines);

	return 0;
}

int titleColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->titleColor(Wgt::view()->getColorById(color));

	return 0;
}

int textColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	AlertDialogPtr dialog = _widgets[id];
	dialog->textColor(Wgt::view()->getColorById(color));

	return 0;
}

void init(lua_State *L)
{
	std::cout << "[lua] alertdialog::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",         create},
			{"callback",    callback},

			{"title",    title},
			{"icon",     icon},
			{"text",     text},

			{"titleColor",      titleColor},
			{"textColor",       textColor},

			{NULL,  NULL}
	};

	luaL_register(L, "alertdialog", methods);
}
} // alertdialog


//--------------------------------------------------------------------------//

namespace basicdialog {

std::map<WidgetId, DialogPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int zIndex = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const int borderWidth   = luaL_checkint(L, narg++);

	DialogPtr ww = boost::make_shared<Dialog>(x,y, w,h, bgColor, borderColor, borderWidth);
	ww->zIndex(zIndex);
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int title(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* title = luaL_checkstring(L, narg++);

	DialogPtr dialog = _widgets[id];
	dialog->title(title);

	return 0;
}

int icon(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* path = luaL_checkstring(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);

	DialogPtr dialog = _widgets[id];
	dialog->iconPath(path, w,h);

	return 0;
}

int text(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);
	const int maxLines = luaL_checkint(L, narg++);

	DialogPtr dialog = _widgets[id];
	dialog->text()->text(text);
	dialog->text()->lines(maxLines);

	return 0;
}

int addChild(lua_State* L)
{
	int narg = 2;
	const WidgetId fid = luaL_checkint(L, narg++);
	const WidgetId sid = luaL_checkint(L, narg++);

	DialogPtr father = _widgets[fid];
	WidgetPtr son  = Wgt::view()->getWidgetById(sid);

	father->addChild(son);

	return 0;
}

int titleColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	DialogPtr dialog = _widgets[id];
	dialog->titleColor(Wgt::view()->getColorById(color));

	return 0;
}

int textColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	DialogPtr dialog = _widgets[id];
	dialog->textColor(Wgt::view()->getColorById(color));

	return 0;
}

void init(lua_State *L)
{
	std::cout << "[lua] basicdialog::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",         create},
			{"addChild",    addChild},

			{"title",    title},
			{"icon",     icon},
			{"text",     text},

			{"titleColor",      titleColor},
			{"textColor",       textColor},

			{NULL,  NULL}
	};

	luaL_register(L, "basicdialog", methods);
}
} // basicdialog

//--------------------------------------------------------------------------//

namespace pindialog {

std::map<WidgetId, PINDialogPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int zIndex = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const int borderWidth   = luaL_checkint(L, narg++);

	PINDialogPtr ww = boost::make_shared<PINDialog>(x,y, w,h, bgColor, borderColor, borderWidth);
	ww->zIndex(zIndex);
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int open(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int pinSize = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->open( pinSize );

	return 0;
}

int prompt(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* prompt = luaL_checkstring(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->prompt(prompt);

	return 0;
}

int pinCorrect(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->pinCorrect();

	return 0;
}

int pinIncorrect(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->pinIncorrect();

	return 0;
}

int pinClose(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->pinClose();

	return 0;
}


int callbacks(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* fOk        = luaL_checkstring(L, narg++);
	const char* fCancelled = luaL_checkstring(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->callbacks(std::string(fOk), std::string(fCancelled));

	return 0;
}

int title(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* title = luaL_checkstring(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->title(title);

	return 0;
}

int icon(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* path = luaL_checkstring(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->iconPath(path, w,h);

	return 0;
}

int text(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);
	const int maxLines = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->text()->text(text);
	dialog->text()->lines(maxLines);

	return 0;
}

int help(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* help = luaL_checkstring(L, narg++);

	PINDialogPtr widget = _widgets[id];
	widget->help(help);

	return 0;
}

int lines(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int lns = luaL_checkint(L, narg++);

	PINDialogPtr widget = _widgets[id];
	widget->text()->lines(lns);

	return 0;
}

int titleColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->titleColor(Wgt::view()->getColorById(color));

	return 0;
}

int textColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id   = luaL_checkint(L, narg++);
	const ColorId color = luaL_checkint(L, narg++);

	PINDialogPtr dialog = _widgets[id];
	dialog->textColor(Wgt::view()->getColorById(color));

	return 0;
}

void init(lua_State *L)
{
	std::cout << "[lua] pindialog::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",      create},
			{"open",     open},
			{"prompt",   prompt},
			{"callbacks", callbacks},
			{"pinCorrect", pinCorrect},
			{"pinIncorrect", pinIncorrect},
			{"pinClose", pinClose},

			{"title",    title},
			{"icon",     icon},
			{"text",     text},
			{"help",     help},
			{"lines",    lines},

			{"titleColor",      titleColor},
			{"textColor",       textColor},

			{NULL,  NULL}
	};

	luaL_register(L, "pindialog", methods);
}
} // pindialog

} // wgt

