/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * menupage.cpp
 *
 *  Created on: Jun 17, 2011
 *      Author: gonzalo
 */

#include "luawgt.h"

#include "wgt.h"
#include "../widget/tabbedwindow.h"
#include "../widget/page.h"

#include <boost/make_shared.hpp>

#include <iostream>

namespace wgt {

namespace menupage {

std::map<WidgetId, PagePtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;

	const int px = luaL_checkint(L, narg++);
	const int py = luaL_checkint(L, narg++);
	const int pw = luaL_checkint(L, narg++);
	const int ph = luaL_checkint(L, narg++);

	const int ty = luaL_checkint(L, narg++);
	int withHelp = 1;
	if (lua_gettop(L) == 7) {
		withHelp = luaL_checkint(L, narg++);
	}
	PagePtr widget = boost::make_shared<Page>(px,py,pw,ph, ty, withHelp==1);
	WidgetId id = Wgt::view()->addWidget(widget);
	_widgets[id] = widget;

	lua_pushnumber(L, id);
	return 1;
}

int addChild(lua_State* L)
{
	int narg = 2;
	const WidgetId fid = luaL_checkint(L, narg++);
	const WidgetId sid = luaL_checkint(L, narg++);

	PagePtr father = _widgets[fid];
	WidgetPtr son  = Wgt::view()->getWidgetById(sid);

	father->addChild(son);

	return 0;
}

int setIcons(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* activePath   = luaL_checkstring(L, narg++);
	const char* disabledPath = luaL_checkstring(L, narg++);
	const char* inactivePath = luaL_checkstring(L, narg++);

	PagePtr widget = _widgets[id];
	widget->iconActive(activePath);
	widget->iconInactive(inactivePath);
	widget->iconDisabled(disabledPath);

	return 0;
}

int setLabel(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* text = luaL_checkstring(L, narg++);

	PagePtr widget = _widgets[id];
	widget->name(text);

	return 0;
}

int setBorderColors(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color colorActive   = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color colorInactive = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr widget = _widgets[id];
	widget->borderColorActive(colorActive);
	widget->borderColorInactive(colorInactive);

	return 0;
}

int setBackgroundColor(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color color = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr widget = _widgets[id];
	widget->page()->bgColor(color);

	return 0;
}

int setSelected(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int b  = luaL_checkint(L, narg++);

	PagePtr widget = _widgets[id];
	widget->selected(b!=0);
	widget->needsRepaint(true);

	return 0;
}

int setActive(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int b  = luaL_checkint(L, narg++);

	PagePtr widget = _widgets[id];
	widget->activated(b!=0);
	widget->needsRepaint(true);

	return 0;
}

int setHelp(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const char* help = luaL_checkstring(L, narg++);

	PagePtr widget = _widgets[id];
	widget->setHelp(help);
	widget->helpBox()->needsRepaint(true);

	return 0;
}

int setDefaultMargin(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::margin(value);
	return 0;
}
int setDefaultBorder(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::border(value);
	return 0;
}
int setDefaultTabSpace(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::tabSpace(value);
	return 0;
}
int setDefaultTabWidth(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::tabW(value);
	return 0;
}
int setDefaultTabHeight(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::tabH(value);
	return 0;
}
int setDefaultHelpHeight(lua_State *L)
{
	int narg = 2;
	const WidgetId value = luaL_checkint(L, narg++);
	Page::helpHeight(value);
	return 0;
}

int isSelected(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	PagePtr ww = _widgets[id];

	lua_pushnumber(L, ww->isSelected());
	return 1;
}

int isActive(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	PagePtr ww = _widgets[id];

	lua_pushnumber(L, ww->isActive());
	return 1;
}

int tabColorsActive(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color textColor   = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr ww = _widgets[id];
	ww->tabColorsActive(bgColor, borderColor, textColor);

	return 0;
}

int tabColorsSelected(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color textColor   = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr ww = _widgets[id];
	ww->tabColorsSelected(bgColor, borderColor, textColor);

	return 0;
}

int tabColorsUnselected(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color textColor   = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr ww = _widgets[id];
	ww->tabColorsUnselected(bgColor, borderColor, textColor);

	return 0;
}

int helpBoxColors(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const Color bgColor     = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color borderColor = Wgt::view()->getColorById(luaL_checkint(L, narg++));
	const Color textColor   = Wgt::view()->getColorById(luaL_checkint(L, narg++));

	PagePtr ww = _widgets[id];
	ww->helpBoxColors(bgColor, borderColor, textColor);

	return 0;
}

void init(lua_State *L){
	std::cout << "[lua] LuaMenuPage::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",      create},
			{"addChild", addChild},
			{"setIcons", setIcons},
			{"setLabel", setLabel},
			{"setSelected", setSelected},
			{"setActive",   setActive},
			{"setHelp",   setHelp},
			{"setBackgroundColor", setBackgroundColor},
			{"setBorderColors",    setBorderColors},

			{"setDefaultMargin", setDefaultMargin},
			{"setDefaultBorder", setDefaultBorder},
			{"setDefaultTabSpace", setDefaultTabSpace},
			{"setDefaultTabWidth", setDefaultTabWidth},
			{"setDefaultTabHeight", setDefaultTabHeight},
			{"setDefaultHelpHeight", setDefaultHelpHeight},

			{"tabColorsActive",     tabColorsActive},
			{"tabColorsSelected",   tabColorsSelected},
			{"tabColorsUnselected", tabColorsUnselected},

			{"helpBoxColors", helpBoxColors},

			{"isSelected", isSelected},
			{"isActive",   isActive},
			{NULL,  NULL}
	};

	luaL_register(L, "menupage", methods);
}

} // menupage

//--------------------------------------------------------------------------//
namespace tabbedwindow {

std::map<WidgetId, TabbedWindowPtr> _widgets;

int create(lua_State* L)
{
	int narg = 2;
	const int x = luaL_checkint(L, narg++);
	const int y = luaL_checkint(L, narg++);
	const int w = luaL_checkint(L, narg++);
	const int h = luaL_checkint(L, narg++);
	const int zIndex = luaL_checkint(L, narg++);

	TabbedWindowPtr ww = boost::make_shared<TabbedWindow>(x,y, w,h);
	ww->zIndex(zIndex);
	WidgetId id = Wgt::view()->addWidget(ww);
	_widgets[id] = ww;

	lua_pushnumber(L, id);
	return 1;
}

int addPage(lua_State* L)
{
	int narg = 2;
	const WidgetId tabbedid = luaL_checkint(L, narg++);
	const WidgetId pageid   = luaL_checkint(L, narg++);

	TabbedWindowPtr window = _widgets[tabbedid];
	PagePtr page = menupage::_widgets[pageid]; //wgt::view()->getWidgetById(wgtid);

	window->addPage(page);

	return 0;
}

int currentPage(lua_State* L)
{
	int narg = 2;
	const WidgetId tabbedid = luaL_checkint(L, narg++);

	TabbedWindowPtr ww = _widgets[tabbedid];

	lua_pushnumber(L, ww->currentPage()->id());
	return 1;
}

int currentPageIndex(lua_State* L)
{
	int narg = 2;
	const WidgetId tabbedid = luaL_checkint(L, narg++);

	TabbedWindowPtr ww = _widgets[tabbedid];

	lua_pushnumber(L, ww->currentPageIndex());
	return 1;
}

int selectNextTab(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	_widgets[id]->selectNextTab();
	_widgets[id]->currentPage()->helpBox()->needsRepaint(true);
//	_widgets[id]->needsRepaint(true);

	return 0;
}

int selectPrevTab(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);

	_widgets[id]->selectPrevTab();
	_widgets[id]->currentPage()->helpBox()->needsRepaint(true);
//	_widgets[id]->needsRepaint(true);

	return 0;
}

int selectTab(lua_State* L)
{
	int narg = 2;
	const WidgetId id = luaL_checkint(L, narg++);
	const int ix      = luaL_checkint(L, narg++);

	_widgets[id]->selectTab(ix);
	_widgets[id]->needsRepaint(true);

	return 0;
}

void init(lua_State *L){
	std::cout << "[lua] tabbedwindow::init" << std::endl;
	const struct luaL_Reg methods[] = {
			{"new",         create},
			{"addPage",     addPage},
			{"currentPage", currentPage},
			{"currentPageIndex", currentPageIndex},
			{"selectNextTab", selectNextTab},
			{"selectPrevTab", selectPrevTab},
			{"selectTab",     selectTab},

			{NULL,  NULL}
	};

	luaL_register(L, "tabbedwindow", methods);
}

} // tabbedwindow


} // wgt
