/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * button.cpp
 *
 *  Created on: May 19, 2011
 *      Author: gonzalo
 */

#include "button.h"

#include "../view.h"
#include "../modules/wgt.h"
#include <boost/make_shared.hpp>
#include <iostream>
#include <cstdio>

namespace wgt
{

Button::Button(int x, int y, int w, int h, const Color& bgColor, const Color& activeBgColor, const Color& disabledBgColor/*=Color::Gray2*/)
	: Window(x,y, w,h, bgColor)
	, _label(boost::make_shared<Label>(0,0, w,h, "",14, Color::White, alignment::vTop|alignment::hCenter))
	, _icon(ImagePtr())
	, _normalBgColor(bgColor)
	, _activeBgColor(activeBgColor)
	, _disabledBgColor(disabledBgColor)
	, _normalTextColor(Color::White)
	, _activeTextColor(Color::Black)
	, _normalIconPath("")
	, _activeIconPath("")
	, _laction("")
	, _selectedCbk("")
	, _enabled(true)
{
	addChild(_label);
}

Button::~Button()
{}

void Button::draw(View* view){
	view->draw(this);
}

void Button::textColor(const Color& color, const Color& activeColor)
{
	_normalTextColor = color;
	_activeTextColor = activeColor;
}

void Button::buttonColor(const Color& color, const Color& activeColor)
{
	_normalBgColor = color;
	_activeBgColor = activeColor;
}

void Button::activate(bool b, bool /* fromTop */)
{
	if (_enabled) {
		Widget::activate(b);

		_label->color(textColor());
		if (_icon) {
			_icon->path(iconPath());
		}

		if (_selectedCbk.size()) {
			Wgt::call(_selectedCbk, b);
		}
	}
}

void Button::enable( bool enabled/*=true*/ ) {
	_enabled = enabled;
}

bool Button::isEnabled() {
	return _enabled;
}

void Button::label(const std::string& t)
{
	_label->text(t);
}

void Button::icon(const std::string& path, const std::string& activePath, int imgw, int imgh)
{
	_normalIconPath = path;
	_activeIconPath = activePath;

	if (_icon) {
		_icon->path(iconPath());
	} else {
		_icon = boost::make_shared<Image>((w()-imgw)/2, 4, imgw, imgh, iconPath());
		_label->y(_label->y() + imgh/2);
		_label->h(_label->h() - imgh/2);
		_label->alignment(alignment::vCenter|alignment::hCenter);
		addChild(_icon);
	}
}

const std::string& Button::iconPath()
{
	return (_active)? _activeIconPath : _normalIconPath;
}

const Color& Button::bgColor()
{
	if (!_enabled) {
		return _disabledBgColor;
	}
	return (_active)? _activeBgColor : _normalBgColor;
}

const Color& Button::textColor()
{
	return (_active)? _activeTextColor : _normalTextColor;
}

void Button::action(Action f)
{
	_action = boost::optional<Action>(f);
}

void Button::action(const std::string& f)
{
	_laction = f;
}

void Button::execute()
{
	if (_action) {
		(*_action)();
	}
	if (_laction.size()) {
		//std::cout << "[lua] Button::execute, action=" << _laction << std::endl;
		Wgt::call(_laction);
	}
}

bool Button::onOk()
{
	execute();
	return true;
}

LabelPtr Button::label()
{
	return _label;
}

void Button::selectedCallback(const std::string& cbk)
{
	_selectedCbk = cbk;
}

}
