/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * buttonset.cpp
 *
 *  Created on: May 20, 2011
 *      Author: gonzalo
 */

#include "buttonset.h"

namespace wgt
{

ButtonSet::ButtonSet(int x, int y, int w, int h, const Color& bgColor)
	: Window(x,y,w,h,bgColor)
	, _circular(true)
{}

ButtonSet::~ButtonSet()
{}

bool ButtonSet::onLeftArrow()
{
	parent()->updatePreviousActiveChild();
	activeChild()->needsRepaint(true);
	selectPreviousChild(_circular);
	activeChild()->needsRepaint(true);
	return true;
}

bool ButtonSet::onRightArrow()
{
	parent()->updatePreviousActiveChild();
	activeChild()->needsRepaint(true);
	selectNextChild(_circular);
	activeChild()->needsRepaint(true);
	return true;
}

void ButtonSet::addButton(ButtonPtr b)
{
	addChild(b);
}

void ButtonSet::circular(bool b)
{
	_circular = b;
}

#define MOVE(circular, way) \
	ButtonPtr btn; \
	ButtonPtr first = boost::static_pointer_cast<Button>(activeChild()); \
	do { \
		way(circular); \
		btn = boost::static_pointer_cast<Button>(activeChild()); \
	} while (!btn->isEnabled() && first != btn );

void ButtonSet::selectPreviousChild(bool circular)
{
	MOVE(circular, Window::selectPreviousChild);
}

void ButtonSet::selectNextChild(bool circular)
{
	MOVE(circular, Window::selectNextChild);
}

#undef MOVE

void ButtonSet::selectChild(size_t ix)
{
	Window::selectChild(ix);
	ButtonPtr btn = boost::static_pointer_cast<Button>(activeChild());
	if (!btn->isEnabled()) {
		selectNextChild(_circular);
	}
}

const std::string& ButtonSet::currentHelp()
{
	return activeChild()->help();
}

}
