/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * dialog.cpp
 *
 *  Created on: May 24, 2011
 *      Author: gonzalo
 */

#include "dialog.h"

#include "../view.h"
#include "../modules/wgt.h"

#include <boost/bind.hpp>
#include <boost/make_shared.hpp>

#include <iostream>

namespace wgt
{

Dialog::Dialog(int x, int y, int w, int h, const Color& bgColor, const Color& borderColor, int borderWidth)
	: BorderedWindow(x,y,w,h, bgColor, borderColor, borderWidth)
	, _margin(20)
	, _titleHeight(24)
	, _title(TextAreaPtr(new TextArea(borderWidth,borderWidth, w-2*borderWidth,_titleHeight, borderColor, "", 14, 1, Color::White, borderWidth, alignment::vTop|alignment::hLeft)))
	, _text(TextAreaPtr(new TextArea(borderWidth + _margin, _titleHeight + borderWidth +_margin, w-2*(borderWidth+_margin), h-_titleHeight - 2*(borderWidth +_margin), bgColor, "", 14, 1, Color::White, 0, alignment::vTop|alignment::hLeft)))
	, _icon(ImagePtr())
{
	addChild(_title, false);
	addChild(_text, false);
	setVisible(false);
}

Dialog::~Dialog()
{
}

void Dialog::title(const std::string& t)
{
	_title->text(t);
}

TextAreaPtr Dialog::text()
{
	return _text;
}

void Dialog::iconPath(const std::string& path, int imgw, int imgh, const std::string& pathDisabled)
{
	if (_icon) {
		_icon->path(path);
		if (!pathDisabled.empty()) {
			_icon->dimmedPath(pathDisabled);
		}
	} else {
		_icon = boost::make_shared<Image>(borderWidth()+_margin, _titleHeight + borderWidth()+_margin, imgw, imgh, path);
		_text->x(_text->x() + imgw + _margin);
		_text->w(_text->w() - imgw - _margin);
		addChild(_icon, false);
		_icon->fixOffsets(this);
	}
}

bool Dialog::onExit()
{
	close();
	return true;
}

void Dialog::titleColor(const Color& color)
{
	_title->textColor(color);
}

void Dialog::textColor(const Color& color)
{
	_text->textColor(color);
}

//----------------------------------------------------------------------------------//
YesNoDialog::YesNoDialog(int x, int y, int w, int h, const Color& bgColor, const Color& borderColor, int borderWidth, int bw, int bh)
	: Dialog(x,y, w,h, bgColor, borderColor, borderWidth)
{
	_yes = boost::make_shared<Button>(0,0, bw, bh, Color::Blue, Color::Yellow);
	_yes->textColor(Color::White, Color::Black);
	_yes->label("SI");
	_yes->label()->fontSize(20);
	_yes->action(boost::bind(&YesNoDialog::close, this));

	_no = boost::make_shared<Button>(bw + 25,0, bw, bh, Color::Blue, Color::Yellow);
	_no->textColor(Color::White, Color::Black);
	_no->label("NO");
	_no->label()->fontSize(20);
	_no->action(boost::bind(&YesNoDialog::close, this));

	int buttonset_w = _no->x() + _no->w();
	int buttonset_h = _no->h();
	_buttonSet = boost::make_shared<ButtonSet>((w - 2*bw-10)/2, _titleHeight + text()->h(), buttonset_w, buttonset_h, bgColor);
	_buttonSet->addButton(_yes);
	_buttonSet->addButton(_no);
	addChild(_buttonSet);

	fixOffsets();
}

void YesNoDialog::buttonColor(const Color& color, const Color& activeColor)
{
	_yes->buttonColor(color, activeColor);
	_no->buttonColor(color, activeColor);
}

void YesNoDialog::buttonTextColor(const Color& color, const Color& activeColor)
{
	_yes->textColor(color, activeColor);
	_no->textColor(color, activeColor);
}

void YesNoDialog::closeFirst(Button::Action f)
{
	close();
	f();
}

void YesNoDialog::yesAction(Button::Action fYes)
{
	_yes->action(boost::bind(&YesNoDialog::closeFirst, this, fYes));
}

void YesNoDialog::noAction(Button::Action fNo)
{
	_no->action(boost::bind(&YesNoDialog::closeFirst, this, fNo));
}

void YesNoDialog::yesAction(const std::string& fYes)
{
	_yes->action(boost::bind(&YesNoDialog::close, this));
	_yes->action(fYes);
}

void YesNoDialog::noAction(const std::string& fNo)
{
	_no->action(boost::bind(&YesNoDialog::close, this));
	_no->action(fNo);
}

bool YesNoDialog::onExit()
{
	Dialog::onExit();
	_no->execute(); // cancel
	return true;
}

void YesNoDialog::open()
{
	_buttonSet->selectChild(1); // button 'NO'
	Dialog::open();
}

//----------------------------------------------------------------------------------//

AlertDialog::AlertDialog(int x, int y, int w, int h, const Color& bgColor, const Color& borderColor, int borderWidth)
	: Dialog(x,y, w,h, bgColor, borderColor, borderWidth)
	, _lcallback("")
{
	fixOffsets();
}

void AlertDialog::callback(Callback f)
{
	_callback = boost::optional<Callback>(f);
}

void AlertDialog::callback(const std::string& f)
{
	_lcallback = f;
}

bool AlertDialog::onExit()
{
	return onOk();
}

bool AlertDialog::onOk()
{
	close();
	if (_callback) {
		(*_callback)();
	}
	if (_lcallback.size()) {
		//View::instance()->call(_lcallback);
		Wgt::call(_lcallback);
	}

	return true;
}

} // end namespace
