/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * List.cpp
 *
 *  Created on: May 10, 2011
 *      Author: gonzalo
 */

#include "list.h"

#include "../view.h"

#include <boost/foreach.hpp>
#include <iostream>

namespace wgt
{

List::List(int x, int y, int w, int h, int rowH, int rowSep, int fontSize)
	: Widget(x, y, w, h)
	, _current(-1)
	, _visible(0)
	, _updateScrollbar(true)
	, _rowHeight(rowH)
	, _rowSep(rowSep)
	, _fontSize(fontSize)
	, _cellColor        (Color::DarkBlue)
	, _selectedCellColor(Color::Yellow)
	, _textColor        (Color::White)
	, _selectedTextColor(Color::Black)
	, _headerColor      (Color::Blue)
	, _textHeaderColor  (Color::White)
{
	_marginsNumber = 2;
}

List::~List()
{}

const std::vector<List::Row>& List::contents()
{
	return _contents;
}

int List::current()
{
	return _current;
}

int List::visible()
{
	return _visible;
}

int List::rowHeight()
{
	return _rowHeight;
}

int List::rowSep()
{
	return _rowSep;
}

int List::fontSize()
{
	return _fontSize;
}

void List::setUpdateScrollbar(bool update)
{
	_updateScrollbar = update;
}

void List::contents(const std::vector<List::Row>& rows, const std::vector<std::string>& values)
{
	_contents.clear();
	_contents.assign(rows.begin(), rows.end());
	_values.clear();
	_values.assign(values.begin(), values.end());
	initFromTo(_current, _visible, _contents.size());
	initScrollbar(this, _visible, _contents.size(), _rowHeight, _rowSep, 2 + _rowHeight + _rowSep);
}

void List::updateRow(size_t index, const Row& row, const std::string& value)
{
	if (index < _contents.size()) {
		_contents[index] = row;
		_values[index] = value;
	}
}

void List::addRow(const Row& row, const std::string& value)
{
	_contents.push_back(row);
	_values.push_back(value);
	initFromTo(_current, _visible, _contents.size());
	initScrollbar(this, _visible, _contents.size(), _rowHeight, _rowSep, 2 + _rowHeight + _rowSep);
}

void List::selected(int n)
{
	_current = n;
}

int List::selected()
{
	return _current;
}

void List::setVisibleRows(int n)
{
	_visible = n;
}

void List::addImage(const std::string& key, const std::string& value)
{
	_images[key] = value;
}
ColumnMapping& List::images_normal()
{
	return _mappings_normal;
}
ColumnMapping& List::images_selected()
{
	return _mappings_selected;
}

void List::cellWidths(const std::vector<int>& widths)
{
	_cellWidths.clear();
	_cellWidths.insert(_cellWidths.begin(), widths.begin(), widths.end());

	int listW = 0;
	BOOST_FOREACH(int width, _cellWidths) {
		listW += width + _rowSep;
	}
	w(listW);
}

const std::vector<std::string>& List::cellHeaders()
{
	return _headers;
}

void List::cellHeaders(const std::vector<std::string>& headers)
{
	_headers = headers;
}

const std::vector<int>& List::cellWidths()
{
	return _cellWidths;
}

void List::updateScrollPosition()
{
	if (_updateScrollbar) {
		updateScrollbar(_current, _visible, _contents.size());
	}
}

void List::updateFromTo()
{
	WithScrollbar::updateFromTo(_current, _visible, (int)_contents.size());
}

void List::moveOneUp()
{
	if (_contents.size()) {
		_current = (_current>0) ? _current-1 : _contents.size()-1;
//		std::cout << "[lua] up, current: " << _current << std::endl;
		updateFromTo();
		updateScrollPosition();
	}
}
void List::moveOneDown()
{
	if (_contents.size()) {
//		std::cout << "[lua] down, current: " << _current << std::endl;
		_current++;
		if (_current>=(int)_contents.size()) {
			_current=0;
		}
		updateFromTo();
		updateScrollPosition();
	}
}

bool List::onUpArrow()
{
	const int oldCurrent = _current;
	moveOneUp();
	if (oldCurrent != _current) {
		needsRepaint(true);
	}
	return true;
}

bool List::onDownArrow()
{
	const int oldCurrent = _current;
	moveOneDown();
	if (oldCurrent != _current) {
		needsRepaint(true);
	}
	return true;
}

bool List::onPageUp()
{
	if (_contents.size()) {
		const int oldCurrent = _current;
		_current-=_visible;
		if (_current<0) {
			_current = 0;
		}
		updateFromTo();
		updateScrollPosition();
		if (oldCurrent != _current) {
			needsRepaint(true);
		}
	}
	return true;
}

bool List::onPageDown() {
	if (_contents.size()) {
		const int oldCurrent = _current;
		_current+=_visible;
		if (_current>=(int)_contents.size()) {
			_current = _contents.size() - 1;
		}
		updateFromTo();
		updateScrollPosition();
		if (oldCurrent != _current) {
			needsRepaint(true);
		}
	}
	return true;
}

const Color& List::cellColor()
{
	return _cellColor;
}
const Color& List::selectedCellColor()
{
	return _selectedCellColor;
}
const Color& List::textColor()
{
	return _textColor;
}
const Color& List::selectedTextColor()
{
	return _selectedTextColor;
}
const Color& List::headerColor()
{
	return _headerColor;
}
const Color& List::textHeaderColor()
{
	return _textHeaderColor;
}

void List::draw(View* view)
{
	updateFromTo();
	updateScrollPosition();
	view->draw(this);
}

void List::addColumnMapping(size_t column, const std::string& valueFrom, const std::string& valueToN, const std::string& valueToS)
{
	_mappings_normal[column][valueFrom]   = valueToN;
	_mappings_selected[column][valueFrom] = valueToS;
}

const std::string& List::value()
{
	return _values[_current];
}

void List::fixOffsets(Widget* window)
{
	Widget::fixOffsets(window);
	initScrollbar(this, _visible, _contents.size(), _rowHeight, _rowSep, 2 + _rowHeight + _rowSep);
}

void List::cellColor(const Color& color, const Color& selectedColor)
{
	_cellColor = color;
	_selectedCellColor = selectedColor;
}

void List::textColor(const Color& color, const Color& selectedColor)
{
	_textColor = color;
	_selectedTextColor = selectedColor;
}

void List::headerColor(const Color& headerColor, const Color& textColor)
{
	_headerColor = headerColor;
	_textHeaderColor = textColor;
}


//----------------------------------------------------------------------//

} // end namespace
