/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * menupage.cpp
 *
 *  Created on: May 16, 2011
 *      Author: gonzalo
 */

#include "page.h"

#include "../color.h"
#include "../view.h"
#include "label.h"
#include "../../modules/canvas.h"
#include <boost/make_shared.hpp>
#include <iostream>
#include <stdio.h>

namespace wgt
{
//-----------------------------------------------------------------------------------//
PageTab::PageTab(int x, int y, int w, int h, const Color& bgColor, const Color& borderColor, int borderWidth)
	: BorderedWindow(x,y,w,h, bgColor, borderColor, borderWidth)
{}

void PageTab::label(const std::string& name)
{
	_label = LabelPtr(new Label(0, 2*16, w(), h()/2, name, 16, Color::White, alignment::vCenter|alignment::hCenter));
	addChild(_label);
	widgetname(std::string("Tab ") + name);
}

void PageTab::icon( const std::string& path)
{
	_img = boost::make_shared<Image>((w()-50)/2, 6, 50, 35, path);
	addChild(_img);
}

ImagePtr PageTab::image()
{
	return _img;
}

LabelPtr PageTab::label()
{
	return _label;
}

//-----------------------------------------------------------------------------------//

int Page::_margin = 0;
int Page::_border = 0;
int Page::_tabSpace = 0;
int Page::_tabW = 0;
int Page::_tabH = 0;
int Page::_helpHeight = 0;

Page::Page(int pagex, int pagey, int pagew, int pageh, int taby, bool withHelp)
	: Widget(pagex,pagey, pagew,pageh)
	, _state(page::unselected)

	, _tabActive(    new PageTab(0, taby, Page::_tabW+10, Page::_tabH, Color::Black, Color::Yellow, 4))
	, _tabSelected(  new PageTab(0, taby, Page::_tabW,    Page::_tabH, Color::Black, Color::Yellow, 4))
	, _tabUnselected(new PageTab(0, taby, Page::_tabW,    Page::_tabH, Color::Blue,  Color::Blue,   0))

	, _helpBox(new TextArea(Page::_tabW + Page::_tabSpace, pagey + pageh + Page::_tabSpace, pagew, Page::_helpHeight, Color::Black, "", 14, 2, Color::White, Page::_margin, alignment::vCenter|alignment::hLeft, Color::Yellow, 4))

	, _page(boost::make_shared<BorderedWindow>(Page::_tabW + Page::_tabSpace,0,pagew,pageh, Color::Black, Color::Yellow, 4))
	, _borderColorActive(Color::Yellow)
	, _borderColorInactive(Color::Blue)
	, _name("")
	, _helpVisible(withHelp)
{
	widgetname("Page");
	page()->borderColor(_borderColorInactive);
	_tabActive->parent(this);
	_tabSelected->parent(this);
	_tabUnselected->parent(this);
	_helpBox->parent(this);
	_page->parent(this);

}

Page::~Page()
{
	delete _tabActive;
	delete _tabSelected;
	delete _tabUnselected;
}

void Page::margin(int x)
{
	_margin = x;
}
void Page::border(int x)
{
	_border = x;
}
void Page::tabSpace(int x)
{
	_tabSpace = x;
}
void Page::tabW(int x)
{
	_tabW = x;
}
void Page::tabH(int x)
{
	_tabH = x;
}
void Page::helpHeight(int x)
{
	_helpHeight = x;
}

void Page::draw(View* view)
{
	view->draw(this);
}

PageTab* Page::tab()
{
	switch (_state) {
		case page::active:     return _tabActive;
		case page::selected:   return _tabSelected;
		case page::unselected: return _tabUnselected;
	}
	return 0;
}

TextAreaPtr Page::helpBox()
{
	return _helpBox;
}

void Page::selected(bool b)
{
	if (b) {
		_state = page::selected;
	} else {
		_state = page::unselected;
	}
}

void Page::activated(bool b)
{
	if (b) {
		_state = page::active;
		page()->borderColor(_borderColorActive);
	} else {
		_state = page::selected;
		page()->borderColor(_borderColorInactive);
	}
}

bool Page::isActive()
{
	return _state == page::active;
}

bool Page::isSelected()
{
	return _state == page::active || _state == page::selected;
}

void Page::addChild(WidgetPtr child, bool canHandleKey)
{
	_page->addChild(child,canHandleKey);
}

BorderedWindowPtr Page::page()
{
	return _page;
}

void Page::setHelp(const std::string& text)
{
	help(text);
	helpBox()->text(text);
}

void Page::updateHelp()
{
	helpBox()->text(help());
}

void Page::name(const std::string& text)
{
	widgetname(std::string("Page ") + text);
	_name = text;
	setName(_tabActive,     text);
	setName(_tabSelected,   text);
	setName(_tabUnselected, text);
}

void Page::iconActive(const std::string& path)
{
	setIcon(_tabActive, path);
	setIcon(_tabSelected, path);
}

void Page::iconInactive(const std::string& path)
{
	setIcon(_tabUnselected, path);
}

void Page::iconDisabled(const std::string& path)
{
	_tabActive->image()->dimmedPath(path);
}

void Page::setIcon(PageTab* tab, const std::string& path)
{
	tab->icon(path);
}

void Page::setName(PageTab* tab, const std::string& name)
{
	tab->label(name);
}

void Page::setVisible(bool visible)
{
	page()->setVisible(visible);
}

bool Page::onExit()
{
//	std::cout << "[lua] desactivando la pagina" << std::endl;
	activated(false);
	updateHelp();

	return true;
}

void Page::fixOffsets(Widget* window)
{
	Widget::fixOffsets(window);

	_tabActive->fixOffsets(this);
	_tabSelected->fixOffsets(this);
	_tabUnselected->fixOffsets(this);
	_helpBox->fixOffsets(this);
	_page->fixOffsets(this);

	fillBoundaries(0, _outerPoints);
	fillBoundaries(Page::_border, _innerPoints);
	calculateBorder();
}

void Page::calculateBorder()
{
	_borderPoints.insert(_borderPoints.begin(), _outerPoints.begin(), _outerPoints.end());
}

page::State Page::state()
{
	return _state;
}

const std::vector<std::pair<int,int> >& Page::outerPoints()
{
	return _outerPoints;
}
const std::vector<std::pair<int,int> >& Page::innerPoints()
{
	return _innerPoints;
}

const std::vector<std::pair<int,int> >& Page::borderPoints()
{
	return _borderPoints;
}

void Page::fillBoundaries(int border, std::vector<std::pair<int,int> >& points)
{
	//		       (3)--------(4)
	//		        |          |
	//		        |          |
	//		 (1)---(2)         |
	//		 |                 |
	//		 |                 |
	//		 (8)---(7)         |
	//		        |          |
	//		        |          |
	//		        |          |
	//		        |          |
	//		       (6)--------(5)

	PageTab* tab = _tabActive;
	const int tabx = tab->x();
	const int taby = tab->y();
	const int tabh = tab->h();
	const int pagex = page()->x();
	const int pagey = page()->y();
	const int pagew = page()->w();

	points.push_back(std::make_pair(tabx          + border, taby        + border)); // (1)
	points.push_back(std::make_pair(pagex         + border, taby        + border)); // (2)
	points.push_back(std::make_pair(pagex         + border, pagey       + border)); // (3)
	points.push_back(std::make_pair(pagex + pagew - border - 1, pagey     + border)); // (4)
	points.push_back(std::make_pair(pagex + pagew - border - 1, y() + h() - border -1)); // (5)
	points.push_back(std::make_pair(pagex         + border, y() + h()   - border -1)); // (6)
	points.push_back(std::make_pair(pagex         + border, taby + tabh - border-1)); // (7)
	points.push_back(std::make_pair(tabx          + border, taby + tabh - border-1)); // (8)
}

const std::string& Page::name()
{
	return _name;
}

void Page::borderColorActive(const Color& color)
{
	_borderColorActive = color;
}

void Page::borderColorInactive(const Color& color)
{
	_borderColorInactive = color;
}

void Page::tabColorsActive(const Color& color, const Color& borderColor, const Color& textColor)
{
	_tabActive->bgColor(color);
	_tabActive->borderColor(borderColor);
	_tabActive->label()->color(textColor);
}

void Page::tabColorsSelected(const Color& color, const Color& borderColor, const Color& textColor)
{
	_tabSelected->bgColor(color);
	_tabSelected->borderColor(borderColor);
	_tabSelected->label()->color(textColor);
}

void Page::tabColorsUnselected(const Color& color, const Color& borderColor, const Color& textColor)
{
	_tabUnselected->bgColor(color);
	_tabUnselected->borderColor(borderColor);
	_tabUnselected->label()->color(textColor);
}

void Page::helpBoxColors(const Color& bgColor, const Color& borderColor, const Color& textColor)
{
	_helpBox->bgColor(bgColor);
	_helpBox->borderColor(borderColor);
	_helpBox->textColor(textColor);
}

bool Page::helpVisible()
{
	return _helpVisible;
}

void Page::clear()
{
	luaz::canvas::fillRect(
			_borderPoints[0].first,
			_borderPoints[0].second,
			_borderPoints[6].first - _borderPoints[0].first+Page::_border,
			_borderPoints[6].second - _borderPoints[0].second+1);
	luaz::canvas::fillRect(
				_borderPoints[2].first,
				_borderPoints[2].second,
				_borderPoints[4].first - _borderPoints[2].first + 1,
				_borderPoints[4].second - _borderPoints[2].second + 1);
}

void Page::drawBorder()
{
	luaz::canvas::fillRect(_borderPoints[0].first, _borderPoints[0].second, _borderPoints[1].first - _borderPoints[0].first+Page::_border, Page::_border);
	luaz::canvas::fillRect(_borderPoints[2].first, _borderPoints[2].second, Page::_border, _borderPoints[1].second - _borderPoints[2].second + 1);
	luaz::canvas::fillRect(_borderPoints[2].first, _borderPoints[2].second, _borderPoints[3].first - _borderPoints[2].first + 1, Page::_border);
	luaz::canvas::fillRect(_borderPoints[3].first-Page::_border+1, _borderPoints[3].second, Page::_border, _borderPoints[4].second - _borderPoints[3].second + 1);
	luaz::canvas::fillRect(_borderPoints[5].first, _borderPoints[5].second-Page::_border+1, _borderPoints[4].first - _borderPoints[5].first + 1, Page::_border);
	luaz::canvas::fillRect(_borderPoints[6].first, _borderPoints[6].second, Page::_border, _borderPoints[5].second - _borderPoints[6].second + 1);
	luaz::canvas::fillRect(_borderPoints[7].first, _borderPoints[7].second-Page::_border+1, _borderPoints[6].first - _borderPoints[7].first+Page::_border, Page::_border);
	luaz::canvas::fillRect(_borderPoints[0].first, _borderPoints[0].second, Page::_border, _borderPoints[7].second - _borderPoints[0].second + 1);
}

}
