/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * menupage.h
 *
 *  Created on: May 16, 2011
 *      Author: gonzalo
 */

#ifndef PAGE_H_
#define PAGE_H_

#include "borderedwindow.h"
#include "textarea.h"
#include "image.h"

#include <string>

namespace wgt
{

namespace page {
	typedef enum {active=0, selected, unselected} State;
}

class PageTab : public BorderedWindow
{
public:
	PageTab(int x, int y, int w, int h, const Color& bgColor, const Color& borderColor, int borderWidth);

	void label(const std::string& n);
	void icon( const std::string& i);

	ImagePtr image();
	LabelPtr label();
private:
	ImagePtr _img;
	LabelPtr _label;
};

class Page : public Widget
{
public:
	Page(int pagex, int pagey, int pagew, int pageh, int taby, bool withHelp=true);
	virtual ~Page();

	void draw(View* view);

	void name(const std::string& n);
	void iconActive(const std::string& i);
	void iconInactive(const std::string& i);
	void iconDisabled(const std::string& i);

	void borderColorActive(const Color& color);
	void borderColorInactive(const Color& color);
	void tabColorsActive(    const Color& color, const Color& borderColor, const Color& textColor);
	void tabColorsSelected(  const Color& color, const Color& borderColor, const Color& textColor);
	void tabColorsUnselected(const Color& color, const Color& borderColor, const Color& textColor);
	void helpBoxColors(const Color& bgColor, const Color& borderColor, const Color& textColor);

	virtual void selected(bool b);
	virtual void activated(bool b);

	const std::string& name();

	bool isActive();
	bool isSelected();

	page::State state();

	PageTab* tab();
	BorderedWindowPtr page();
	TextAreaPtr helpBox();

	virtual void fixOffsets(Widget* window=0);
	void addChild(WidgetPtr child, bool canHandleKey=true);

	static void margin(int x);
	static void border(int x);
	static void tabSpace(int x);
	static void tabW(int x);
	static void tabH(int x);
	static void helpHeight(int x);

	void setHelp(const std::string& t);
	void updateHelp();

	const std::vector<std::pair<int,int> >& outerPoints();
	const std::vector<std::pair<int,int> >& innerPoints();
	const std::vector<std::pair<int,int> >& borderPoints();
	
	void setVisible(bool visible);

	bool helpVisible();

	void clear();
	void drawBorder();

protected:
	static int _margin;
	static int _border;
	static int _tabSpace;
	static int _tabW;
	static int _tabH;
	static int _helpHeight;

	virtual bool onExit();
private:
	page::State _state;

	PageTab* _tabActive; // no se comparten con nadie
	PageTab* _tabSelected;
	PageTab* _tabUnselected;

	TextAreaPtr _helpBox;

	BorderedWindowPtr _page;

	Color _borderColorActive;
	Color _borderColorInactive;

	std::string _name;
	std::vector<std::pair<int,int> > _outerPoints;
	std::vector<std::pair<int,int> > _innerPoints;
	std::vector<std::pair<int,int> > _borderPoints;

	bool _helpVisible;

	void setIcon(PageTab* tab, const std::string& path);
	void setName(PageTab* tab, const std::string& name);

	void fillBoundaries(int border, std::vector<std::pair<int,int> >& points);
	void calculateBorder();

};

DEF_SHARED(Page);

}

#endif /* MENUPAGE_H_ */
