/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "progressbar.h"
#include "../view.h"
#include <boost/make_shared.hpp>
#include <boost/lexical_cast.hpp>

namespace wgt {

ProgressBar::ProgressBar(int x, int y, int w, int h, int value, int minValue, int maxValue, const std::string& text)
	: Window(x, y, w, h, Color(Color::Blue))
{
	int posX, posY, sW, sH;

	//	Position and size for background and bar
	posX = x + (w * 0.35);
	posY = y + (h * 1 / 8);
	sW = w * 0.50;
	sH = h * 3 / 4;

	_background = WindowPtr(new Window(posX, posY, sW, sH, Color(Color::Black)));
	_bar = boost::make_shared <Bar> (posX, posY, sW, sH, Color(Color::White), value, minValue, maxValue);
	_label = LabelPtr(new Label(x + 2, y, w * 0.20, h * 0.75, text, 14, Color(Color::White), (int) (alignment::vCenter), Color(Color::None), 0));
	_valueView = LabelPtr(new Label(x + (w * 0.90), y, w * 0.10, h * 0.75, text, 14, Color(Color::White), (alignment::vCenter), Color(Color::None), 2));

	this->addChild(_background);
	this->addChild(_bar);
	this->addChild(_label);
	this->addChild(_valueView);
}

ProgressBar::~ProgressBar()
{
	removeChildren();
}

void ProgressBar::show(int value) {
	_bar->value(value);
}

void ProgressBar::setText(const std::string &value) {
	_valueView->text( value );
}

void ProgressBar::setFontSize(int size) {
	_label->fontSize(size);
	_valueView->fontSize(size);
}

}
