/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "slidebar.h"

#include "window.h"
#include "image.h"
#include "../view.h"
#include "../modules/wgt.h"

#include <boost/make_shared.hpp>

#include <sstream>

#define LABEL_W 150

namespace wgt{

SlideBar::SlideBar(int x, int y, int w, int h
		, const Color& frameColor
		, const Color& selectedColor
		, const Color& deepColor
		, const std::string& title
		, int titleFontSize
		, const Color& titleNormalColor
		, const Color& titleActiveColor
		, int minValue
		, int maxValue
		, int step) 
	: Window(x, y, w, h, frameColor)
	, _laction("")
{
	_minValue = minValue;
	_maxValue = maxValue;
	_maxW = 180-2*3;
	_value = _minValue;
	_step = step;
	_frameColor    = frameColor;
	_selectedColor = selectedColor;
	_titleNormalColor = titleNormalColor;
	_titleActiveColor = titleActiveColor;
	_title = boost::make_shared<Label>(10, 0, LABEL_W, h, title, titleFontSize, _titleNormalColor, alignment::vTop|alignment::hLeft);
	addChild(_title);
	
	_valueL = LabelPtr(new Label(177, 3, 30, 20, "", 15, _titleNormalColor, alignment::vBottom|alignment::hRight, deepColor, 3));
	addChild(_valueL);
	
	ImagePtr arrow_left = boost::make_shared<Image>(213, 8, 6, 11, "arrow_left.png");
	addChild(arrow_left);
	
	WindowPtr barBackground = boost::make_shared<Window>(222, 3, 180, 20, deepColor);
	addChild(barBackground);
	
	_valueW = boost::make_shared<Window>(3, 3, _maxW, 20-2*3, _titleNormalColor);
	barBackground->addChild(_valueW);

	ImagePtr arrow_right = boost::make_shared<Image>(405, 8, 6, 11, "arrow_right.png");
	addChild(arrow_right);
}

SlideBar::~SlideBar(){
}

void SlideBar::setValue(int value) {
	_value = value;
	int p = (100*(value-_minValue))/(_maxValue-_minValue);
	int w = _maxW * p / 100;
	_valueW->w(w);

	std::string valueStr;
	std::stringstream out;
	out << value;
	valueStr = out.str();
	_valueL->text(valueStr);
	if (_action) {
		(*_action)();
	}
	if (_laction.size()) {
		//View::instance()->call(_laction);
		Wgt::call(_laction);
	}
}

int SlideBar::value(){
	return _value;
}

void SlideBar::action(const std::string& f)
{
	_laction = f;
}

void SlideBar::action(Action f) {
	_action = boost::optional<Action>(f);
}

void SlideBar::activate(bool b, bool /* fromTop */) {
	Window::activate(b);
	if (b) {
		_title->color(_titleActiveColor);
		_bgColor = _selectedColor;
	} else {
		_title->color(_titleNormalColor);
		_bgColor = _frameColor;
	}
}

bool SlideBar::onRightArrow(){
	parent()->updatePreviousActiveChild();
	setValue(std::min(_value+_step, _maxValue));
	needsRepaint(true);
	return true;
}

bool SlideBar::onLeftArrow(){
	parent()->updatePreviousActiveChild();
	setValue(std::max(_value-_step, _minValue));
	needsRepaint(true);
	return true;
}

bool SlideBar::onOk(){
	return true;
}

}
