/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * mainmenu.cpp
 *
 *  Created on: May 17, 2011
 *      Author: gonzalo
 */

#include "tabbedwindow.h"
#include "../view.h"

namespace wgt
{

TabbedWindow::TabbedWindow(int x, int y, int w, int h)
	: Window(x,y, w,h, Color::None)
	, _currentPage(0)
	, _enabled(true)
	, _circular(true)
{
	widgetname("Tabbed window");
}

TabbedWindow::~TabbedWindow()
{}

void TabbedWindow::draw(View* view)
{
	view->draw(this);
}

void TabbedWindow::addPage(PagePtr page)
{
	page->parent(this);
	_pages.push_back(page);
	page->setVisible(false);
}

PagePtr TabbedWindow::currentPage()
{
	if (_pages.size() > _currentPage) {
		return _pages[_currentPage];
	}
	return PagePtr();
}

int TabbedWindow::currentPageIndex()
{
	return _currentPage;
}

const std::vector<PagePtr>& TabbedWindow::pages()
{
	return _pages;
}

bool TabbedWindow::processKey(util::key::type key)
{
	if (currentPage()->isActive() && currentPage()->processKey(key)) {
		return true;
	}
	return Widget::processKey(key);
}

void TabbedWindow::fixOffsets(Widget* window)
{
	if (window) {
		_x += window->x();
		_y += window->y();
	}
	for (std::vector<PagePtr>::iterator it = _pages.begin(); it != _pages.end(); ++it) {
		(*it)->fixOffsets(this);
	}
}

void TabbedWindow::selectNextTab()
{
	currentPage()->selected(false);
	if (_circular) {
		_currentPage = (_currentPage==_pages.size()-1)? 0 : _currentPage+1;
	} else {
		_currentPage = std::min((int)_pages.size()-1, (int) _currentPage+1);
	}
	currentPage()->selected(true);
}

void TabbedWindow::selectPrevTab()
{
	currentPage()->selected(false);
	if (_circular) {
		_currentPage  = (_currentPage ==0)? _pages.size()-1 : _currentPage-1;
	} else {
		_currentPage = std::max(0, (int) _currentPage-1);
	}
	currentPage()->selected(true);
}

void TabbedWindow::selectTab(size_t ix)
{
	currentPage()->selected(false);
	if (ix < _pages.size()) {
		_currentPage = ix;
		currentPage()->selected(true);
	}
}


bool TabbedWindow::onOk()
{
	currentPage()->activated(true);
	currentPage()->updateHelp();

	return true;
}

bool TabbedWindow::onExit()
{
	close();
	_needsClear = true;
	return true;
}

bool TabbedWindow::onMenu()
{
	close();
	_needsClear = true;
	return true;
}

bool TabbedWindow::onUpArrow()
{
	selectPrevTab();
	return true;
}

bool TabbedWindow::onDownArrow()
{
	selectNextTab();
	return true;
}

}
