/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-luaz implementation.

    DTV-luaz is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-luaz is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-luaz.

    DTV-luaz es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-luaz se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

/*
 * withscrollbar.cpp
 *
 *  Created on: Jun 24, 2011
 *      Author: gonzalo
 */

#include "withscrollbar.h"

#include "widget.h"

#include <iostream>

namespace wgt
{

WithScrollbar::WithScrollbar()
	: _from(0)
	, _to(0)
{}

WithScrollbar::~WithScrollbar()
{}

void WithScrollbar::initScrollbar(Widget* widget, int visible, int count, int rowHeight, int rowSep, int ymargin)
{
	scroll.margin = 3;
	scroll.x = widget->x() + widget->w();
	scroll.y = widget->y() + ymargin;
	scroll.h = (rowHeight + rowSep)*visible - rowSep;
	scroll.w = 20;

	int visible_p  = (count > visible)? 100*visible / count : 100;
//	std::cout << "[lua] initScrollbar, p: " << visible_p << std::endl;
	handle.x = scroll.x + scroll.margin;
	handle.w = scroll.w - 2*scroll.margin;
	handle.h = (scroll.h - 2*scroll.margin)*visible_p/100;
}

void WithScrollbar::updateScrollbar(int current, int visible, int count)
{
	if (current < 0) {
		current = 0;
	}
	int visible_xp = (count > visible)? 100*current / (count-1) : 100;
//	std::cout << "[lua] updatePosition, current: " << current << ", visible: " << visible << ", count: " << count << std::endl;
//	std::cout << "[lua] updatePosition, xp: " << visible_xp << std::endl;
	handle.y = scroll.y + scroll.margin + (scroll.h - _marginsNumber*scroll.margin - handle.h)*visible_xp/100;
}

int WithScrollbar::from()
{
	return _from;
}

int WithScrollbar::to()
{
	return _to;
}

void WithScrollbar::updateFromTo(int current, int visible, int count)
{
//	std::cout << "[lua] updateFromTo, current: " << current << ", visible: " << visible << ", count: " << count << std::endl;
//	std::cout << "[lua]             , from: " << _from << ", to: " << _to << std::endl;
	if (current >= 0) {
		if (_from + visible <= current) {
			const int d = std::max(0,current-_to);
			_from += d;
			_to = std::min(_to + d, count);
		} else if (_to - visible >= current) {
			const int d = std::max(0,_from-current);
			_from = std::max(0, _from - d);
			_to -= d;
		}
	}
//	std::cout << "[lua]             , from: " << _from << ", to: " << _to << std::endl;
}

void WithScrollbar::initFromTo(int current, int visible, int count)
{
	_to   = std::min(count, visible) - 1;
	_from = std::max(0, _to - visible + 1);
	updateFromTo(current, visible, count);

//	std::cout << "[lua] initFromTo, visible: " << visible << ", count: " << count << std::endl;
//	std::cout << "[lua] initFromTo, from: " << _from << ", to: " << _to << std::endl;
}

}
