/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "descriptors.h"
#include "descriptors/generic.h"
#include <util/mcr.h>
#include <util/log.h>
#include <util/string.h>
#include <util/cfg/configregistrator.h>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

REGISTER_INIT_CONFIG( tuner_demuxer ) {
	root().addNode( "demuxer" );
}

namespace tuner {
namespace desc {

std::string show( const MapOfDescriptors &desc ) {
	std::string tmp;
	
#ifdef _DEBUG
	tmp = util::format( "%d", desc.size() );
	if (desc.size()) {
		tmp += " (";		
		int i=0;
		BOOST_FOREACH( const DescriptorItem &item, desc ) {
			tmp += util::format( i ? ",%02x" : "%02x", item.first );
			i++;
		}
		tmp += ")";
	}
#else
	UNUSED(desc);
#endif
	return tmp;
}

util::BYTE getStreamIdentifier( const MapOfDescriptors &desc ) {
	//	Find stream identifier descriptor
	MapOfDescriptors::const_iterator it = desc.find( ident::stream_identifier );
	return (it != desc.end()) ? (*it).second.get<util::BYTE>() : 0xFF;
}

bool getVideoDecode( const MapOfDescriptors &desc, int &width, int &height ) {
	MapOfDescriptors::const_iterator it = desc.find( ident::video_decode );
	if (it != desc.end()) {
		bool result=true;
		const VideoDecodeDescriptor &vDesc = (*it).second.get<VideoDecodeDescriptor>();
		switch (vDesc.videoEncodeFormat) {
			case 0:	//	1080p
			case 1:	//	1080i
				width = 1920;
				height = 1080;
				break;
			case 2:	//	720p
				width = 1280;
				height = 720;
				break;
			case 3:	//	480p
			case 4:	//	480i
				width = 720;
				height = 576;
				break;
			default:
				result=false;
		};
		return result;
	}
	return false;
}

}
}
