/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "funcs.h"
#include "../generic.h"
#include <util/log.h>
#include <boost/foreach.hpp>

namespace tuner {
namespace desc {

bool fnc0d6Parser( MapOfDescriptors &descriptors, BYTE *descPayload, size_t len ) {
	EventGroupDescriptor desc;
	size_t offset = 0;

	util::BYTE tmp = RB(descPayload, offset);
	desc.groupType = (tmp & 0xF0) >> 4; 
	int count = tmp & 0x0F;
	
	for(int i=0; i < count; i++ ) {
		Event ev;
		ev.serviceID = RW( descPayload, offset );
		ev.eventID = RW( descPayload, offset );
		desc.events.push_back(ev);
	}

	if (offset < len) {
		if ( desc.groupType == 4 || desc.groupType == 5) {
			while (offset < len) {
				Data data;
				data.groupData.originalNetworkID = RW(descPayload, offset);
				data.groupData.transportStreamID = RW(descPayload, offset);
				data.groupData.serviceID = RW(descPayload, offset);
				data.groupData.eventID = RW(descPayload, offset);
				desc.data.push_back(data);
			}
		} else {
			while (offset < len) {
				Data data;
				data.privateData = RW(descPayload, offset);
				desc.data.push_back(data);
			}
		}
	}
	
	descriptors[ident::event_group] = desc;
	LTRACE("TAG::D6", "Event Group: type=0x%X, event count=%d, data count=%d", desc.groupType, desc.events.size(), desc.data.size());
	BOOST_FOREACH( Event event, desc.events ) {
		LTRACE("TAG::D6", "\t serviceID=%d, eventID=%d", event.serviceID, event.eventID);
	}
	if ( desc.groupType == 4 || desc.groupType == 5) {
		BOOST_FOREACH( Data data, desc.data ) {
			LTRACE("TAG::D6", "\t originalNetworkID=%d, transportStreamID=%d, serviceID=%d, eventID=%d", 
				data.groupData.originalNetworkID, 
				data.groupData.transportStreamID,
				data.groupData.serviceID,
				data.groupData.eventID
      			);
		}
	} else {
		std::stringstream stream;
		BOOST_FOREACH( Data data, desc.data ) {
			stream << data.privateData << ',';
		}
		std::string str( stream.str() );
		LTRACE("TAG::D6", "\t privateData=[%s]", str.substr(0,str.length()-1).c_str());
	}
	return true;
}

}
}
