/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "funcs.h"
#include "../generic.h"
#include <util/log.h>
#include <boost/foreach.hpp>

namespace tuner {
namespace desc {

bool fnc0d9Parser( MapOfDescriptors &descriptors, BYTE *descPayload, size_t /*len*/ ) {
	ComponentGroupDescriptor desc;
	size_t offset = 0;

	util::BYTE tmp = RB(descPayload,offset);
	desc.type = (tmp & 0xE0) >> 5;
	desc.totalBitrateFlag = (tmp & 0x10) ? true : false;
	int cantGroups = tmp & 0x0F;
	for (int i=0; i<cantGroups; i++) {
		ComponentGroupMember cgm;

		tmp = RB(descPayload,offset);
		cgm.id = (tmp & 0xF0) >> 4;
		int cantUnits = tmp & 0x0F;
		for (int j=0; j<cantUnits; j++) {
			CAUnit cau;
			
			tmp = RB(descPayload,offset);
			cau.id = (tmp & 0xF0) >> 4;
			int cantComponents = tmp & 0x0F;
			for (int k=0; k<cantComponents; k++) {
				util::BYTE componentTag = RB(descPayload,offset);
				cau.componentsTag.append((char*)&componentTag, 1);
			}

			cgm.units.push_back(cau);
		}
		if (desc.totalBitrateFlag) {
			cgm.totalBitrate = RB(descPayload,offset);
		} else {
			cgm.totalBitrate = 0;
		}
		offset += parseText( cgm.text, descPayload+offset );

		desc.groups.push_back(cgm);
	}

	descriptors[ident::component_group] = desc;
	LTRACE("TAG::D9", "Component Group: type=0x%X, totalBitrateFlag=%s, group count=%d", desc.type, desc.totalBitrateFlag ? "true" : "false", cantGroups);
	BOOST_FOREACH( ComponentGroupMember cgm, desc.groups ) {
		LTRACE("TAG::D9", "\tGroup: id=%d, totalBitrate=%d, text=%s, CAUnit count=%d", cgm.id, cgm.totalBitrate, cgm.text.c_str(), cgm.units.size() );
		BOOST_FOREACH( CAUnit cau, cgm.units) {
			LTRACE("TAG::D9", "\t\tCAUnit: id=%d, components tags=%s", cau.id, cau.componentsTag.asHexa().c_str());
		}
	}
	return true;
}

}
}
