/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "types.h"
#include <util/log.h>
#include <boost/foreach.hpp>

namespace tuner {
namespace ait {

void show( const ApplicationDescriptor &app ) {
	LDEBUG( "ait::Descriptor", "srvBoundFlag=%d, visibility=%x, priority=%d",
		app.serviceBoundFlag, app.visibility, app.priority );

	BOOST_FOREACH( const ApplicationProfile &profile, app.profiles ) {
		LDEBUG( "ait::Descriptor", "\t profile=%04x, version=(%02x,%02x,%02x)",
			profile.profile, profile.major, profile.minor, profile.micro );
	}

	BOOST_FOREACH( BYTE label, app.transports ) {
		LDEBUG( "ait::Descriptor", "\t Transport label: %02x", label );
	}
}

void show( const ExternalApplications &apps ) {
	BOOST_FOREACH( const ExternalApplication &app, apps ) {
		LDEBUG( "ait::ExternalApplication", "orgID=%08x, appID=%04x, priority=%02x", app.orgID, app.id, app.priority );
	}
}

void show( const TransportDescriptor &transports ) {
	LDEBUG( "ait::Transports", "size=%d", transports.size() );
	BOOST_FOREACH( const TransportProtocolDescriptor &transport, transports ) {
		LDEBUG( "ait::TPDescriptor", "protocolID=%04x, label=%x",
			transport.protocolID, transport.label );

		switch (transport.protocolID) {
			case TP_OBJECT_CAROUSEL:
			case TP_DATA_CAROUSEL:
			{
				const DSMCCTransportProtocolDescriptor dsmcc = transport.info.get<DSMCCTransportProtocolDescriptor>();
				if (dsmcc.remote) {
					LDEBUG( "ait::TPDesc", "\t DSMCC: remote=%02x, networkID=%04x, tsID=%04x, serviceID=%04x, tag=%02x",
						dsmcc.remote, dsmcc.networkID, dsmcc.tsID, dsmcc.serviceID, dsmcc.componentTag );
				}
				else {
					LDEBUG( "ait::TPDesc", "\t DSMCC: remote=%02x, tag=%02x",
						dsmcc.remote, dsmcc.componentTag );
				}
				break;
			}
			default:
				break;
		};
	}
}

void show( const ApplicationNameDescriptor &desc ) {
	BOOST_FOREACH( const ApplicationName &app, desc ) {
		LDEBUG( "ait::NameDescriptor", "Name=%s, language=%s",
			app.name.c_str(), app.language.code() );
	}
}

void show( const GingaApplicationDescriptor &desc ) {
	BOOST_FOREACH( const std::string &param, desc ) {
		LDEBUG( "ait::GingaApp", "param=%s", param.c_str() );
	}
}

void show( const GingaApplicationLocationDescriptor &desc ) {
	LDEBUG( "ait::GingaAppLocation", "base=%s, classPath=%s, initial=%s",
		desc.base.c_str(), desc.classPath.c_str(), desc.initialClass.c_str() );
}

void show( const Application &app ) {
	LDEBUG( "ait::Application", "Application: organization=%x, appID=%x, ctrlCode=%x, descriptors=%s",
		app.orgID, app.id, app.ctrlCode, desc::show( app.descriptors ).c_str() );
}

}
}
