/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-mpegparser implementation.

    DTV-mpegparser is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-mpegparser is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-mpegparser.

    DTV-mpegparser es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-mpegparser se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "isdbtfrontend.h"
#include "generated/config.h"
#include <util/cfg/configregistrator.h>
#include <util/log.h>
#include <boost/lexical_cast.hpp>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

//	VHF
#define FIRST_VHF_CHANNEL   7
#define LAST_VHF_CHANNEL    13
#define FIRST_VHF_FREQUENCY 177142

//	UHF
#define FIRST_UHF_CHANNEL   14
#define LAST_UHF_CHANNEL    69
#define FIRST_UHF_FREQUENCY 473142


namespace tuner {

REGISTER_INIT_CONFIG( tuner_provider_isdbt ) {
	root().addNode( "isdbt" )
	.addValue( "bandwidth", "Bandwidth", 6000 )
	.addValue( "first_channel", "First channel", 0 )
	.addValue( "max_channels", "Max channels", -1 );
}

ISDBTFrontend::ISDBTFrontend()
{
	util::cfg::PropertyNode &isdbtNode = util::cfg::get("tuner.provider.isdbt");

	_bandwidth = isdbtNode.get<int>( "bandwidth" );
	_firstChannel = isdbtNode.get<int>( "first_channel" );
	_maxChannels = isdbtNode.get<int>( "max_channels" );
	_chan = 0;
	
	buildChannels();
}

ISDBTFrontend::~ISDBTFrontend()
{
}

size_t ISDBTFrontend::getCount() const {
	if (_maxChannels > 0) {
		return _maxChannels;
	}
	return _channels.size();
}

std::string ISDBTFrontend::getName( size_t nIndex ) const {
	const Channel &ch = _channels[ first(nIndex) ];
	return boost::lexical_cast<std::string>(ch.freq/1000);
}

size_t ISDBTFrontend::find( const std::string &name ) const {
	int freq = boost::lexical_cast<int>(name);
	for (size_t i=0; i<_channels.size(); i++) {
		if (_channels[i].freq == freq) {
			return i+_firstChannel;
		}
	}
	return size_t(-1);
}

bool ISDBTFrontend::start( size_t nIndex ) {
	const Channel &ch = _channels[ first(nIndex) ];
	return setFrequency( ch.freq, _bandwidth );
}

size_t ISDBTFrontend::first( size_t nIndex ) const {
	return nIndex+_firstChannel;
}

void ISDBTFrontend::buildChannels() {
	LDEBUG( "ISDBTFrontend", "Build frequenies list" );
	
	//	HIGH VHF Channels
	addChannel( FIRST_VHF_FREQUENCY, FIRST_VHF_CHANNEL, LAST_VHF_CHANNEL );
	//	UHF Channels	
	addChannel( FIRST_UHF_FREQUENCY, FIRST_UHF_CHANNEL, LAST_UHF_CHANNEL );
}

void ISDBTFrontend::addChannel( int firstFreq, int firstChannel, int lastChannel ) {
	Channel ch;
	for (int i=0;i<=lastChannel-firstChannel; i++) {
		ch.id = firstChannel+i;
		if (ch.id == 37) {	//	Skip!
			continue;
		}
		ch.freq = firstFreq+(i*_bandwidth);
		_channels.push_back( ch );
	}
}

}

