/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "networkfrontend.h"
#include "../../../demuxer/ts/demuxer.h"
#include <util/buffer.h>
#include <util/cfg/configregistrator.h>
#include <util/log.h>
#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <string>


REGISTER_INIT_CONFIG( tuner_provider_tsdata_network ) {
	root().addNode( "network" )
		.addValue( "host", "Host ip address", std::string("127.0.0.1") )
		.addValue( "firstport", "First port to scan", 5000 )
		.addValue( "lastport", "Last port to scan", 5050 );
}

namespace tuner {
namespace ts {

using boost::asio::ip::tcp;

NetworkFrontend::NetworkFrontend( demuxer::ts::Demuxer *demux )
	: URLFrontend(demux), _socket(_io)
{
	const std::string &host = util::cfg::getValue<std::string>("tuner.provider.tsdata.network.host");
	util::WORD firstPort = util::cfg::getValue<util::WORD>("tuner.provider.tsdata.network.firstport");
	util::WORD lastPort = util::cfg::getValue<util::WORD>("tuner.provider.tsdata.network.lastPort");

	init(host, firstPort, lastPort);
}

NetworkFrontend::~NetworkFrontend()
{
}

//	Set current network
bool NetworkFrontend::start( size_t nIndex ) {
	bool result;
	std::string net = getName( nIndex );
	//	Connect to url
	result=connect( net );
	if (result) {
		assert( !_thread.joinable() );
		
		//  start network thread
		_thread = boost::thread( boost::bind( &NetworkFrontend::netReader, this, net ) );
	}

	LDEBUG( "ts::NetworkFrontend", "Start network: net=%s, result=%d", net.c_str(), result );

	return result;
}

void NetworkFrontend::stop() {
	LDEBUG( "ts::NetworkFrontend", "Stop network" );
		
	//	Cancel socket ...
	_socket.close();

	//	Join thread
	_thread.join();
}

bool NetworkFrontend::connect( const std::string &url ) {
	//  Try connect to url
	tcp::resolver resolver(_io);
	std::string host, port;

	bool result = parseUrl(url, &host, &port);
	if (result) {

		tcp::resolver::query query( host, port );
		tcp::resolver::iterator it = resolver.resolve(query);
		boost::system::error_code error = boost::asio::error::host_not_found;
		_socket.connect(*it, error);
		if (error) {
			_socket.close();
			LERROR( "ts::NetworkFrontend", "Cannot connect to %s", url.c_str() );
			return false;
		}

		//  Set low water mark (warning, that is not strict!)
		boost::asio::socket_base::receive_low_watermark option(188*4);
		_socket.set_option(option);
	}

	return result;
}

//	Running in provider thread
void NetworkFrontend::netReader( const std::string net ) {
	boost::system::error_code error;
	util::Buffer *buf=NULL;

	LINFO( "NetworkFrontend", "Network thread started: net=%s", net.c_str() );

	bool exit=false;
	while (!exit) {
		//  Get a buffer
		if (!buf) {
			buf = demux()->allocBuffer();
			assert(buf);
		}

		//  Read from network
		size_t len = _socket.read_some( boost::asio::buffer(buf->buffer(),buf->capacity()), error );
		if (error) {
			demux()->freeBuffer( buf );
			assert(!len);
			exit=true;
		}
		else if (len) {
			//	Enqueue buffer into demuxer
			buf->resize( len );
			demux()->pushData( buf );
			buf = NULL;
		}
	}
	LINFO( "ts::NetworkFrontend", "Network thread terminated: net=%s", net.c_str() );
}

}
}
