/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "applicationcontroller.h"
#include "spawner.h"
#include "../application.h"
#include "../applicationextension.h"
#include "../../../servicemanager.h"
#include <util/log.h>
#include <boost/bind.hpp>

namespace tuner {
namespace app {

ApplicationController::ApplicationController()
{
	_mgr = NULL;
	_extension = NULL;
	_layerEnabled = false;
	_layerMode = video::mode::sd;
}

ApplicationController::~ApplicationController()
{
}

//	Create and setup extension
ApplicationExtension *ApplicationController::initialize() {
	assert(!_extension);

	//	Create extension
	_extension = new ApplicationExtension( this );

	//	Add application factories
	setupFactories( _extension );

	return _extension;
}

void ApplicationController::finalize() {
	_extension = NULL;
}

void ApplicationController::attach( ServiceManager *mgr ) {
	assert(mgr);
	_mgr = mgr;
}

ServiceManager *ApplicationController::mgr() const {
	assert(_mgr);
	return _mgr;
}

void ApplicationController::onApplicationChanged( Application *app ) {
	bool anyRunning = spawner()->running() > 0;
	if (!_layerEnabled && anyRunning && app->status() > tuner::app::status::started) {
		LDEBUG( "ApplicationController", "Enable layer" );
		enableLayer( true );
		_layerEnabled = true;
	}
	else if (_layerEnabled && !anyRunning) {
		LDEBUG( "ApplicationController", "Layer disabled" );
		enableLayer( false );
		_layerEnabled = false;		
	}	
}

bool ApplicationController::isLayerEnabled() const {
	return _layerEnabled;
}

ApplicationExtension *ApplicationController::extension() const {
	assert(_extension);
	return _extension;
}

//	Start/stop/scan
video::mode::type ApplicationController::setupLayerResolution( const std::vector<video::mode::type> &modes ) {
	if (!isLayerEnabled()) {
		_layerMode = setupLayer( modes );
	}
	return _layerMode;
}

video::mode::type ApplicationController::setupLayer( const std::vector<video::mode::type> & /*modes*/ ) {
	return video::mode::sd;	
}

}
}

