/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "ait.h"
#include "../../../src/demuxer/psi/psi.h"
#include "../../../src/demuxer/psi/application/types.h"
#include "../../../src/demuxer/psi/application/ait.h"
#include "../../../src/demuxer/psi/application/aitdemuxer.h"
#include <boost/bind.hpp>
#include <util/buffer.h>
#include <boost/foreach.hpp>

AitTest::AitTest() {
}

AitTest::~AitTest() {
}

void AitTest::onAit( const boost::shared_ptr<tuner::Ait> &ait ) {
	_ait = ait;
}

tuner::PSIDemuxer *AitTest::createDemux( ) {
	return tuner::createDemuxer<tuner::AITDemuxer, tuner::Ait > (
			pid( ),
			boost::bind( &AitTest::onAit, this, _1 ),
			boost::bind( &SectionDemuxerTest::onExpired, this, _1 ));
}

void AitTest::cleanProbe() {
	SectionDemuxerTest::cleanProbe( );
	_ait.reset();
}

tuner::ID AitTest::pid() const {
	return 0x04; //TODO: ver pid en documentacion de ARIB
}

TEST_F( AitTest, create_demux ) {
}

TEST_F( AitTest, ait_simple_noapp ) {
	probeSection( "ait_simple_ver0_sec0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( _ait->version( ) == 0 );
	ASSERT_TRUE( _ait->appType( ) == 0x09 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
}

inline tuner::desc::MapOfDescriptors::const_iterator findDesc(
		const tuner::desc::MapOfDescriptors &desc, util::BYTE tag ) {
	return desc.find( tag );
}

void testFirstApplicationProfile(std::vector<tuner::ait::ApplicationProfile> &profiles, const util::WORD profile,
		const util::BYTE major, const util::BYTE minor, const util::BYTE micro ) {
	ASSERT_TRUE( profiles.size( ) == 1 );
	const tuner::ait::ApplicationProfile &appProfile = profiles[0];
	ASSERT_TRUE( appProfile.profile == profile );
	ASSERT_TRUE( appProfile.major == major );
	ASSERT_TRUE( appProfile.minor == minor );
	ASSERT_TRUE( appProfile.micro == micro );
}

TEST_F( AitTest, ait_singleapp ) {
	probeSection( "ait_single_app_ver1_sec0_lsn0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 1 );
	ASSERT_TRUE( _ait->appType( ) == 0x09 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 4 );
}

TEST_F( AitTest, ait_singleapp_application_name_descriptor ) {
	probeSection( "ait_single_app_ver1_sec0_lsn0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 4 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications( )[0].descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc.end() );

	tuner::ait::ApplicationNameDescriptor appNameDesc = (*it)
			.second.get<tuner::ait::ApplicationNameDescriptor > ();
	const tuner::ait::ApplicationName &appName = appNameDesc[0];
	ASSERT_TRUE( appName.name == "COCINEROS" );
}

TEST_F( AitTest, ait_singleapp_ginga_application_descriptor ) {
	probeSection( "ait_single_app_ver1_sec0_lsn0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 4 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications()[0].descriptors;
	
	tuner::desc::MapOfDescriptors::const_iterator it;

	tuner::ait::GingaApplicationLocationDescriptor gingaLoc;
	it = desc.find( AIT_GINGA_NCL_APP_LOCATION_DESC );
	ASSERT_TRUE( it != desc.end() );

	gingaLoc = (*it).second.get<tuner::ait::GingaApplicationLocationDescriptor > ();
	ASSERT_TRUE( gingaLoc.base == "/" );
	ASSERT_TRUE( gingaLoc.classPath == "" );
	ASSERT_TRUE( gingaLoc.initialClass == "main.ncl" );
}

TEST_F( AitTest, ait_singleapp_application_descriptor ) {
	probeSection( "ait_single_app_ver1_sec0_lsn0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );

	const tuner::ait::Application &app0 = _ait->applications()[0];
	ASSERT_TRUE( app0.descriptors.size( ) == 4 );
	const tuner::desc::MapOfDescriptors &desc = app0.descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	tuner::ait::ApplicationDescriptor appDesc;

	it = desc.find( AIT_APP_DESC );
	appDesc = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	ASSERT_TRUE( it != desc.end() );

	testFirstApplicationProfile( appDesc.profiles, 1, 1, 0, 0 );
	ASSERT_FALSE( HasFatalFailure() );

	ASSERT_TRUE( appDesc.serviceBoundFlag == 1 );
	ASSERT_TRUE( appDesc.visibility == 3 );
	ASSERT_TRUE( appDesc.priority == 1 );
	ASSERT_TRUE( appDesc.transports.size( ) == 1 );
	ASSERT_TRUE( appDesc.transports[0] == 0 );
}

TEST_F( AitTest, ait_singleapp_protocol_descriptors ) {
	probeSection( "ait_single_app_ver1_sec0_lsn0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );

	const tuner::ait::Application &app0 = _ait->applications()[0];
	ASSERT_TRUE( app0.descriptors.size( ) == 4 );

	const tuner::desc::MapOfDescriptors &desc = app0.descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	it = desc.find( AIT_TRANSPORT_PROTOCOL_DESC );
	ASSERT_TRUE( it != desc.end() );

	tuner::ait::TransportDescriptor protocols;
	protocols = (*it).second.get<tuner::ait::TransportDescriptor > ();

	ASSERT_TRUE( protocols.size( ) == 1 );
	ASSERT_TRUE( protocols[0].protocolID == 1 );
	ASSERT_TRUE( protocols[0].label == 0 );

	const tuner::ait::DSMCCTransportProtocolDescriptor &dsmccDesc = protocols[0]
			.info.get<tuner::ait::DSMCCTransportProtocolDescriptor > ();
	ASSERT_TRUE( dsmccDesc.remote == 0 );
	//ASSERT_TRUE( dsmccDesc.networkID == ? ? );
	//ASSERT_TRUE( dsmccDesc.tsID == ? ? );
	//ASSERT_TRUE( dsmccDesc.serviceID == ? ? );
	ASSERT_TRUE( dsmccDesc.componentTag == 0x0C );
}

TEST_F( AitTest, ait_singleapp_application_descriptor_limits_0 ) {
	probeSection( "ait_app_desc_limits_0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 1 );

	const tuner::ait::Application &app0 = _ait->applications()[0];
	const tuner::desc::MapOfDescriptors &desc = app0.descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	tuner::ait::ApplicationDescriptor appDesc;
	it = desc.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc.end() );

	appDesc = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	testFirstApplicationProfile( appDesc.profiles, 0xFFFF, 0xFF, 0xFF, 0xFF );
	ASSERT_FALSE( HasFatalFailure() );
	
	ASSERT_TRUE( appDesc.serviceBoundFlag == 1 );
	ASSERT_TRUE( appDesc.visibility == 3 );
	ASSERT_TRUE( appDesc.priority == 0xFF );
	ASSERT_TRUE( appDesc.transports.size( ) == 7 );
	ASSERT_TRUE( appDesc.transports[0] == 0 );
	ASSERT_TRUE( appDesc.transports[1] == 1 );
	ASSERT_TRUE( appDesc.transports[2] == 2 );
	ASSERT_TRUE( appDesc.transports[3] == 0x0F );
	ASSERT_TRUE( appDesc.transports[4] == 0xAA );
	ASSERT_TRUE( appDesc.transports[5] == 0xFF );
	ASSERT_TRUE( appDesc.transports[6] == 0 );
}

TEST_F( AitTest, ait_application_descriptor_limits_1 ) {
	probeSection( "ait_app_desc_limits_1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 2 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 1 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications( )[0].descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	tuner::ait::ApplicationDescriptor appDesc;

	it = desc.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc.end() );
	appDesc = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	testFirstApplicationProfile( appDesc.profiles, 0x0180, 0, 0xF0, 0xFF );
	ASSERT_FALSE( HasFatalFailure() );

	ASSERT_TRUE( appDesc.serviceBoundFlag == 0 );
	ASSERT_TRUE( appDesc.visibility == 0 );
	ASSERT_TRUE( appDesc.priority == 0xF0 );
	ASSERT_TRUE( appDesc.transports.size( ) == 5 );
	ASSERT_TRUE( appDesc.transports[0] == 0xAA );
	ASSERT_TRUE( appDesc.transports[1] == 0xFF );
	ASSERT_TRUE( appDesc.transports[2] == 0xFF );
	ASSERT_TRUE( appDesc.transports[3] == 0xF0 );
	ASSERT_TRUE( appDesc.transports[4] == 0x0F );
}

TEST_F( AitTest, ait_application_descriptor_limits_2 ) {
	probeSection( "ait_app_desc_limits_2.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 2 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 1 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications( )[0].descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	tuner::ait::ApplicationDescriptor appDesc;

	it = desc.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc.end() );
	appDesc = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	testFirstApplicationProfile( appDesc.profiles, 0x0180, 0, 0xF0, 0xFF );
	ASSERT_FALSE( HasFatalFailure() );
	
	ASSERT_TRUE( appDesc.serviceBoundFlag == 0 );
	ASSERT_TRUE( appDesc.visibility == 0 );
	ASSERT_TRUE( appDesc.priority == 0xF0 );
	ASSERT_TRUE( appDesc.transports.size( ) == 240 );

	for ( int i = 0; i < 240; i++ )
		ASSERT_TRUE( appDesc.transports[i] == i );
}

TEST_F( AitTest, ait_long_name_descriptor ) {
	probeSection( "ait_long_name_descriptor.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 1 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 4 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications( )[0].descriptors;
	ASSERT_TRUE( desc.size( ) == 4 );
	tuner::desc::MapOfDescriptors::const_iterator it;

	tuner::ait::ApplicationNameDescriptor appNameDesc;
	it = desc.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc.end() );
	appNameDesc = (*it).second.get<tuner::ait::ApplicationNameDescriptor > ();
	const tuner::ait::ApplicationName &appName = appNameDesc[0];
	ASSERT_TRUE( appName.name == "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJK"
			"LMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRS"
			"TUVWXYZ1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopq"
			"rstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmnoprst" );

	tuner::ait::GingaApplicationLocationDescriptor gingaLoc;
	it = desc.find( AIT_GINGA_NCL_APP_LOCATION_DESC );
	ASSERT_TRUE( it != desc.end() );

	gingaLoc = (*it).second.get<tuner::ait::GingaApplicationLocationDescriptor > ();
	ASSERT_TRUE( gingaLoc.base == "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJ"
			"KLMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQR"
			"STUVWXYZ1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnop"
			"qrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmnoprst" );

	ASSERT_TRUE( gingaLoc.classPath == "1234567890abcdefghijklmnopqrstuvwxyzABCDE"
			"FGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLM"
			"NOPQRSTUVWXYZ1234567890abcdef" );

	ASSERT_TRUE( gingaLoc.initialClass == "1234567890abcdefghijklmnopqrstuvwxyzAB"
			"CDEFGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJ"
			"KLMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefgh"
			"ijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmno" );
}

TEST_F( AitTest, ait_singleapp_application_limits ) {
	probeSection( "ait_orgid_appid_ccode_limit.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );

	const tuner::ait::Application app = _ait->applications( )[0];
	ASSERT_TRUE( app.orgID == 0xFFFFFFFF );
	ASSERT_TRUE( app.id == 0xFF );
	ASSERT_TRUE( app.ctrlCode == 0xFF );
}

TEST_F( AitTest, ait_singleapp_values1fff1fff ) {
	probeSection( "ait_orgid_1fff1fff.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 0 );
	const tuner::ait::Application app = _ait->applications( )[0];
	ASSERT_TRUE( app.orgID == 0x1FFF1FFF );
}

TEST_F( AitTest, ait_singleapp_applicationid_wildcard_fffe ) {
	probeSection( "ait_appID_wildcard_fffe.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 0 );
	const tuner::ait::Application app = _ait->applications( )[0];
	ASSERT_TRUE( app.id == 0xFFFE );
}

TEST_F( AitTest, ait_singleapp_applicationID_wildcard_ffff ) {
	probeSection( "ait_appID_wildcard_ffff.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 0 );
	const tuner::ait::Application app = _ait->applications( )[0];
	ASSERT_TRUE( app.id == 0xFFFF );
}

TEST_F( AitTest, ait_singleapp_applicationID_reserved ) {
	probeSection( "ait_appID_reserved_fffd.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->descriptors( ).size( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	ASSERT_TRUE( _ait->applications( )[0].descriptors.size( ) == 0 );
	const tuner::ait::Application app = _ait->applications( )[0];
	ASSERT_TRUE( app.id == 0xFFFD );
}

TEST_F( AitTest, ait_change_version ) {
	probeSection( "ait_ver0_cni1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 0 );

	probeSection( "ait_ver0_cni1.sec" );
	ASSERT_FALSE( _ait );
	ASSERT_TRUE( wasExpired( ) == 0 );

	probeSection( "ait_ver1_cni1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 1 );
	ASSERT_TRUE( _ait->version( ) == 1 );

	probeSection( "ait_ver1_cni1.sec" );
	ASSERT_FALSE( _ait );
	ASSERT_TRUE( wasExpired( ) == 0 );
}

TEST_F( AitTest, ait_change_version_cni ) {
	probeSection( "ait_ver0_cni1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 0 );

	probeSection( "ait_ver1_cni0.sec" );
	ASSERT_FALSE( _ait );
	ASSERT_TRUE( wasExpired( ) == 1 );

	probeSection( "ait_ver1_cni1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 1 );
}

TEST_F( AitTest, ait_multi_app1 ) {
	probeSection( "ait_multiple_app_10_11_12_ver1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 1 );

	// should be 3 application within this section
	ASSERT_TRUE( _ait->applications( ).size( ) == 3 );
	ASSERT_TRUE( _ait->applications( )[0].id == 0x10 );
	ASSERT_TRUE( _ait->applications( )[1].id == 0x11 );
	ASSERT_TRUE( _ait->applications( )[2].id == 0x12 );
	tuner::desc::MapOfDescriptors::const_iterator it;

	//	Application 0
	const tuner::desc::MapOfDescriptors desc0 = _ait->applications( )[0].descriptors;
	ASSERT_TRUE( desc0.size( ) == 4 );
	
	it = desc0.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc0.end( ) );
	tuner::ait::ApplicationNameDescriptor appNameDesc = (*it).second.get<tuner::ait::ApplicationNameDescriptor > ();
	ASSERT_TRUE( appNameDesc.size() == 1 );
	const tuner::ait::ApplicationName &appName = appNameDesc[0];
	ASSERT_TRUE( appName.name == "app_a" );

	it = desc0.find( AIT_TRANSPORT_PROTOCOL_DESC );
	ASSERT_TRUE( it != desc0.end( ) );
	tuner::ait::TransportDescriptor transports = (*it).second.get<tuner::ait::TransportDescriptor > ();
	ASSERT_TRUE( transports.size( ) == 1 );
	tuner::ait::TransportProtocolDescriptor transport = transports[0];
	ASSERT_TRUE( transport.protocolID == 0x01 );
	ASSERT_TRUE( transport.label == 0x00 );

	const tuner::ait::DSMCCTransportProtocolDescriptor &dsmccDesc = transport.info.get<tuner::ait::DSMCCTransportProtocolDescriptor > ();
	ASSERT_TRUE( dsmccDesc.remote == 0 ); 
	ASSERT_TRUE( dsmccDesc.componentTag == 0x0C ); 

	it = desc0.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc0.end( ) );
	tuner::ait::ApplicationDescriptor application = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	ASSERT_TRUE( application.visibility == 3 );
	ASSERT_TRUE( application.priority == 1 );
	ASSERT_TRUE( application.serviceBoundFlag == 1 );

	testFirstApplicationProfile( application.profiles, 1, 1, 0, 0 );
	ASSERT_FALSE( HasFatalFailure() );
	ASSERT_TRUE( application.transports.size( ) == 1 );
	ASSERT_TRUE( application.transports[0] == 0 );

	//	Application 1
	const tuner::desc::MapOfDescriptors desc1 = _ait->applications( )[1].descriptors;
	ASSERT_TRUE( desc1.size( ) == 4 );

	it = desc1.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc1.end( ) );
	appNameDesc = (*it).second.get<tuner::ait::ApplicationNameDescriptor > ();
	ASSERT_TRUE( appNameDesc.size() == 1 );
	const tuner::ait::ApplicationName &appName0 = appNameDesc[0];
	ASSERT_TRUE( appName0.name == "app_b" );

	it = desc1.find( AIT_TRANSPORT_PROTOCOL_DESC );
	ASSERT_TRUE( it != desc1.end( ) );
	transports = (*it).second.get<tuner::ait::TransportDescriptor > ();
	ASSERT_TRUE( transports.size() == 1 );
	transport = transports[0];

	ASSERT_TRUE( transport.protocolID == 0x01 );
	ASSERT_TRUE( transport.label == 0x00 );
	tuner::ait::DSMCCTransportProtocolDescriptor dsmcc0 = transport.info.get<tuner::ait::DSMCCTransportProtocolDescriptor > ();
	ASSERT_TRUE( dsmcc0.remote == 0 );
	ASSERT_TRUE( dsmcc0.componentTag == 0x0C );

	it = desc1.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc1.end( ) );
	application = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	ASSERT_TRUE( application.visibility == 3 );
	ASSERT_TRUE( application.priority == 1 );
	ASSERT_TRUE( application.serviceBoundFlag == 1 );
	testFirstApplicationProfile( application.profiles, 1, 1, 0, 0 );
	ASSERT_FALSE( HasFatalFailure() );
	ASSERT_TRUE( application.transports.size( ) == 1 );
	ASSERT_TRUE( application.transports[0] == 0 );

	//	Application 2
	const tuner::desc::MapOfDescriptors desc2 = _ait->applications( )[2].descriptors;
	ASSERT_TRUE( desc2.size( ) == 4 );

	it = desc2.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc2.end( ) );
	appNameDesc = (*it).second.get<tuner::ait::ApplicationNameDescriptor > ();
	ASSERT_TRUE( appNameDesc.size() == 1 );
	ASSERT_TRUE( appNameDesc[0].name == "app_c" );

	it = desc2.find( AIT_TRANSPORT_PROTOCOL_DESC );
	ASSERT_TRUE( it != desc2.end( ) );
	transports = (*it).second.get<tuner::ait::TransportDescriptor > ();
	ASSERT_TRUE( transports.size( ) == 1 );
	transport = transports[0];

	tuner::ait::DSMCCTransportProtocolDescriptor dsmcc = transport.info.get<tuner::ait::DSMCCTransportProtocolDescriptor > ();
	ASSERT_TRUE( transport.protocolID == 0x01 );
	ASSERT_TRUE( transport.label == 0x00 );
	ASSERT_TRUE( dsmcc.remote == 0 );
	ASSERT_TRUE( dsmcc.componentTag == 0x0C );

	it = desc2.find( AIT_APP_DESC );
	ASSERT_TRUE( it != desc2.end( ) );
	application = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	ASSERT_TRUE( application.visibility == 3 );
	ASSERT_TRUE( application.priority == 1 );
	ASSERT_TRUE( application.serviceBoundFlag == 1 );
	testFirstApplicationProfile( application.profiles, 1, 1, 0, 0 );
	ASSERT_FALSE( HasFatalFailure() );

	ASSERT_TRUE( application.transports.size( ) == 1 );
	ASSERT_TRUE( application.transports[0] == 0 );
}

void testApplicationNameDescriptor( const tuner::desc::MapOfDescriptors desc, const std::string name ) {
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_APP_NAME_DESC );
	ASSERT_TRUE( it != desc.end( ) );
	tuner::ait::ApplicationNameDescriptor appNameDesc = (*it).second
			.get<tuner::ait::ApplicationNameDescriptor > ();
	ASSERT_TRUE( appNameDesc.size() == 1 );
	ASSERT_TRUE( appNameDesc[0].name == name );
}

TEST_F( AitTest, ait_multi_app2 ) {
	probeSection( "ait_multiple_app_10_12_13_ver2.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->version( ) == 2 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 3 );
	ASSERT_TRUE( _ait->applications( )[0].id == 0x10 );
	ASSERT_TRUE( _ait->applications( )[1].id == 0x12 );
	ASSERT_TRUE( _ait->applications( )[2].id == 0x13 );

	const tuner::desc::MapOfDescriptors &desc0 = _ait->applications( )[0].descriptors;
	const tuner::desc::MapOfDescriptors &desc1 = _ait->applications( )[1].descriptors;
	const tuner::desc::MapOfDescriptors &desc2 = _ait->applications( )[2].descriptors;

	testApplicationNameDescriptor( desc0, "app_a" );
	testApplicationNameDescriptor( desc1, "app_c" );
	testApplicationNameDescriptor( desc2, "app_d" );
}

TEST_F( AitTest, ait_multi_app_change ) {
	probeSection( "ait_multiple_app_10_11_12_ver1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired() == 0 );
	ASSERT_TRUE( _ait->version() == 1 );

	probeSection( "ait_multiple_app_10_12_13_ver2.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired() != 0 ); //	it should be expired
	ASSERT_TRUE( _ait->version() == 2 );
	ASSERT_TRUE( _ait->applications().size( ) == 3 );
	ASSERT_TRUE( _ait->applications()[0].id == 0x10 );
	ASSERT_TRUE( _ait->applications()[1].id == 0x12 );
	ASSERT_TRUE( _ait->applications()[2].id == 0x13 );

	const tuner::desc::MapOfDescriptors &desc0 = _ait->applications()[0].descriptors;
	const tuner::desc::MapOfDescriptors &desc1 = _ait->applications()[1].descriptors;
	const tuner::desc::MapOfDescriptors &desc2 = _ait->applications()[2].descriptors;

	testApplicationNameDescriptor( desc0, "app_a" );
	testApplicationNameDescriptor( desc1, "app_c" );
	testApplicationNameDescriptor( desc2, "app_d" );
}

TEST_F( AitTest, ait_repeat ) {
	probeSection( "ait_multiple_app_10_11_12_ver1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) == 0 );

	probeSection( "ait_multiple_app_10_11_12_ver1.sec" );
	ASSERT_FALSE( _ait );
	ASSERT_TRUE( wasExpired( ) == 0 );
}

TEST_F( AitTest, ait_version_change ) {
	probeSection( "ait_multiple_app_10_11_12_ver1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_FALSE( wasExpired( ) );
	ASSERT_TRUE( _ait->version( ) == 1 );

	probeSection( "ait_multiple_app_10_12_13_ver2.sec" );
	ASSERT_TRUE( wasExpired( ) != 0 );
	ASSERT_TRUE( _ait->version( ) == 2 );
}

void testBoundFlag( const tuner::desc::MapOfDescriptors desc, const bool flag ) {
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_APP_DESC );
	tuner::ait::ApplicationDescriptor app = (*it).second.get<tuner::ait::ApplicationDescriptor > ();
	ASSERT_TRUE( it != desc.end( ) );
	ASSERT_TRUE( app.serviceBoundFlag == flag );
}

TEST_F( AitTest, ait_external_application ) {
	probeSection( "ait_ext_app_ver0.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( _ait->version( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 2 );
	ASSERT_TRUE( _ait->applications( )[0].id == 0x10 );
	ASSERT_TRUE( _ait->applications( )[1].id == 0x11 );
	ASSERT_TRUE( _ait->applications( )[0].orgID == 0xa );
	ASSERT_TRUE( _ait->applications( )[1].orgID == 0xb );

	const tuner::desc::MapOfDescriptors &desc0 = _ait->applications()[0].descriptors;
	const tuner::desc::MapOfDescriptors &desc1 = _ait->applications()[1].descriptors;

	testApplicationNameDescriptor( desc0, "app_10" );
	testApplicationNameDescriptor( desc1, "app_11" );
	testBoundFlag( desc0, 1 );
	testBoundFlag( desc1, 0 );

	probeSection( "ait_ext_app_ver1.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( wasExpired( ) != 0 );
	ASSERT_TRUE( _ait->version( ) == 1 );

	const tuner::desc::MapOfDescriptors &desc = _ait->descriptors( );
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_EXTERNAL_APP_AUTH_DESC );
	ASSERT_TRUE( it != desc.end( ) );
	tuner::ait::ExternalApplications apps = (*it).second.get<tuner::ait::ExternalApplications > ();
	ASSERT_TRUE( apps.size( ) == 1 );
	ASSERT_TRUE( apps[0].orgID == 0xB );
	ASSERT_TRUE( apps[0].id == 0x11 );
	ASSERT_TRUE( apps[0].priority == 0x10 );
}

TEST_F( AitTest, ait_transport_protocol_in_common_loop ) {
	probeSection( "ait_common_loop_02.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_FALSE( wasExpired( ) );
	ASSERT_TRUE( _ait->version( ) == 1 );

	const tuner::desc::MapOfDescriptors &desc = _ait->descriptors( );
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_TRANSPORT_PROTOCOL_DESC );
	ASSERT_TRUE( it != desc.end( ) );

	tuner::ait::TransportDescriptor transports = (*it).second.get<tuner::ait::TransportDescriptor > ();
	ASSERT_TRUE( transports.size( ) == 1 );
	tuner::ait::TransportProtocolDescriptor transport = transports[0];

	ASSERT_TRUE( transport.label == 0x03 );
	ASSERT_TRUE( transport.protocolID == 0x02 );
}

void testTransportProtocol( tuner::ait::TransportProtocolDescriptor desc,
		const util::WORD id, const util::BYTE label ) {
	ASSERT_TRUE( desc.protocolID == id );
	ASSERT_TRUE( desc.label == label );
}

TEST_F( AitTest, ait_several_transport_protocols ) {
	probeSection( "ait_several_transport_protocol.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( _ait->version( ) == 2 );

	const tuner::desc::MapOfDescriptors &desc = _ait->applications( )[0].descriptors;
	tuner::desc::MapOfDescriptors::const_iterator it;
	it = desc.find( AIT_TRANSPORT_PROTOCOL_DESC );
	tuner::ait::TransportDescriptor protocols;

	ASSERT_TRUE( it != desc.end( ) );
	protocols = (*it).second.get<tuner::ait::TransportDescriptor > ();

	ASSERT_TRUE( protocols.size( ) == 8 );
	testTransportProtocol( protocols[0], 1, 0 );
	testTransportProtocol( protocols[1], 1, 1 );
	testTransportProtocol( protocols[2], 1, 2 );
	testTransportProtocol( protocols[3], 1, 0xA );
	testTransportProtocol( protocols[4], 1, 0xF );
	testTransportProtocol( protocols[5], 1, 0xFA );
	testTransportProtocol( protocols[6], 1, 0xFE );
	testTransportProtocol( protocols[7], 1, 0xFF );
}

TEST_F( AitTest, ait_big_common_desc_loop ) {
	probeSection( "ait_big_common_desc_loop.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( _ait->version( ) == 1 );
	ASSERT_TRUE( wasExpired( ) == 0 );
}

void testGingaLocationDescriptor( const tuner::desc::MapOfDescriptors desc, const std::string base,
		const std::string classPath, const std::string initialClass ) {
	tuner::desc::MapOfDescriptors::const_iterator it = desc.find( AIT_GINGA_NCL_APP_LOCATION_DESC );
	ASSERT_TRUE( it != desc.end( ) );
	tuner::ait::GingaApplicationLocationDescriptor appLoc = (*it).second
			.get<tuner::ait::GingaApplicationLocationDescriptor > ();
	ASSERT_TRUE( appLoc.base == base );
	ASSERT_TRUE( appLoc.classPath == classPath );
	ASSERT_TRUE( appLoc.initialClass == initialClass );
}

TEST_F( AitTest, ait_empty_names_and_location ) {
	probeSection( "ait_empty_names_and_location.sec" );
	ASSERT_TRUE( _ait != NULL );
	ASSERT_TRUE( _ait->version( ) == 1 );
	ASSERT_TRUE( wasExpired( ) == 0 );
	ASSERT_TRUE( _ait->applications( ).size( ) == 1 );
	
	const tuner::desc::MapOfDescriptors &desc0 = _ait->applications( )[0].descriptors;
	testApplicationNameDescriptor( desc0, "" );
	testGingaLocationDescriptor( desc0 , "", "", "" );
}

TEST_F( AitTest, ait_syntx_ind_0_crc32_error ) { 
	probeSection( "ait_ver0_cni1_synt_crc.sec" );
	ASSERT_FALSE( _ait );
}

TEST_F( AitTest, ait_syntx_ind0 ) {
	probeSection( "ait_ver0_cni1_synt.sec" );
	ASSERT_FALSE( _ait );
}

TEST_F( AitTest, ait_err ) {
	probeSection( "ait_err.sec" );
	ASSERT_FALSE( _ait );
}

TEST_F( AitTest, ait_crc_error ) {
	probeSection( "ait_ver0_cni1_crc.sec" );
	ASSERT_FALSE( _ait );
}

TEST_F( AitTest, ait_invalid_tableid ) {
	probeSection( "ait_invalid_tid.sec" );
	ASSERT_FALSE( _ait );
}
