/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementacin de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los trminos de la Licencia Pblica General Reducida GNU como es publicada por la
  Free Software Foundation, segn la versin 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTA; ni siquiera la garanta implcita de COMERCIALIZACIN o ADECUACIN
  PARA ALGN PROPSITO PARTICULAR. Para ms detalles, revise la Licencia Pblica
  General Reducida GNU.

    Ud. debera haber recibido una copia de la Licencia Pblica General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "../../../src/types.h"
#include "../../../src/demuxer/pes/pesdemuxer.h"
#include "../../../src/demuxer/pes/caption/captiondemuxer.h"
#include "../util.h"
#include <util/buffer.h>
#include <gtest/gtest.h>
#include <boost/filesystem.hpp>

namespace fs = boost::filesystem;

class PESTest : public testing::Test {
public:
	PESTest( void ) {}
	virtual ~PESTest( void ) {}

	virtual void SetUp() { _demux = NULL; }	
	virtual void TearDown() { delete _demux; }

protected:
	bool probe( const char *file ) {
		fs::path rootPath(test::getTestRoot());
		rootPath /= "ts";
		rootPath /= file;

		FILE *f = fopen( rootPath.string().c_str(),"r+b");
		if (f) {
			util::Buffer tmp( 1024 );
			int bytes = ::fread( tmp.buffer(), 1, tmp.capacity(), f );
			while (bytes) {
				tmp.resize( bytes );
				// printf( "[test] Parse: bytes=%d, data=%s\n",
				// 	bytes, tmp.asHexa().c_str() );
				_demux->parse( (util::BYTE *)tmp.buffer(), bytes );
				bytes = ::fread( tmp.buffer(), 1, tmp.capacity(), f );
			}
			fclose( f );
			return true;
		}
		return false;
	}

	tuner::PESDemuxer *_demux;

private:
};

void onManagement( tuner::arib::Management *m, bool *check ) {
	*check = (m->pts() == 0x4706f23f);
}

TEST_F( PESTest, caption_pes ) {
	tuner::arib::CaptionDemuxer *caption = new tuner::arib::CaptionDemuxer();
	bool result=false;
	caption->onManagement( boost::bind(&onManagement,_1,&result) );
	_demux = caption;
	ASSERT_TRUE( probe( "caption.pes" ) );
	ASSERT_TRUE( result );
}

TEST_F( PESTest, video_pes ) {
	_demux = new tuner::PESDemuxer( tuner::pes::video );
	ASSERT_TRUE( probe( "video.pes" ) );
}

TEST_F( PESTest, audio_pes ) {
	_demux = new tuner::PESDemuxer( tuner::pes::audio );
	ASSERT_TRUE( probe( "audio.pes" ) );
}
