/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "provider.h"
#include "../util.h"
#include "../../../src/provider/filter.h"
#include "../../../src/provider/provider.h"
#include "../../../src/demuxer/psi/patdemuxer.h"
#include <util/buffer.h>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>


OkProvider::OkProvider( void )
{
	stopCount = 0;
	_myDisp = new util::task::DispatcherImpl();
	setDispatcher( _myDisp );
}

OkProvider::~OkProvider()
{
	delete _myDisp;
}

void showStatus( const ReadyStatus &st ) {
	printf( "[ReadyStatus] status: size=%d\n", st.size() );
	BOOST_FOREACH( bool elem, st ) {
		printf( "[ReadStatus]\t elem=%d\n", elem );
	}
}

TEST( Provider, default_constructor ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, start ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );	
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, start_with_engine_fail ) {
	EngineFailProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );	
	ASSERT_FALSE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::fail );
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, set_network_without_start ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_FALSE( prov.setNetwork( 1 ) );
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, set_network_ok ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	ASSERT_TRUE( prov.stopCount == 0 );
	ASSERT_TRUE( prov.setNetwork( 1 ) );
	ASSERT_TRUE( prov.state() > tuner::state::idle );
	prov.stop();
	ASSERT_TRUE( prov.stopCount == 1 );	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, set_network_fail ) {
	StartNetworkFailProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	ASSERT_FALSE( prov.setNetwork( 1 ) );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, first_scan_ok ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.stopCount == 0 );	
	ASSERT_TRUE( prov.scan( 0 ) );
	ASSERT_TRUE( prov.state() == tuner::state::scanning );
	ASSERT_TRUE( prov.stopCount == 0 );
	
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, first_scan_after_set ) {
	OkProvider prov;
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	ASSERT_TRUE( prov.stopCount == 0 );	
	ASSERT_TRUE( prov.setNetwork( 1 ) );
	ASSERT_TRUE( prov.state() >= tuner::state::tunning );
	ASSERT_TRUE( prov.stopCount == 0 );	

	ASSERT_TRUE( prov.scan( 0 ) );
	ASSERT_TRUE( prov.state() == tuner::state::scanning );
	ASSERT_TRUE( prov.stopCount == 1 );
	
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, next_scan_ok ) {
	ScanProvider prov(4);
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.stopCount == 0 );

	size_t net = 0;
	while (net < 4) {
		ASSERT_TRUE( prov.scan( net ) );
		ASSERT_TRUE( prov.state() == tuner::state::scanning );
		ASSERT_TRUE( prov.stopCount == net );
		net++;
	}

	ASSERT_FALSE( prov.scan( net ) );
	ASSERT_TRUE( prov.state() == tuner::state::idle );
	ASSERT_TRUE( prov.stopCount == 4 );
	
	prov.stop();
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, start_section_filter_ok ) {
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	tuner::PSIDemuxer *demux = new tuner::PATDemuxer( 0 );
	ASSERT_TRUE( prov.startFilter( demux ) );
	prov.stopFilter( 0 );

	//	TODO: Check initialize,start was called?
	
	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, start_section_filter_create_fail ) {
	CreateFailProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	tuner::PSIDemuxer *demux = new tuner::PATDemuxer( 0 );
	ASSERT_FALSE( prov.startFilter( demux ) );

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, start_section_filter_check_max ) {
	MaxFilterProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	tuner::PSIDemuxer *demux = new tuner::PATDemuxer( 0 );
	ASSERT_TRUE( prov.startFilter( demux ) );

	tuner::PSIDemuxer *demux1 = new tuner::PATDemuxer( 1 );
	ASSERT_FALSE( prov.startFilter( demux1 ) );	

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, start_section_filter_check_same_pid ) {
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	tuner::PSIDemuxer *demux = new tuner::PATDemuxer( 0 );
	ASSERT_TRUE( prov.startFilter( demux ) );

	tuner::PSIDemuxer *demux1 = new tuner::PATDemuxer( 0 );
	ASSERT_FALSE( prov.startFilter( demux1 ) );	

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, start_section_filter_init_fail ) {
	FilterProvider<InitFailFilter> prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	tuner::PSIDemuxer *demux = new tuner::PATDemuxer( 0 );
	ASSERT_FALSE( prov.startFilter( demux ) );

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, start_section_filter_start_fail ) {
	FilterProvider<StartFailFilter> prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_FALSE( prov.startFilter( new tuner::PATDemuxer( 0 ) ) );

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );	
}

TEST( Provider, probe_section_filter_when_idle ) {//TODO
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	TestDemuxer *demux=new TestDemuxer(0);
	ASSERT_TRUE( prov.startFilter( demux ) );

	prov.probeSection( 0, "pat_nit_ver0_cni1.sec" );
	prov.stopFilter( 0 );
	prov.stop();

	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, probe_section_filter_ok ) {
	NotifyProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.setNetwork( 0 ) );
	ASSERT_TRUE( prov.state() > tuner::state::idle );

	TestDemuxer *demux=new TestDemuxer(0);
	ASSERT_TRUE( prov.startFilter( demux ) );
	prov.probeSection( 0, "pat_nit_ver0_cni1.sec" );	

	ASSERT_TRUE( demux->started == 1 );
	ASSERT_TRUE( demux->pushed == 0 );	

	prov.stopFilter( 0 );
	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, enqueue_without_filter ) {
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start( 10 ) );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.setNetwork( 0 ) );
	ASSERT_TRUE( prov.state() > tuner::state::idle );

	prov.probeSection( 0, "pat_nit_ver0_cni1.sec" );	
	
	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, double_stop_filter ) {
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.setNetwork( 0 ) );
	ASSERT_TRUE( prov.state() > tuner::state::idle );

	TestDemuxer *demux = new TestDemuxer( 0 );
	ASSERT_TRUE( prov.startFilter( demux ) );
	
	prov.stopFilter( 0 );
	prov.stopFilter( 0 );

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

TEST( Provider, stop_invalid_filter ) {
	OkProvider prov;
	
	ASSERT_TRUE( prov.state() == tuner::state::off );
	ASSERT_TRUE( prov.start() );
	ASSERT_TRUE( prov.state() == tuner::state::idle );

	ASSERT_TRUE( prov.setNetwork( 0 ) );
	ASSERT_TRUE( prov.state() > tuner::state::idle );

	prov.stopFilter( 0 );

	prov.stop();	
	ASSERT_TRUE( prov.state() == tuner::state::off );
}

