/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#include "../../../src/service/service.h"
#include "../../../src/demuxer/psi/psi.h"
#include "../../../src/demuxer/psi/pmt.h"
#include <gtest/gtest.h>

TEST( Service, custom_constructor ) {
	tuner::Service ser( 8, 1, 1, 10, 11 );

	ASSERT_TRUE( ser.networkID() == 8 );
	ASSERT_TRUE( ser.nitID() == 1 );
	ASSERT_TRUE( ser.tsID() == 1 );
	ASSERT_TRUE( ser.id() == 10 );
	ASSERT_TRUE( ser.pid() == 11 );
	ASSERT_TRUE( ser.pcrPID() == TS_PID_NULL );
	ASSERT_TRUE( ser.type() == tuner::service::type::digital_television );
	ASSERT_TRUE( ser.status() == tuner::service::status::undefined );
	ASSERT_TRUE( ser.state() == tuner::service::state::present );	
	ASSERT_TRUE( ser.provider().empty() );
	ASSERT_TRUE( ser.name().empty() );
	ASSERT_TRUE( ser.networkName().empty() );
	ASSERT_TRUE( ser.elements().empty() );
	ASSERT_TRUE( ser.remote() == 0 );
	ASSERT_TRUE( ser.tsName().empty() );
	ASSERT_TRUE( ser.logicalChannel() == "00.13" );	
}

TEST( Service, process_valid_pmt ) {
	tuner::ID pid = 20;
	tuner::ID programID = 5000;
	tuner::ID pcrPID = 10;
	tuner::desc::MapOfDescriptors desc;
	std::vector<tuner::ElementaryInfo> elems;
	tuner::Service ser( 8, 1, 1, programID, pid );

	tuner::ElementaryInfo info;
	elems.push_back( info );
	boost::shared_ptr<tuner::Pmt> pmt( new tuner::Pmt( pid, 0, programID, pcrPID, desc, elems ) );	
    ASSERT_TRUE( ser.process( pmt ) );

	ASSERT_TRUE( ser.networkID() == 8 );
	ASSERT_TRUE( ser.nitID() == 1 );	
	ASSERT_TRUE( ser.tsID() == 1 );
	ASSERT_TRUE( ser.id() == programID );
	ASSERT_TRUE( ser.pid() == pid );
	ASSERT_TRUE( ser.pcrPID() == pcrPID );
	ASSERT_TRUE( ser.type() == tuner::service::type::digital_television );
	ASSERT_TRUE( ser.state() == tuner::service::state::complete );		
	ASSERT_TRUE( ser.provider().empty() );
	ASSERT_TRUE( ser.name().empty() );
	ASSERT_TRUE( ser.networkName().empty() );
	ASSERT_TRUE( ser.elements().size() == 1 );
	ASSERT_TRUE( ser.remote() == 0 );
	ASSERT_TRUE( ser.tsName().empty() );
	ASSERT_TRUE( ser.logicalChannel() == "00.11" );	
}

TEST( Service, process_other_pmt ) {
	tuner::ID pid = 20;
	tuner::ID programID = 5000;
	tuner::ID pcrPID = 10;
	tuner::desc::MapOfDescriptors desc;
	std::vector<tuner::ElementaryInfo> elems;
	tuner::Service ser( 8, 1, 1, programID, pid );

	boost::shared_ptr<tuner::Pmt> pmt( new tuner::Pmt( 0, 0, 0, pcrPID, desc, elems ) );	
	ASSERT_FALSE( ser.process( pmt ) );

	ASSERT_TRUE( ser.networkID() == 8 );
	ASSERT_TRUE( ser.nitID() == 1 );	
	ASSERT_TRUE( ser.tsID() == 1 );
	ASSERT_TRUE( ser.pcrPID() == TS_PID_NULL );
	ASSERT_TRUE( ser.type() == tuner::service::type::digital_television );
	ASSERT_TRUE( ser.state() == tuner::service::state::present );		
	ASSERT_TRUE( ser.provider().empty() );
	ASSERT_TRUE( ser.name().empty() );
	ASSERT_TRUE( ser.networkName().empty() );
	ASSERT_TRUE( ser.elements().empty() );
	ASSERT_TRUE( ser.remote() == 0 );
	ASSERT_TRUE( ser.tsName().empty() );
	ASSERT_TRUE( ser.logicalChannel() == "00.11" );
}
