/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "log.h"
#include "log/api.h"
#include "cfg/configregistrator.h"
#include "cfg/propertynode.h"
#include "generated/config.h"
#include <boost/filesystem.hpp>

namespace fs = boost::filesystem;

#if LOG_USE_LOG
// LOG_LEVELS: none, error, warn, info, debug, trace, all
REGISTER_INIT_CONFIG( log ) {
	std::string bType(UTIL_BUILD_TYPE);
	std::string level = (bType=="Release") ? "warn" : "info";
	boost::filesystem::path path( fs::temp_directory_path() );
	path /= "tool.log";

	root().addNode("log")
		.addValue("enabled", "Logging is enabled/disabled", true);

	root()("log").addNode("filter").addNode("all")
		.addValue("all", "Level to log for all categories in this group", level );

	root()("log").addNode("output").addNode("stdout")
		.addValue("use", "Log to cout?", true );

	root()("log.output").addNode("file")
		.addValue("use", "Log to file?", false )
		.addValue("filename", "Full name of the file where the log will be directed", path.string());
}
#endif //LOG_USE_LOG

namespace util {
namespace log {

#if LOG_USE_LOG
static bool _logEnabled;

static void addOutputs() {
	const bool &useCout = util::cfg::getValue<bool>("log.output.stdout.use");
	const bool &useFile = util::cfg::getValue<bool>("log.output.file.use");
	if (useCout) {
		addStdout();
	}
	if (useFile) {
		const std::string &file = util::cfg::getValue<std::string>("log.output.file.filename");
		addLogFile(file);
	}
}

typedef std::pair<std::string, std::string> Category;
struct Visitor {
	std::map<std::string, std::string> _categories;
	void operator()( cfg::PropertyNode *p ) {
		p->visitValues(*this);
		BOOST_FOREACH(  Category c, _categories ) {
			setLevel(p->name().c_str(), c.first.c_str(), c.second.c_str());
		}
		_categories.clear();
	}
	void operator()( cfg::PropertyValue *v ) {
		_categories[v->name()] = v->get<std::string>();
	}
};
static void loadLevels() {
	Visitor nodeVisitor;

	cfg::PropertyNode &groups = cfg::get(std::string("log.filter"));
	groups.visitNodes(nodeVisitor);
}

void init() {
	_logEnabled = cfg::getValue<bool>("log.enabled");
	if (_logEnabled) {
		initialize( cfg::getValue<std::string>("log.filter.all.all") );
		addOutputs();
		loadLevels();
	}
}

void flush() {
	if (_logEnabled) {
		flushMessages();
	}
}

void fin() {
	if (_logEnabled) {
		finalize();
	}
}

#else //LOG_USE_LOG

void init() {}
void flush() {}
void fin() {}

#endif //LOG_USE_LOG

}
}
