/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "api.h"
#include "statics.h"
#include "message.h"
#include "../string.h"
#include "../cfg/configregistrator.h"
#include <stdarg.h>
#include <boost/thread/mutex.hpp>

namespace util {
namespace log {

static bool _logEnabled=false;

static LOG_LEVEL_TYPE getLevelValue( const std::string &levelstr ) {
	LOG_LEVEL_TYPE level;
	if (levelstr == "none") {
		level = LOG_LEVEL_NONE;
	} else if (levelstr == "error") {
		level = LOG_LEVEL_ERROR;
	} else if (levelstr == "warn") {
		level = LOG_LEVEL_WARN;
	} else if (levelstr == "info") {
		level = LOG_LEVEL_INFO;
	} else if (levelstr == "debug") {
		level = LOG_LEVEL_DEBUG;
	} else if (levelstr == "trace") {
		level = LOG_LEVEL_TRACE;
	} else if (levelstr == "all") {
		level = LOG_LEVEL_ALL;
	} else {
		throw std::runtime_error( "bad log severity name");
	}
	return level;
}

void initialize( const std::string &level ) {
	Statics::initStatics( getLevelValue(level) );
	_logEnabled = cfg::getValue<bool>("log.enabled");
}

void finalize( ) {
	_logEnabled = false;
	Statics::finStatics();
}

void flushMessages() {
	Statics::flushMessages();
}

void addStdout() {
	Statics::addTarget(new Cout());
}

void addLogFile( const std::string &filename ) {
	Statics::addTarget(new LogFile(filename));
}

void setLevel( const char *group, const char *category, const char *level ) {
	Statics::setLevel( group, category, getLevelValue(level) );
}

bool canLog( LOG_LEVEL_TYPE level, const char *group, const char *category ) {
	return _logEnabled && Statics::canLog(level, group, category);
}

void log( LOG_LEVEL_TYPE level, const char *group, const char *category, const char *format, ... ) {
	va_list args;
	va_start(args, format);
	std::string formatedMsg = format_va( format, args );
	va_end( args );

	Message msg(level, group, category, formatedMsg);
	Statics::transmitMessage(msg);
}

}
}
