/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "const.h"
#include "target.h"
#include <map>
#include <vector>
#include <string>
#include <boost/thread.hpp>

namespace util {
namespace log {

class Message;
class Formatter;

typedef std::map<std::string, LOG_LEVEL_TYPE> Categories;
typedef std::map<std::string, Categories> Groups;

class Statics {
public:
	static void initStatics( LOG_LEVEL_TYPE level );
	static void finStatics();
	static bool _initialized;

	static bool isLogEnabled();

	static bool canLog( LOG_LEVEL_TYPE level, const std::string &group, const std::string &category );
	static void transmitMessage( const Message &m );

	static Formatter *getFormatter();
	static void addTarget( Target *t );
	static void flushMessages();

	static void setLevel( const std::string &group, const std::string &category, const LOG_LEVEL_TYPE level );

private:
	Statics() {}
	~Statics() {}

	static boost::mutex _initializedMutex;

	static int _nTransmitting;
	static boost::mutex _nTransmittingMutex;
	static boost::condition_variable _nTransmittingCond;

	static Groups _groups;
	static std::vector<Target*> _outputs;
	
	static Formatter *_formatter;
	static LOG_LEVEL_TYPE _defaultLevel;
};

}
}
