/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "formatter.h"
#include "statics.h"
#include <cstdio>
#include <boost/thread.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>

namespace util {
namespace log {

Target::Target() {
	_formatter = Statics::getFormatter();
	_fin = false;
	_waitingFlush = false;
	_thread = new boost::thread(boost::bind( &Target::loggerThread, this ));
}

Target::~Target() {
}

void Target::receive( const Message &message ) {
	_linesMutex.lock();
	_lines.push(_formatter->format(message));
	_linesMutex.unlock();
}

void Target::flush() {
	boost::unique_lock<boost::mutex> lock(_flushMutex);
	_waitingFlush = true;
	_linesCond.notify_one();
	_flushCond.wait(lock);
	flushImpl();
	_waitingFlush = false;
}

void Target::flushImpl() {
}

void Target::loggerThread() {
	_finMutex.lock();
	bool fin = _fin;
	_finMutex.unlock();

	while (!fin) {
		std::vector<std::string> localLines;
		{
			boost::unique_lock<boost::mutex> lock(_linesMutex);
			if(_lines.size()<1) {
				_linesCond.timed_wait(lock, boost::posix_time::milliseconds(100));
			}
			while (_lines.size()>0) {
				localLines.push_back(_lines.front());
				_lines.pop();
			}
		}
		BOOST_FOREACH(std::string l, localLines) {
			output( l );
		}
		localLines.clear();

		_flushMutex.lock();
		if (_waitingFlush) {
			_flushCond.notify_one();
		}
		_flushMutex.unlock();

		_finMutex.lock();
		fin = _fin;
		_finMutex.unlock();
	}
	while (_lines.size()>0) {
		output(_lines.front());
		_lines.pop();
	}
}

void Target::fin() {
	_finMutex.lock();
	_fin = true;
	_finMutex.unlock();
	_linesCond.notify_one();

	_thread->join();
	delete _thread;
	_thread = NULL;
}

void Cout::output( const std::string &line ) {
	printf( "%s", line.c_str() );
}

LogFile::LogFile( const std::string &fileName ) {
	_file = NULL;
	_fileOpened = false;
	_fileName = fileName;
}

LogFile::~LogFile() {
	if (_fileOpened) {
		char cTimeString[100];
		time_t tRawTime;
		struct tm *tmInfo;

		time( &tRawTime );
		tmInfo = localtime( &tRawTime );
		strftime(cTimeString, 100, "%c", tmInfo);

		std::string msg("Log ended at: ");
		msg += cTimeString;
		msg += "\n";
		fwrite( msg.c_str(), sizeof( char ), msg.size(), _file );
		fclose( _file );
		_file = NULL;
	}
}

void LogFile::open() {
	_file = fopen( _fileName.c_str(), "w" );
	if (_file) {
		size_t result;

		char cTimeString[100];
		time_t tRawTime;
		struct tm *tmInfo;

		time( &tRawTime );
		tmInfo = localtime( &tRawTime );
		strftime(cTimeString, 100, "%c", tmInfo);
	
		std::string msg("Log started at: ");
		msg += cTimeString;
		msg += "\n";
		result = fwrite( msg.c_str(), sizeof( char ), msg.size(), _file );
		if (result>0) {
			_fileOpened = true;
		} else {
			fclose( _file );
			_file = NULL;
		}
	}
}

void LogFile::output( const std::string &line ) {
	if (!_fileOpened) {
		open();
	}
	if (_fileOpened) {
		fwrite( line.c_str(), sizeof( char ), line.size(), _file );
	}
}

void LogFile::flushImpl() {
	assert(_file);
	fflush(_file);
}

}
}
