/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include <string>
#include <queue>
#include <boost/thread.hpp>

namespace util {
namespace log {

class Formatter;
class Message;

class Target {
public :
	Target();
	virtual ~Target();

	void fin();
	void receive( const Message &message );
	void flush();

protected:
	void loggerThread();
	virtual void output( const std::string &line ) = 0;
	virtual void flushImpl();
	Formatter *_formatter;

	std::queue<std::string> _lines;
	boost::mutex _linesMutex;
	boost::condition_variable _linesCond;

	bool _waitingFlush;
	boost::mutex _flushMutex;
	boost::condition_variable _flushCond;
	
	bool _fin;
	boost::mutex _finMutex;
	
	boost::thread *_thread;
};

class Cout : public Target {
    virtual void output( const std::string &line );
};

class LogFile : public Target {
public:
	LogFile( const std::string &fileName );
	virtual ~LogFile();

	virtual void output( const std::string &line );

protected:
	void open();
	void flushImpl();

	std::string _fileName;
	bool _fileOpened;
	FILE *_file;

private:
	LogFile();
};

}
}
