/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "sockaddr.h"
#include <boost/static_assert.hpp>
#include <string.h>

#ifdef _WIN32
#	include <winsock2.h>
#	include <ws2tcpip.h>
#else
#	include <sys/socket.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <netdb.h>
#endif

#define ASSIGN_SA(sa) { struct sockaddr *tmp = (struct sockaddr *)_sa; *tmp = sa; }

namespace util {
namespace net {

bool SockAddr::isFamilySupported( family::type family ) {
	return family == family::ipv4 || family == family::ipv6;
}

SockAddr::SockAddr( void )
{
	BOOST_STATIC_ASSERT(sizeof(_sa) >= sizeof(struct sockaddr));
	memset(_sa,0,sizeof(_sa));
}

SockAddr::SockAddr( const SockAddr &sa )
{
	BOOST_STATIC_ASSERT(sizeof(_sa) >= sizeof(struct sockaddr));
	memcpy(_sa,sa._sa,sizeof(_sa));
}

SockAddr::SockAddr( struct sockaddr &sa )
{
	BOOST_STATIC_ASSERT(sizeof(_sa) >= sizeof(struct sockaddr));
	ASSIGN_SA(sa);
}

SockAddr::SockAddr( ipv4::SockAddr &sa )
{
	BOOST_STATIC_ASSERT(sizeof(_sa) >= sizeof(struct sockaddr));
	ASSIGN_SA(sa);
}

SockAddr::SockAddr( ipv6::SockAddr &sa )
{
	BOOST_STATIC_ASSERT(sizeof(_sa) >= sizeof(struct sockaddr));
	ASSIGN_SA(sa);
}

SockAddr::~SockAddr( void )
{
}

SockAddr &SockAddr::operator=( const SockAddr &sa ) {
	memcpy(_sa,sa._sa,sizeof(_sa));
	return *this;
}

SockAddr &SockAddr::operator=( struct sockaddr &sa ) {
	ASSIGN_SA(sa);
	return *this;
}

bool SockAddr::operator==( const SockAddr &other ) const {
	return !memcmp(_sa,other._sa,sizeof(_sa));
}

class SockAddrBoolVisitor {
public:
	SockAddrBoolVisitor( const sockaddr &sa ) : _sa(sa) {}

	bool operator()( ipv4::SockAddr &sa4 ) { return sa4 == *(sockaddr_in *)&_sa; }
	bool operator()( ipv6::SockAddr &sa6 ) { return sa6 == *(sockaddr_in6 *)&_sa; }

	const sockaddr &_sa;

private:
	SockAddrBoolVisitor &operator=( const SockAddrBoolVisitor &/*sa*/ ) {
		return *this;
	}

};

bool SockAddr::operator==( const struct sockaddr &sa ) const {
	struct sockaddr *tmp = (struct sockaddr *)_sa;
	if (tmp->sa_family == sa.sa_family) {
		return visit<bool>( SockAddrBoolVisitor(sa) );
	}
	return false;
}

SockAddr::operator struct sockaddr() const {
	struct sockaddr *tmp = (struct sockaddr *)_sa;
	return *tmp;
}

//	Methods
family::type SockAddr::family() const {
	struct sockaddr *tmp = (struct sockaddr *)_sa;
	return (family::type)tmp->sa_family;
}

class SockAddrStringVisitor {
public:
	std::string operator()( ipv4::SockAddr &sa4 ) { return sa4; }
	std::string operator()( ipv6::SockAddr &sa6 ) { return sa6; }
};

std::string SockAddr::asString() const {
	return visit<std::string>( SockAddrStringVisitor() );
}

}
}

