/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "dispatcher.h"
#include "../log.h"

namespace util {
namespace task {

Dispatcher::Dispatcher()
{
	registerTarget( NULL );
}

Dispatcher::~Dispatcher()
{
}

void Dispatcher::registerTarget( Target target ) {
	_mutex.lock();
	_targets.push_back( target );
	_mutex.unlock();
}

void Dispatcher::unregisterTarget( Target target ) {
	_mutex.lock();
	Targets::iterator it = std::find(
		_targets.begin(),
		_targets.end(),
		target
	);
	if (it != _targets.end()) {
		_targets.erase( it );
	}
	_mutex.unlock();
}

void Dispatcher::run( Target target, const Type &task ) {
	_mutex.lock();
	Targets::const_iterator it = std::find(
		_targets.begin(),
		_targets.end(),
		target
	);
	bool canExecute = it != _targets.end();
	_mutex.unlock();

	if (canExecute) {
		try {
			task();
		} catch ( const std::exception &e ) {
			LWARN("task::Dispatcher", "Catch exception: %s", e.what());
		}
	}
	else {
		LWARN("task::Dispatcher", "Target not found: %p", target);		
	}
}

void Dispatcher::clear() {
	_mutex.lock();
	_targets.clear();
	_mutex.unlock();
}

}
}

