/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <gtest/gtest.h>
#include "../../src/tool.h"
#include "functions.h"
#include "tool.h"

ToolTest::ToolTest() {
}

ToolTest::~ToolTest() {
}

void ToolTest::SetUp( void ) {
}

void ToolTest::TearDown( void ) {
	util::cfg::get().removeNode("tool");
}


TEST_F( ToolTest, constructor ) {
	util::Tool tool;
	tool.initialize("genericTool","description","2.0.0","0.0.1");

	ASSERT_TRUE( tool.name() == "genericTool" );
	ASSERT_TRUE( tool.description() == "description" );
	ASSERT_TRUE( tool.version() == "2.0.0" );
	ASSERT_TRUE( tool.build() == "0.0.1" );
}

TEST_F( ToolTest, constructor_two_parameters ) {
	util::Tool tool;
	tool.initialize("genericTool","description");

	ASSERT_TRUE( tool.name() == "genericTool" );
	ASSERT_TRUE( tool.description() == "description" );
	ASSERT_TRUE( tool.version() == "1.0.0" );
	ASSERT_TRUE( tool.build() == "0" );
}

TEST_F( ToolTest, execute ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	char **params;

	const char *argv[1] = {"programName"};
	params = util::makeParams( argv, 1 );
	ASSERT_TRUE( tool.execute( 1, params ) == 0 );
	util::deleteParams( params );
}

TEST_F( ToolTest, set_one_bool ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.help=true"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.help=true" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}

TEST_F( ToolTest, set_one_str ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.version=3.0"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.version=3.0" );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.version") == "3.0" );
	util::deleteParams( params );
}

TEST_F( ToolTest, set_two ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.version=3.0:tool.build=Build2398"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.version=3.0:tool.build=Build2398" );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.version") == "3.0" );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.build") == "Build2398" );
	util::deleteParams( params );
}

TEST_F( ToolTest, set_three ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.help=true:tool.version=3.0:tool.build=Build2398"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.help=true:tool.version=3.0:tool.build=Build2398" );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.version") == "3.0" );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.build") == "Build2398" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}

TEST_F( ToolTest, empty_set_parameters ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=:tool.help=true"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == ":tool.help=true" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}

TEST_F( ToolTest, empty_set_parameters_two ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.help=true:"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.help=true:" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}

TEST_F( ToolTest, empty_set_parameters_three ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=:tool.help=true:"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == ":tool.help=true:" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}

TEST_F( ToolTest, empty_set_parameters_four ) {
	util::Tool tool;
	tool.initialize("genericTool","description","1.0.0","version_desc");
	const char *argv[2] = {"programName","--set=tool.help=true::"};
	char **params = util::makeParams( argv, 2);

	ASSERT_TRUE( tool.execute( 2, params ) == 0 );
	ASSERT_TRUE( util::cfg::getValue<std::string> ("tool.set") == "tool.help=true::" );
	ASSERT_TRUE( util::cfg::getValue<bool> ("tool.help") );
	util::deleteParams( params );
}
