/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "show.h"
#include <mpegparser/demuxer/psi/eit.h>
#include <mpegparser/demuxer/descriptors/generic.h>
#include <util/log.h>
#include <util/cfg/cfg.h>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <assert.h>

namespace zapper {
namespace channel {

namespace bg = boost::gregorian;

namespace impl {
	static std::string lockedName = "Bloqueado";
	static std::string lockedDescription = "Este programa esta bloqueado, ingrese PIN para desbloquear";
}

Show::Show()
	: _timeRange(boost::posix_time::ptime(), boost::posix_time::ptime())
{
	_eventID = 0;
	_name = "";
	_description = "";
	_start = "";
	_duration = "";
	_parentalAge = parental::age::none;
	_parentalContent = parental::content::none;
	_locked = false;
}

static inline bpt::time_duration getTime( const tuner::MJDate &d ) {
	return bpt::time_duration( d.hours(), d.minutes(), d.seconds() );
}

Show::Show( struct tuner::eit::Event *event )
	: _timeRange(boost::posix_time::ptime(), boost::posix_time::ptime())
{
	assert(event);

	_eventID = event->eventID;
	_start = event->startTime.asString();
	_duration = event->duration.asString();
	_locked = false;	

	try {
		bg::date start_date( (unsigned short)event->startTime.year(), (unsigned short)event->startTime.month(), (unsigned short)event->startTime.day() );
		bpt::ptime start_time( start_date, getTime( event->startTime ) );
		bpt::ptime end_time = start_time + getTime( event->duration );
		_timeRange = bpt::time_period(start_time, end_time);
	} catch (std::out_of_range) {
		LWARN( "channel::Show", "Invalid startTime/duration in event: eventID=%d", event->eventID );
	}

	{	//	Get name/description from short event descriptor
		tuner::desc::MapOfDescriptors::const_iterator it = event->descs.find( tuner::desc::ident::short_event );
		if (it != event->descs.end()) {
			const tuner::desc::ShortEventDescriptor &desc = (*it).second.get<tuner::desc::ShortEventDescriptor>();
			_name = desc.event;
			_description = desc.text;
		}

		// Find extended event
		it = event->descs.find( tuner::desc::ident::extended_event );
		if (it != event->descs.end()) {
			const tuner::desc::ExtendedEventDescriptor &desc = (*it).second.get<tuner::desc::ExtendedEventDescriptor>();
			BOOST_FOREACH(tuner::desc::ExtendedEvent d, desc) {
				_description += d.text;
			}
		}
	}

	{	//	Get parental rating
		const std::string &code = util::cfg::getValue<std::string>("zapper.countryCode");
		parental::parse( event->descs, code, _parentalAge, _parentalContent );
	}
}

Show::~Show()
{
}

void Show::show() const {
	LDEBUG( "channel::Show", "start=%s, duration=%s, name=%s",
		_start.c_str(), _duration.c_str(), _name.c_str() );
}

//	Getters
tuner::ID Show::eventID() const {
	return _eventID;
}

const std::string &Show::name() const {
	return _locked ? impl::lockedName : _name;
}

const std::string &Show::description() const {
	return _locked ? impl::lockedDescription : _description;
}

const std::string &Show::start() const {
	return _start;
}

const std::string &Show::duration() const {
	return _duration;
}

parental::age::type Show::parentalAge() const {
	return _parentalAge;
}

util::BYTE Show::parentalContent() const {
	return _parentalContent;
}

bool Show::timeIntersects( const bpt::time_period &period ) const {
	return _timeRange.intersects(period);
}

const boost::posix_time::time_period& Show::timeRange() const {
	return _timeRange;
}

void Show::lock( bool value ) {
	_locked = value;
}

}
}

