/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "channelservice.h"
#include "channelplayer.h"
#include "../plugin/pluginmanager.h"
#include <util/mcr.h>
#include <assert.h>

#define NOTIFY(method)	  \
	BOOST_FOREACH( ChannelPlayer *player, _players ) { player->method; }

namespace zapper {
namespace channel {

ChannelService::ChannelService( void )
	: plugin::ServiceImpl<ChannelService>( "ChannelService" )
{
}

ChannelService::~ChannelService( void )
{
	assert( _players.empty() );
}

//	Returns master channel player
ChannelPlayer *ChannelService::master() const {
	assert(!_players.empty());
	return getPlayer( 0 );
}

//	Create a new channel player
ChannelPlayer *ChannelService::addPlayer( void ) {
	ChannelPlayer *tmp = new ChannelPlayer();
	if (tmp) {
		_players.push_back( tmp );
	}
	return tmp;
}

//	Methods
ChannelPlayer *ChannelService::getPlayer( int playerID ) const {
	if (playerID >= 0 && playerID < static_cast<int>(_players.size())) {
		return _players[playerID];
	}
	return NULL;
}

void ChannelService::setFullScreen( int playerID ) {
	ChannelPlayer *player=getPlayer( playerID );
	if (player) {
		player->setFullScreen();
	}
}

void ChannelService::resize( int playerID, int x, int y, int w, int h ) {
	ChannelPlayer *player=getPlayer( playerID );
	LDEBUG( "ChannelService", "Resize (%d,%d,%d,%d)", x, y, w, h );
	if (player) {
		player->resize( x, y, w, h );
	}	
}

//	Service virtual methods
void ChannelService::onAttach( void ) {
	//	Add service dependencies
	addSimpleDependency( "ChannelManager" );
	addSimpleDependency( "Player" );	
}

bool ChannelService::onOnline( void ) {
	//	Create master player
	return addPlayer() ? true : false;
}

void ChannelService::onOffline( void ) {
	CLEAN_ALL( ChannelPlayer *, _players );
}

bool ChannelService::onStart( void ) {
	NOTIFY( onStart( mgr() ) );
	return true;
}

void ChannelService::onStop( void ) {
	NOTIFY( onStop( mgr() ) );
}

//	Standby
void ChannelService::standby() {
	NOTIFY( onStandby( mgr() ) );
}

void ChannelService::wakeup() {
	NOTIFY( onWakeup( mgr() ) );	
}


}
}

