/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "service.h"
#include "../plugin/pluginmanager.h"

namespace zapper {
namespace control {

Service::Service()
	: plugin::ServiceImpl<Service>( "Control" )
{
	_content = parental::content::LAST_CONTENT;
	_age = parental::age::LAST_AGE;
	_sessionExpireInSeconds = -1;	//	Never expire
	_sessionActive = false;
}

Service::~Service()
{
}

bool Service::enabled() const {
	return _pass.empty() ? false : true;
}

OnEnabledChanged &Service::onEnabledChanged() {
	return _enabledChanged;
}

bool Service::sessionIsOpen() const {
	if (enabled()) {
		return isActive();
	}
	return true;
}

OnSessionChanged &Service::onSessionChanged() {
	return _sessionChanged;
}

void Service::expire() {
	LDEBUG( "Service", "Expire session!" );
	if (_sessionActive) {
		_sessionActive = false;
		_sessionChanged();
	}
}

void Service::enableSession() {
	LDEBUG( "Service", "Enable session!" );
	_lastValidation = pt::ptime();
	if (!_sessionActive) {
		_sessionActive = true;
		_sessionChanged();			
	}
}

bool Service::isActive() const {
	bool result=false;
	if (_sessionActive) {
		result=true;
		if (_sessionExpireInSeconds != -1) {
			pt::ptime t1 = pt::ptime();
			if (t1 != _lastValidation) {
				pt::time_duration dur;
				if (t1 < _lastValidation) {
					dur = _lastValidation-t1;
				}
				else {
					dur = t1-_lastValidation;
				}
				result=dur.total_seconds() < _sessionExpireInSeconds;
			}
		}
	}
	LDEBUG( "Control", "Session is active: active=%d", result );
	return result;
}

bool Service::checkPIN( const std::string &pass ) {
	if (_pass == pass) {
		enableSession();
	}
	else if (_sessionActive) {	//	Close session
		expire();
	}

	return _sessionActive;
}

void Service::setPIN( const std::string &pass ) {
	bool oldState = enabled();
	
	_pass = pass;
	mgr()->save( "control.pass", _pass );

	bool newState = enabled();
	if (oldState != newState) {
		//	Disabled?
		if (!newState) {
			expire();
			
			setTimeExpiration( -1 );
			setParentalContent( parental::content::none );
			setParentalAge( parental::age::none );
		}

		_enabledChanged( newState );

		if (newState) {
			enableSession();
		}
	}
}

void Service::setTimeExpiration( int expireInSeconds ) {
	_sessionExpireInSeconds = expireInSeconds;
}

util::BYTE Service::parentalContent() const {
	return _content;
}

void Service::setParentalContent( util::BYTE content ) {
	_content = content;
	mgr()->save( "control.content", _content );
}

parental::age::type Service::parentalAge() const {
	return _age;
}

void Service::setParentalAge( parental::age::type age ) {
	_age = age;
	mgr()->save( "control.age", _age );
}

//	Service virtual methods
void Service::standby() {
	expire();
}

void Service::onLoadConfig() {
	{	//	Setup password
		_pass = "";
		mgr()->load( "control.pass", _pass );
	}

	{	//	Setup time expireation
		int time;
		mgr()->load( "control.timeExpiration", time );
		setTimeExpiration( time );
	}

	{	//	Setup parental content
		util::BYTE content=parental::content::none;
		mgr()->load( "control.content", content );
		setParentalContent( content );
	}

	{	//	Setup parental age
		parental::age::type age=parental::age::none;
		mgr()->load( "control.age", age );
		setParentalAge( age );
	}
}

void Service::onResetConfig() {
	setPIN( "" );
}

}
}

