/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "mediaplayer.h"
#include <canvas/mediaplayer.h>
#include <canvas/size.h>
#include <canvas/point.h>
#include <canvas/rect.h>
#include <util/url.h>
#include <util/log.h>

#define GET_STREAM(v,t)	  \
	if (!v) { v = createStream(t); } \
	return v;

namespace zapper {
namespace player {
namespace impl {

MediaPlayer::MediaPlayer( canvas::MediaPlayer *mp )
	: _mp(mp)
{
}

MediaPlayer::~MediaPlayer( void )
{
	delete _mp;
}

//	Video position
void MediaPlayer::doResize() {
	{	//	Move
		canvas::Point pos( rect() );
		_mp->move( pos );
	}

	{	//	Resize
		canvas::Size size( rect() );
		_mp->resize( size );
	}
}

int MediaPlayer::nextAudio() {
	return -1;
}

int MediaPlayer::nextVideo() {
	return -1;
}

int MediaPlayer::nextSubtitle() {
	return -1;
}

bool MediaPlayer::mute( bool needMute ) {
	_mp->mute( needMute );
	return true;
}

bool MediaPlayer::volume( audio::Volume vol ) {
	_mp->volume( vol );
	return true;
}

bool MediaPlayer::startPlay( const std::string &data ) {
	bool result=false;
	util::Url url(data);

	const canvas::Rect &win = rect();
	
	LDEBUG( "MediaPlayer", "Start play: rect=(%d,%d,%d,%d)",
		win.x, win.y, win.w, win.h );
	
	//	Initialize canvas media player
	if (win.isEmpty()) {
		result=_mp->initialize( data );
	}
	else {
		result=_mp->initialize( win, data );
	}

	if (result) {
		_mp->play();
	}
	
	return result;
}

void MediaPlayer::stopPlay() {
	_mp->stop();
	_mp->finalize();
}

}
}
}

