/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "captionviewer.h"
#include <canvas/surface.h>
#include <canvas/canvas.h>
#include <canvas/system.h>
#include <mpegparser/language.h>
#include <util/task/dispatcher.h>
#include <boost/bind.hpp>
#include <stdio.h>

#define CAPTION_LAYER 100

namespace zapper {
namespace player {
namespace mpeg {

CaptionViewer::CaptionViewer( canvas::System *sys )
	: _sys(sys),
	  _bgColor(0,0,0,255),
	  _textColor(255,255,255,255),
	  _origin(150,450),
	  _size(420,75)
{
	_surface = sys->canvas()->createSurface(canvas::Rect(_origin, _size));
	_surface->setZIndex( CAPTION_LAYER );
	_surface->setVisible(false);
	_surface->setColor(_bgColor);
	_surface->fillRect(canvas::Rect(canvas::Point(0,0), _size));
	_surface->setFont(canvas::Font("Tiresias", 16));
	_surface->autoFlush(true);

	_sys->dispatcher()->registerTarget( this );
}

CaptionViewer::~CaptionViewer()
{
	_sys->dispatcher()->unregisterTarget( this );	
	_sys->canvas()->destroy(_surface);
}

void CaptionViewer::show( const std::string &data ) {
	_sys->post( this, boost::bind(&CaptionViewer::showData,this,data) );
}

void CaptionViewer::showData( const std::string &data ) {
	//printf("[mpeg::CaptionViewer] Show: %s\n", data.c_str());
	_surface->setVisible( true );
	_surface->clear();
	_surface->setColor(_textColor);
	_surface->drawText(canvas::Rect(5,5,410,65), tuner::iso8859ToUTF8(data) );
	_sys->canvas()->flush();
}

void CaptionViewer::hide() {
	//printf("[mpeg::CaptionViewer] Hide\n");
	_surface->setVisible( false );
	_surface->setColor(canvas::Color(0,0,0,0));
	_surface->clear();
	_sys->canvas()->flush();	
}

} //	namespace mpeg
} //	namespace player
} //	namespace zapper

