/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "serviceplayer.h"
#include "streaminfo.h"
#include "streamplayer.h"
#include "caption/captionplayer.h"
#include "caption/captionviewer.h"
#include "../../tuner/tuner.h"
#include <mpegparser/service/service.h>
#include <mpegparser/demuxer/ts.h>
#include <util/url.h>
#include <util/log.h>
#include <util/mcr.h>
#include <boost/foreach.hpp>

namespace zapper {
namespace player {
namespace mpeg {

ServicePlayer::ServicePlayer( Tuner *tuner, player::MediaPlayer *media )
	: _tuner(tuner), _media(media)
{
	assert(_tuner);
	assert(_media);

	_pcrPID = TS_PID_NULL;
	_pcrStarted = false;

	_mediaStarted = false;

	//	Add to notificatator
	_tuner->onServiceStarted().connect( boost::bind( &ServicePlayer::onServiceStarted,this,_1,_2) );
}

ServicePlayer::~ServicePlayer()
{
	assert(!_mediaStarted);	
	
	BOOST_FOREACH( StreamPlayer *player, _players ) {
		player->finalize();
		delete player;
	}
	delete _media;
}

void ServicePlayer::addPlayers( canvas::System *sys ) {
	{	//	Add audio player
		std::vector<StreamInfo *> streams;
		mpegAudioStreams( streams );
		isdbtAudioStreams( streams );
		addPlayer( create( tuner::pes::audio, streams ) );
	}

	{	//	Add video player
		std::vector<StreamInfo *> streams;
		mpegVideoStreams( streams );
		isdbtVideoStreams( streams );
		addPlayer( create( tuner::pes::video, streams ) );
	}

	{	//	Add CC player
		CaptionViewer *view = new CaptionViewer( sys );
		addPlayer( new CaptionPlayer( _media, view ) );
	}
}

//	Add elementary stream player
void ServicePlayer::addPlayer( StreamPlayer *player ) {
	if (player->initialize( tuner() )) {
		_players.push_back( player );
	}
}

//	Media
void ServicePlayer::setFullScreen() {
	_media->setFullScreen();
}

void ServicePlayer::resize( int x, int y, int w, int h ) {
	_media->resize( x, y, w, h );
}

int ServicePlayer::nextAudio() {
	StreamPlayer *p = getPlayer( tuner::pes::audio );
	return p ? p->next() : -1;
}

int ServicePlayer::audioCount() const  {
	StreamPlayer *p = getPlayer( tuner::pes::audio );
	return p ? p->count() : -1;
}

int ServicePlayer::nextVideo() {
	StreamPlayer *p = getPlayer( tuner::pes::video );
	return p ? p->next() : -1;
	return getPlayer( tuner::pes::video )->next();
}

int ServicePlayer::videoCount() const {
	StreamPlayer *p = getPlayer( tuner::pes::video );
	return getPlayer( tuner::pes::video )->count();
	return p ? p->count() : -1;
}

int ServicePlayer::nextSubtitle() {
	StreamPlayer *p = getPlayer( tuner::pes::subtitle );
	return p ? p->next() : -1;
}

int ServicePlayer::subtitleCount() const {
	StreamPlayer *p = getPlayer( tuner::pes::subtitle );
	return p ? p->count() : -1;
}

//	Volume
bool ServicePlayer::mute( bool needMute ) {
	return _media->mute( needMute );
}

bool ServicePlayer::volume( audio::Volume vol ) {
	return _media->volume( vol );
}

//	Display notifications
void ServicePlayer::onDisplayChanged() {
	_media->onDisplayChanged();
}

bool ServicePlayer::startPlay( const std::string &params ) {
	size_t networkID=size_t(-1);
	util::Url url(params);
	tuner::ID nitID = 0, tsID = 0, srvID = 0;
	bool result;

	LINFO( "mpeg::ServicePlayer", "Start channel: params=%s", params.c_str() );

	assert(url.type() == "service");

	//	Get and validate network parameter
	result=url.getParam( "network", networkID );
	if (!result || networkID == size_t(-1)) {
		LWARN( "mpeg::ServicePlayer", "invalid network parameter in url" );
		return false;
	}

	//	Get nitID parameter
	result=url.getParam( "nitID", nitID );
	if (!result) {
		LWARN( "mpeg::ServicePlayer", "invalid nitID parameter in url" );
		return false;
	}

	//	Get tsID parameter
	result=url.getParam( "tsID", tsID );
	if (!result) {
		LWARN( "mpeg::ServicePlayer", "invalid tsID parameter in url" );
		return false;
	}

	//	Get srvID parameter
	result=url.getParam( "srvID", srvID );
	if (!result) {
		LWARN( "mpeg::ServicePlayer", "invalid srvID parameter in url" );
		return false;
	}

	//	Register serviceID;
	_serviceID = ServiceID( nitID, tsID, srvID );

	//	Start service
	if (!_tuner->startService( networkID, srvID )) {
		LWARN( "mpeg::ServicePlayer", "cannot start service" );
		return false;
	}

	return true;
}

void ServicePlayer::stopPlay() {
	LINFO( "mpeg::ServicePlayer", "Stop channel" );
	//	Stop tuner
	_tuner->stopService( _serviceID.programID() );
	_serviceID = ServiceID();

	//	HACK: Stop media without wait onServiceStoped, becouse
	//	onServiceStopped is enqueued and called after stopped and
	//	vlc have a another bug
	stopMedia();
}

void ServicePlayer::refreshAudio() {
	setAudio( _media->getAudio() );
}

void ServicePlayer::refreshAspect() {
	setAspect( _media->getAspect() );
}

void ServicePlayer::refreshDisplay() {
	setDisplay( _media->getResolution() );
}

//	Player extension methods
void ServicePlayer::onServiceStarted( tuner::Service *srv, bool started ) {
	static ServiceID waitStop;

	LDEBUG( "mpeg::ServicePlayer", "On service started: id=%04x, started=%d, _serviceID=%04x, waitStop=%d",
		srv->id(), started, _serviceID.programID(), waitStop.programID() );	

	ServiceID id(srv->nitID(), srv->tsID(), srv->id() );
	if (started) {
		if (_serviceID == id) {
			startMedia( srv );
			waitStop = _serviceID;
		}
	}
	else {
		if (waitStop == id) {
			stopMedia();
			waitStop = ServiceID();
		}
	}
}

void ServicePlayer::startMedia( tuner::Service *srv ) {
	LDEBUG( "mpeg::ServicePlayer", "Start media" );

	assert( !_mediaStarted );

	//	Save PCR and reset flag
	_pcrPID = srv->pcrPID();
	bool needPCRFilter = true;

	//	Collect all streams supported
	const std::vector<tuner::ElementaryInfo> &elements = srv->elements();
	BOOST_FOREACH( tuner::ElementaryInfo info, elements ) {
		//	Add streams for c/player
		BOOST_FOREACH( StreamPlayer *player, _players ) {
			player->collectStream( info );
		}

		if (_pcrPID == info.pid) {
			needPCRFilter = false;
		}
	}

	onStartBegin();

	//	Start all players
	BOOST_FOREACH( StreamPlayer *player, _players ) {
		player->start();
	}

	//	Start PCR
	needPCRFilter &= (_pcrPID != TS_PID_NULL);
	_pcrStarted=startPCR( needPCRFilter );

	onStartEnd();

	_mediaStarted = true;
}

void ServicePlayer::stopMedia() {
	if (_mediaStarted) {
		LDEBUG( "mpeg::ServicePlayer", "Stop media" );

		//	Begin stop
		onStopBegin();

		//	Stop all streams
		BOOST_FOREACH( StreamPlayer *player, _players ) {
			player->stop( true );
		}

		//	Stop pcr
		if (_pcrStarted) {
			stopPCR();
			_pcrStarted=false;
		}

		//	End stop
		onStopEnd();

		_mediaStarted = false;
	}
}

void ServicePlayer::onStartBegin() {
	_media->beginStartService();
}

void ServicePlayer::onStartEnd() {
	_media->endStartService();	
}

void ServicePlayer::onStopBegin() {
	_media->beginStopService();
}

void ServicePlayer::onStopEnd() {
	_media->endStopService();	
}

tuner::ID ServicePlayer::pcrPID() const {
	return _pcrPID;
}

bool ServicePlayer::startPCR( bool needFilter ) {
	//	Setup media
	_media->startPCR( _pcrPID );

	if (needFilter) {
		//	Start PCR
		LINFO( "mpeg::ServicePlayer", "Start PCR stream: pid=%04x", _pcrPID );
		return tuner()->startFilter( _pcrPID, tuner::pes::pcr, getPCRPipe() );
	}
	return false;
}

void ServicePlayer::stopPCR() {
	tuner()->stopFilter( _pcrPID );
}

//	Streams
tuner::stream::Pipe *ServicePlayer::getPCRPipe() {
	return NULL;
}

StreamPlayer *ServicePlayer::create( tuner::pes::type pesType, const std::vector<StreamInfo *> &streams ) {
	return new StreamPlayer( _media, pesType, streams );
}

StreamPlayer *ServicePlayer::getPlayer( tuner::pes::type pesType ) const {
	std::vector<StreamPlayer *>::const_iterator it=std::find_if(
		_players.begin(),
		_players.end(),
		boost::bind(&StreamPlayer::type,_1) == pesType );
	return (it == _players.end()) ? NULL : (*it);
}

Tuner *ServicePlayer::tuner() const {
	return _tuner;
}

player::MediaPlayer *ServicePlayer::media() const {
	return _media;
}

}
}
}

