/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "streaminfo.h"
#include <boost/foreach.hpp>

namespace zapper {
namespace player {
namespace mpeg {

StreamInfo::StreamInfo( tuner::ID streamType, const std::string &name )
	: _type(streamType), _name(name)
{
}

StreamInfo::~StreamInfo()
{
}

tuner::ID StreamInfo::type() const {
	return _type;
}

const std::string &StreamInfo::name() const {
	return _name;
}

bool StreamInfo::checkTag( tuner::ID tag ) const {
	bool valid=false;
	BOOST_FOREACH( const Tag &t, _tags ) {
		if (t.first <= tag && tag <= t.second) {
			valid=true;
			break;
		}
	}
	return valid;
}

void StreamInfo::addTag( tuner::ID first, tuner::ID last ) {
	_tags.push_back( std::make_pair( first, last ) );
}

#define ADD_STREAM( n, name )	\
	{ StreamInfo *si = new StreamInfo( n, name ); si->addTag( 0x00, 0xFF ); tmp.push_back( si ); }

void mpegVideoStreams( std::vector<StreamInfo *> &tmp ) {
	ADD_STREAM( 0x01, "MPEG-1 Video" );
	ADD_STREAM( 0x02, "MPEG-2 Video" );
}

void mpegAudioStreams( std::vector<StreamInfo *> &tmp ) {
	ADD_STREAM( 0x03, "MPEG-1 Audio" );
	ADD_STREAM( 0x04, "MPEG-1 Audio" );
	ADD_STREAM( 0x0F, "MPEG-2 AAC Audio" );
	ADD_STREAM( 0x81, "AC3 Audio" );
}

void isdbtVideoStreams( std::vector<StreamInfo *> &tmp ) {
	//	H.264 video
	StreamInfo *i = new StreamInfo( 0x1b, "H.264 Video" );
	i->addTag( 0x00, 0x0F );
	i->addTag( 0x81, 0x82 );
	i->addTag( 0x00, 0x00 );
	tmp.push_back( i );
}

void isdbtAudioStreams( std::vector<StreamInfo *> &tmp ) {
	//	MPEG-4 AAC Audio (LATM, 48Khz)
	StreamInfo *i = new StreamInfo( 0x11, "MPEG-4 AAC Audio (LATM, 48Khz)" );
	i->addTag( 0x10, 0x2F );
	i->addTag( 0x83, 0x86 );  //	24Khz
	i->addTag( 0x90, 0x91 );  //	16Khz
	i->addTag( 0x90, 0x91 );  //	16Khz		
	i->addTag( 0x00, 0x00 );
	tmp.push_back( i );
}

}
}
}

