/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "player.h"
#include "mediaplayer.h"
#include "mpeg/serviceplayer.h"
#include "mpeg/streaminfo.h"
#include "../tuner/tuner.h"
#include <mpegparser/service/types.h>
#include <util/url.h>
#include <util/log.h>
#include <util/mcr.h>
#include <boost/bind.hpp>

#define NOTIFY( method )	\
	BOOST_FOREACH( MediaPlayer *player, _players ) { player->method; }

namespace zapper {
namespace player {

Player::Player( canvas::System *sys )
	: _sys( sys )
{
	_tuner = NULL;
	_dResolution = std::make_pair(0,0);
	_dAspect = display::aspect::LAST_ASPECT;
}

Player::~Player()
{
	if (_players.size()) {
		LWARN( "Player", "Not all players destroyed" );
	}
	CLEAN_ALL( MediaPlayer *,_players );
}

//	Initialization
bool Player::initialize( Tuner *tuner ) {
	_tuner = tuner;
	return doInitialize();
}

bool Player::doInitialize() {
	return true;
}

void Player::finalize() {
	doFinalize();
	_tuner = NULL;
}

void Player::doFinalize() {
}

//	Display notifications
void Player::setResolution( int width, int height ) {
	LDEBUG( "Player", "Set resolution: w=%d, h=%d", width, height );
	_dResolution = std::make_pair(width,height);
	onDisplayChanged();
	NOTIFY( onDisplayChanged() );
}

const Resolution &Player::getDisplayResolution() const {
	return _dResolution;
}

void Player::setAspectRatio( display::aspect::type vAspect ) {
	LDEBUG( "Player", "Set aspect ratio: aspect=%d", vAspect );
	_dAspect = vAspect;
	onDisplayChanged();	
	NOTIFY( onDisplayChanged() );	
}

display::aspect::type Player::getDisplayAspect() const {
	return _dAspect;
}

//	Display notifications
void Player::onDisplayChanged( void ) {
	//	Do nothing
}

//	MediaPlayer
MediaPlayer *Player::create( type::type typeOfService ) {
	assert(_tuner);
	MediaPlayer *player;
	
	switch (typeOfService) {
		case type::service: {
			mpeg::ServicePlayer *mp = createService( _tuner );
			mp->addPlayers( _sys );
			player = mp;
			break;
		}
		case type::audio:
			player = createAudio();
			break;
		case type::video:
			player = createVideo();
			break;
		default:
			player = NULL;
			LWARN( "Player", "Invalid type of player: type=%d", typeOfService );
			break;
	}
	
	if (player) {
		_players.push_back( player );
	}
	return player;
}

mpeg::ServicePlayer *Player::createService( Tuner *tuner ) const {
	return new mpeg::ServicePlayer( tuner, createVideo() );
}

void Player::destroy( MediaPlayer *player ) {
	std::vector<MediaPlayer *>::iterator it=std::find( _players.begin(), _players.end(), player );
	if (it != _players.end()) {
		_players.erase( it );
		player->stop();
		delete player;
	}
}

//	Getter
canvas::System *Player::sys() const {
	return _sys;
}

}
}

