/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "playerservice.h"
#include "player.h"
#include "../display/displayservice.h"
#include "../tuner/tuner.h"
#include "../plugin/pluginmanager.h"
#include <util/mcr.h>
#include <util/log.h>
#include <assert.h>

namespace zapper {
namespace player {

PlayerService::PlayerService( const std::string &tunerID, Player *player )
	: plugin::ServiceImpl<PlayerService>( "Player" ), _tunerID(tunerID)
{
	assert(player);
	_player = player;
	_tuner = NULL;
}

PlayerService::~PlayerService( void )
{
	delete _player;
}

//	Service virtual methods
void PlayerService::onAttach( void ) {
	//	Add service dependencies
	addSimpleDependency( "Display" );
	addSimpleDependency( _tunerID );
}

bool PlayerService::onOnline( void ) {
	{	//	Setup display dependencies
		display::DisplayService *display = mgr()->findService<display::DisplayService>( "Display" );
		assert(display);
		display->onModeChanged().connect( boost::bind( &PlayerService::onDisplayModeChanged, this, _1 ) );
		display->onAspectChanged().connect( boost::bind( &PlayerService::onDisplayAspectChanged, this, _1 ) );
	}

	//	Setup tuner
	_tuner = mgr()->addRef<Tuner>( _tunerID );
	assert(_tuner);
	
	return _player->initialize( _tuner );
}

void PlayerService::onOffline( void ) {
	_player->finalize();
	RELEASE_SERVICE(_tuner);	
}

//	Services notifications
void PlayerService::onDisplayModeChanged( display::mode::type vMode ) {
	display::Mode mode = display::getMode( vMode );
	_player->setResolution( mode.width, mode.height );
}

void PlayerService::onDisplayAspectChanged( display::aspect::type vAspect ) {
	_player->setAspectRatio( vAspect );
}

//	MediaPlayer instances
MediaPlayer *PlayerService::create( type::type typeOfService ) {
	return _player->create( typeOfService );
}

void PlayerService::destroy( MediaPlayer *player ) {
	_player->destroy( player );
}

}
}

