/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "dependencyhandler.h"
#include "dependency.h"
#include "pluginmanager.h"
#include <util/mcr.h>
#include <boost/bind.hpp>

namespace zapper {
namespace plugin {

DependencyHandler::DependencyHandler()
{
}

DependencyHandler::~DependencyHandler()
{
	assert(_dependencies.empty());
}

//	Initialize dependencies
void DependencyHandler::add( Dependency *dep ) {
	_dependencies.push_back( dep );
}

void DependencyHandler::initialize( PluginManager *mgr ) {
	if (!_dependencies.empty()) {
		BOOST_FOREACH( Service *srv, mgr->services() ) {
			update( srv );
		}
	}
}

void DependencyHandler::finalize() {
	CLEAN_ALL( Dependency *, _dependencies );
}

//	Notifications
void DependencyHandler::update( Service *srv ) {
	BOOST_FOREACH( Dependency *dep, _dependencies ) {
		dep->update( srv );
	}
}

//	Queries
bool DependencyHandler::areOnline() const {
	bool areOnline=true;
	BOOST_FOREACH( Dependency *dep, _dependencies ) {
		areOnline &= dep->isOnline();
	}
	return areOnline;
}

bool DependencyHandler::areRunning() const {
	bool areRunning=true;
	BOOST_FOREACH( Dependency *dep, _dependencies ) {
		areRunning &= dep->isStarted();
	}	
	return areRunning;
}

}
}
