/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "plugin.h"
#include "pluginmanager.h"
#include <mpegparser/service/extension/extension.h>
#include <util/log.h>
#include <util/string.h>
#include <assert.h>

namespace zapper {
namespace plugin {

Plugin::Plugin( const std::string &name )
	: _name( name ), _isEnabled( false )
{
	_mgr = NULL;
}

Plugin::~Plugin( void )
{
}

//	Getters
const std::string &Plugin::name() const {
	return _name;
}

PluginManager *Plugin::mgr() const {
	return _mgr;
}

//	Initialization
void Plugin::initialize( plugin::PluginManager *mgr ) {
	assert(mgr);
	if (!_mgr) {
		_mgr = mgr;
		
		//	Load state from SRAM
		bool enabled = defaultEnabledState();
		_mgr->load( pluginName(), enabled );

		LDEBUG( "Plugin", "Initialize: name=%s, enabled=%d",
			_name.c_str(), enabled );

		//	Initialize plugin
		if (onInitialize()) {
			//	Enable if necesary
			enable( enabled );
		}
		else {
			LWARN( "Plugin", "Warning, failed to initialize plugin: name=%s", name().c_str() );
			_mgr=NULL;
		}
	}
}

void Plugin::finalize( void ) {
	if (_mgr) {
		if (isEnabled()) {
			onEnable(false);
		}
		onFinalize();
		_mgr=NULL;
	}
}

bool Plugin::isInitialized( void ) const {
	return _mgr ? true : false;
}

bool Plugin::onInitialize( void ) {
	return true;
}

void Plugin::onFinalize( void ) {
}

//	Enable
bool Plugin::defaultEnabledState( void ) const {
	return true;
}

bool Plugin::isEnabled() const {
	return isInitialized() ? _isEnabled : false;
}

void Plugin::enable( bool newState ) {
	if (_isEnabled != newState) {
		LDEBUG( "Plugin", "Enable plugin: name=%s, state=%d",
			_name.c_str(), newState );
		
		_isEnabled = newState;
		_mgr->save( pluginName(), newState );
		onEnable( newState );
	}
}

void Plugin::onEnable( bool /*isEnabled*/ ) {
	//	Do nothing
}

//	Reset config
void Plugin::resetConfig( void ) {
	//	Reset plugin state
	if (isInitialized()) {
		//	Reset plugin config
		onResetConfig();
		//	Change plugin state to default
		enable( defaultEnabledState() );		
	}
}

void Plugin::onResetConfig( void ) {
	//	Do nothing
}

//	Aux
std::string Plugin::pluginName( void ) const {
	assert(_mgr);
	return util::format( "%sPlugin.enabled",
		name().c_str() );
}

}
}

