/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "serviceid.h"
#include <mpegparser/demuxer/psi/psi.h>
#include <util/log.h>

namespace zapper {

ServiceID::ServiceID( void )
{
	_nitID     = NIT_ID_RESERVED;
	_tsID      = 0;
	_programID = 0;
}

ServiceID::ServiceID( tuner::ID nitID, tuner::ID tsID, tuner::ID programID )
{
	_nitID     = nitID;
	_tsID      = tsID;
	_programID = programID;
}

ServiceID::ServiceID( const ServiceID &other )
{
	_nitID     = other._nitID;
	_tsID      = other._tsID;
	_programID = other._programID;
}

ServiceID::~ServiceID( void )
{
}

ServiceID &ServiceID::operator=( const ServiceID &other ) {
	_nitID     = other._nitID;
	_tsID      = other._tsID;
	_programID = other._programID;
	return *this;
}

bool ServiceID::operator==( const ServiceID &other ) const {
	return (_nitID == other._nitID &&
		_tsID == other._tsID &&
		_programID == other._programID);
}

bool ServiceID::operator!=( const ServiceID &other ) const {
	return !(*this == other);
}

bool ServiceID::operator<( const ServiceID &other ) const {
	if (_nitID < other._nitID) {
		return true;
	}
	else if (_nitID == other._nitID) {
		if (_tsID < other._tsID) {
			return true;
		}
		else if (_tsID == other._tsID) {
			return _programID < other._programID;
		}
	}
	return false;
}

tuner::ID ServiceID::nitID() const {
	return _nitID;
}

tuner::ID ServiceID::tsID() const {
	return _tsID;
}

tuner::ID ServiceID::programID() const {
	return _programID;
}

void ServiceID::show( void ) const {
	LDEBUG( "ServiceID", "nitID=%04x, tsID=%04x, programID=%04x",
		_nitID, _tsID, _programID );
}

}

