/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "updateinfo.h"
#include <mpegparser/service/extension/download/downloadinfo.h>
#include <util/log.h>
#include <boost/foreach.hpp>
#include <boost/filesystem.hpp>

namespace zapper {
namespace update {

namespace fs = boost::filesystem;

UpdateInfo::UpdateInfo( void )
{
	_id = -1;
	_mandatory = false;
}

UpdateInfo::~UpdateInfo( void )
{
	//	Delete all files
	BOOST_FOREACH( const std::string &file, _files ) {
		fs::remove( file );
	}
}

void UpdateInfo::setID( int id ) {
	_id = id;
}

int UpdateInfo::id() const {
	return _id;
}

bool UpdateInfo::isMandatory() const {
	return _mandatory;
}

const std::vector<std::string> &UpdateInfo::files() const {
	return _files;
}

bool UpdateInfo::process( tuner::DownloadInfo *info, const std::string &path ) {
	bool result=true;
	
	LDEBUG( "UpdateService", "Procesing download information: files=%d", info->files().size() );

	//	Save update
	_mandatory = info->compulsory();

	//	Move files to update directory
	try {		
		BOOST_FOREACH( const std::string &file, info->files() ) {
			//	Move file to update directory
			fs::path src = file;
			fs::path dst = path;
			dst /= src.filename();
			fs::rename( src, dst );

			//	Add to list
			_files.push_back( dst.string() );
		}
	} catch ( ... ) {
		LWARN( "UpdateService", "Cannot move file, skip update" );
		result=false;
	}

	return result;
}

}
}
