/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "updateservice.h"
#include "updateplugin.h"
#include "updateinfo.h"
#include "../tuner/tuner.h"
#include "../plugin/pluginmanager.h"
#include <mpegparser/service/extension/download/downloadinfo.h>
#include <mpegparser/service/extension/download/downloaderextension.h>
#include <util/cfg/cfg.h>
#include <util/fs.h>
#include <util/mcr.h>
#include <util/log.h>
#include <boost/bind.hpp>
#include <boost/filesystem.hpp>
#include "generated/config.h"

namespace zapper {
namespace update {

namespace fs = boost::filesystem;

UpdateService::UpdateService( const std::string &tuner, const std::string &ramDisk )
	: plugin::ServiceImpl<UpdateService>( "Update" ), _tuner(tuner)
{
	{	//	Setup work directory
		fs::path tmp = ramDisk;
		tmp /= "update";
		_workDirectory = tmp.string();
	}

	_id = 0;
	_ext = NULL;
}

UpdateService::~UpdateService()
{
	assert(_updates.empty());
}

void UpdateService::onAttach() {
	addSimpleDependency( _tuner );
}

bool UpdateService::onStart() {
	assert(_updates.empty());
	_id = 0;
	if (util::safeCleanDirectory( _workDirectory )) {
		//	Create extension
		int makerID = util::cfg::getValue<int>("zapper.otaMakerId");
		int modelID = util::cfg::getValue<int>("zapper.otaModelId");
		int version = util::cfg::getValue<int>("zapper.otaVersion");				
		_ext = new tuner::DownloaderExtension( makerID, modelID, version );
		_ext->onDownload( boost::bind(&UpdateService::onUpdate,this,_1) );
		_ext->onProgressChanged( boost::bind(&UpdateService::downloadProgress,this,_1,_2) );

		//	Attach extension to tuner
		Tuner::attach( mgr(), _ext, _tuner );
		
		return true;		
	}
	return false;
}

void UpdateService::onStop() {
	Tuner::detach( mgr(), _ext, _tuner );	
	CLEAN_ALL( UpdateInfo *, _updates );
}

//	Update notifications/handling
void UpdateService::onUpdateReceived( const OnUpdateReceived &callback ) {
	_onUpdate = callback;
}

void UpdateService::onUpdate( tuner::DownloadInfo *info ) {
	UpdateInfo *update = new UpdateInfo();
	if (update->process(info, _workDirectory)) {
		//	Add to list of downloads
		update->setID( _id++ );
		_updates.push_back( update );

		//	Notify a update is present
		if (!_onUpdate.empty()) {
			_onUpdate( update->id(), update->isMandatory() );
		}
	}
}

//	Download progress
void UpdateService::onDownloadProgress( const OnDownloadProgress &callback ) {
	_onDownloadProgress = callback;
}

void UpdateService::downloadProgress( int step, int total ) {
	if (!_onDownloadProgress.empty()) {
		_onDownloadProgress( step, total );
	}
}

//	Methods
void UpdateService::apply( int updateID, bool apply ) {
	LDEBUG( "UpdateService", "Apply update: id=%d, apply=%d", updateID, apply );

	//	Get update ...
	std::vector<UpdateInfo *>::iterator it=std::find_if(
		_updates.begin(),
		_updates.end(),
		boost::bind(&UpdateInfo::id,_1) == updateID );
	if (it != _updates.end()) {
		//	Remove update from list
		UpdateInfo *tmp = (*it);
		_updates.erase( it );
		
		//	Need apply update?
		if (apply) {
			//	Clean others updates pending for confirmation and exit
			CLEAN_ALL( UpdateInfo *, _updates );
		}
		else {
			delete tmp;
		}
	}
}

}
}

