/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "plugin.h"
#include "../util.h"
#include <util/task/dispatcherimpl.h>
#include <gtest/gtest.h>
#include <boost/bind.hpp>

TEST( Plugin, constructor ) {
	zapper::plugin::Plugin test( "name" );
	ASSERT_TRUE( test.name() == "name" );
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
}

TEST( Plugin, initialize ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 0 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isInitialized() );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 1 );
}

TEST( Plugin, finalize ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 0 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isInitialized() );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 1 );
	test.finalize();
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 1 );	
	ASSERT_TRUE( enabled == 0 );
}

TEST( Plugin, initialize_twice ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 0 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isInitialized() );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 1 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isInitialized() );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 1 );
}

TEST( Plugin, finalize_twice ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 0 );
	ASSERT_TRUE( fin == 0 );		
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isInitialized() );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );
	ASSERT_TRUE( enabled == 1 );
	test.finalize();
	test.finalize();
	ASSERT_FALSE( test.isInitialized() );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 1 );
	ASSERT_TRUE( enabled == 0 );
}

TEST( Plugin, initialize_disable_enable ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( enabled == 1 );
	test.enable( false );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( enabled == 0 );
	test.enable( true );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( enabled == 1 );
}

TEST( Plugin, initialize_disable ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "plugin_test" );
	mgr.save( "testPlugin.enabled",  false);
	int init=0, fin=0, enabled=0;
	OkPlugin test( init, fin, enabled );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( enabled == 0 );
	test.initialize( &mgr );
	ASSERT_FALSE( test.isEnabled() );
	ASSERT_TRUE( enabled == 0 );
	test.enable( true );
	ASSERT_TRUE( test.isEnabled() );
	ASSERT_TRUE( enabled == 1 );
}
