/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "../util.h"
#include "plugin.h"
#include "service.h"
#include <util/task/dispatcherimpl.h>
#include <gtest/gtest.h>
#include <boost/bind.hpp>

TEST( PluginManager, constructor ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
}

TEST( PluginManager, add ) {
	zapper::plugin::PluginManager plugin( new util::task::DispatcherImpl() );
	plugin.add( new zapper::plugin::Plugin( "plugin1" ) );
	ASSERT_TRUE( plugin.isPresent( "plugin1" ) );
}

TEST( PluginManager, initialization ) {
	int init, fin, enabled;
	init = fin = enabled = 0;
	zapper::plugin::PluginManager plugins( new util::task::DispatcherImpl() );
	OkPlugin *plug = new OkPlugin( init, fin, enabled );
	plugins.add( plug );
	ASSERT_TRUE( plugins.isPresent( "test" ) );
	plugins.initialize( "pluginmanager_test" );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );
	plugins.finalize();
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 1 );
}

TEST( PluginManager, initialize_twice ) {
	int init, fin, enabled;
	init = fin = enabled = 0;
	zapper::plugin::PluginManager plugins( new util::task::DispatcherImpl() );
	OkPlugin *plug = new OkPlugin( init, fin, enabled );
	plugins.add( plug );
	ASSERT_TRUE( plugins.isPresent( "test" ) );
	plugins.initialize( "pluginmanager_test" );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );
	plugins.initialize( "pluginmanager_test" );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );
	plugins.finalize();
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 1 );
}

TEST( PluginManager, finalize_twice ) {
	int init, fin, enabled;
	init = fin = enabled = 0;
	zapper::plugin::PluginManager plugins( new util::task::DispatcherImpl() );
	OkPlugin *plug = new OkPlugin( init, fin, enabled );
	plugins.add( plug );
	ASSERT_TRUE( plugins.isPresent( "test" ) );
	plugins.initialize( "pluginmanager_test" );
	ASSERT_TRUE( plugins.isInitialized() );
	ASSERT_TRUE( init == 1 );
	ASSERT_TRUE( fin == 0 );
	plugins.finalize();
	ASSERT_FALSE( plugins.isInitialized() );	
	ASSERT_TRUE( init == 1 );	
	ASSERT_TRUE( fin == 1 );
	plugins.finalize();
	ASSERT_TRUE( init == 1 );	
	ASSERT_TRUE( fin == 1 );
}

TEST( PluginManager, isPresent ) {
	int init, fin, enabled;
	init = fin = enabled = 0;
	zapper::plugin::PluginManager plugins( new util::task::DispatcherImpl() );
	OkPlugin *plug = new OkPlugin( init, fin, enabled );
	plugins.add( plug );
	ASSERT_TRUE( plugins.isPresent( "test" ) );
	ASSERT_FALSE( plugins.isPresent( "test1" ) );
}

TEST( PluginManager, registerService ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	ASSERT_FALSE( mgr.registerService( new DummyService() ) );	
	mgr.initialize( "pluginmanager_test" );
	CheckService check;
	ASSERT_TRUE( mgr.registerService( new CountService( check ) ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService" ) != NULL );
	mgr.unregisterService( "dummyService" );
	ASSERT_FALSE( mgr.findService<DummyService>( "dummyService" ) != NULL );
	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 1 );
	ASSERT_TRUE( check.offline == 1 );		
	ASSERT_TRUE( check.start == 0 );
	ASSERT_TRUE( check.stop == 0 );
	ASSERT_TRUE( check.loadConfig == 0 );
	ASSERT_TRUE( check.resetConfig == 0 );
}

TEST( PluginManager, registerService_with_dependency_order1 ) {
	CheckService check1, check2;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService1" ) != NULL );
	ASSERT_TRUE( mgr.registerService( new ServiceWithDependency( check2, "dummyService2", "dummyService1" ) ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService2" ) != NULL );
	mgr.unregisterService( "dummyService1" );
	mgr.unregisterService( "dummyService2" );
	ASSERT_TRUE( check1.attach == 1 );
	ASSERT_TRUE( check1.dettach == 1 );
	ASSERT_TRUE( check1.online == 1 );
	ASSERT_TRUE( check1.offline == 1 );		
	ASSERT_TRUE( check1.start == 0 );
	ASSERT_TRUE( check1.stop == 0 );
	ASSERT_TRUE( check1.loadConfig == 0 );
	ASSERT_TRUE( check1.resetConfig == 0 );
	
	ASSERT_TRUE( check2.attach == 1 );
	ASSERT_TRUE( check2.dettach == 1 );
	ASSERT_TRUE( check2.online == 1 );
	ASSERT_TRUE( check2.offline == 1 );	
	ASSERT_TRUE( check2.start == 0 );
	ASSERT_TRUE( check2.stop == 0 );
	ASSERT_TRUE( check2.loadConfig == 0 );
	ASSERT_TRUE( check2.resetConfig == 0 );	
}

TEST( PluginManager, registerService_with_dependency_order2 ) {
	CheckService check1, check2;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new ServiceWithDependency( check2, "dummyService2", "dummyService1" ) ) );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );
	mgr.unregisterService( "dummyService1" );
	mgr.unregisterService( "dummyService2" );
	ASSERT_TRUE( check1.attach == 1 );
	ASSERT_TRUE( check1.dettach == 1 );
	ASSERT_TRUE( check1.online == 1 );
	ASSERT_TRUE( check1.offline == 1 );	   
	ASSERT_TRUE( check1.start == 0 );
	ASSERT_TRUE( check1.stop == 0 );
	ASSERT_TRUE( check1.loadConfig == 0 );
	ASSERT_TRUE( check1.resetConfig == 0 );
	
	ASSERT_TRUE( check2.attach == 1 );
	ASSERT_TRUE( check2.dettach == 1 );
	ASSERT_TRUE( check2.online == 1 );
	ASSERT_TRUE( check2.offline == 1 );		
	ASSERT_TRUE( check2.start == 0 );
	ASSERT_TRUE( check2.stop == 0 );
	ASSERT_TRUE( check2.loadConfig == 0 );
	ASSERT_TRUE( check2.resetConfig == 0 );	
}

TEST( PluginManager, registerService_with_unresolved_dependency ) {
	CheckService check;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new UnresolvedService( check ) ) );
	mgr.unregisterService( "dummyService" );	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 0 );
	ASSERT_TRUE( check.offline == 0 );		
	ASSERT_TRUE( check.start == 0 );
	ASSERT_TRUE( check.stop == 0 );
	ASSERT_TRUE( check.loadConfig == 0 );
	ASSERT_TRUE( check.resetConfig == 0 );
}

TEST( PluginManager, registerService_with_multiple_instances ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new DummyService( "dummyService1" ) ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService1" ) != NULL );
	ASSERT_TRUE( mgr.registerService( new DummyService( "dummyService2" ) ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService2" ) != NULL  );
}

TEST( PluginManager, registerService_twice ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new DummyService() ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService" ) != NULL );
	ASSERT_FALSE( mgr.registerService( new DummyService() ) );
	ASSERT_TRUE( mgr.findService<DummyService>( "dummyService" ) != NULL );
}

TEST( PluginManager, findService_fail ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_FALSE( mgr.findService<DummyService>( "dummyService" ) );
}

TEST( PluginManager, findService_invalid_fail ) {
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new DummyService() ) );
	ASSERT_FALSE( mgr.findService<InvalidService>( "dummyService" ) );
}

TEST( PluginManager, basic_start_service ) {
	CheckService check;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new CountService( check ) ) );
	mgr.start();
	ASSERT_TRUE( mgr.isStarted() );
	mgr.stop();
	ASSERT_FALSE( mgr.isStarted() );	
	mgr.unregisterService( "dummyService" );
	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 1 );
	ASSERT_TRUE( check.offline == 1 );
	ASSERT_TRUE( check.start == 1 );
	ASSERT_TRUE( check.stop == 1 );
	ASSERT_TRUE( check.loadConfig == 1 );
	ASSERT_TRUE( check.resetConfig == 0 );
}

TEST( PluginManager, basic_start_service_order2 ) {
	CheckService check;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	mgr.start();
	ASSERT_TRUE( mgr.registerService( new CountService( check ) ) );
	mgr.stop();
	mgr.unregisterService( "dummyService" );	
	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 1 );
	ASSERT_TRUE( check.offline == 1 );
	ASSERT_TRUE( check.start == 1 );
	ASSERT_TRUE( check.stop == 1 );
	ASSERT_TRUE( check.loadConfig == 1 );
	ASSERT_TRUE( check.resetConfig == 0 );
}

TEST( PluginManager, basic_start_service_order3 ) {
	CheckService check;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	mgr.start();
	ASSERT_TRUE( mgr.registerService( new CountService( check ) ) );
	mgr.unregisterService( "dummyService" );
	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 1 );
	ASSERT_TRUE( check.offline == 1 );
	ASSERT_TRUE( check.start == 1 );
	ASSERT_TRUE( check.stop == 1 );
	ASSERT_TRUE( check.loadConfig == 1 );
	ASSERT_TRUE( check.resetConfig == 0 );
}

TEST( PluginManager, resetConfig ) {
	CheckService check;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	mgr.resetConfig();
	ASSERT_TRUE( check.resetConfig == 0 );
	ASSERT_TRUE( mgr.registerService( new CountService( check ) ) );
	mgr.resetConfig();	
	ASSERT_TRUE( check.resetConfig == 1 );	
	mgr.start();
	mgr.resetConfig();	
	ASSERT_TRUE( check.resetConfig == 2 );	
	mgr.unregisterService( "dummyService" );
	mgr.resetConfig();	
	ASSERT_TRUE( check.resetConfig == 2 );	
	
	ASSERT_TRUE( check.attach == 1 );
	ASSERT_TRUE( check.dettach == 1 );
	ASSERT_TRUE( check.online == 1 );
	ASSERT_TRUE( check.offline == 1 );
	ASSERT_TRUE( check.start == 1 );
	ASSERT_TRUE( check.stop == 1 );
	ASSERT_TRUE( check.loadConfig == 1 );
}

TEST( PluginManager, start_service_with_dep1 ) {
	CheckService check1, check2;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	mgr.start();
	ASSERT_TRUE( mgr.registerService( new ServiceWithDependency( check2, "dummyService2", "dummyService1" ) ) );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );	
	mgr.unregisterService( "dummyService1" );
	mgr.unregisterService( "dummyService2" );
	
	ASSERT_TRUE( check1.attach == 1 );
	ASSERT_TRUE( check1.dettach == 1 );
	ASSERT_TRUE( check1.online == 1 );
	ASSERT_TRUE( check1.offline == 1 );
	ASSERT_TRUE( check1.start == 1 );
	ASSERT_TRUE( check1.stop == 1 );
	ASSERT_TRUE( check1.loadConfig == 1 );
	ASSERT_TRUE( check1.resetConfig == 0 );

	ASSERT_TRUE( check2.attach == 1 );
	ASSERT_TRUE( check2.dettach == 1 );
	ASSERT_TRUE( check2.online == 1 );
	ASSERT_TRUE( check2.offline == 1 );
	ASSERT_TRUE( check2.start == 1 );
	ASSERT_TRUE( check2.stop == 1 );
	ASSERT_TRUE( check2.loadConfig == 1 );
	ASSERT_TRUE( check2.resetConfig == 0 );
}

TEST( PluginManager, start_service_with_dep2 ) {
	CheckService check1, check2;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new ServiceWithDependency( check2, "dummyService2", "dummyService1" ) ) );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );
	mgr.start();
	mgr.unregisterService( "dummyService1" );
	mgr.unregisterService( "dummyService2" );
	
	ASSERT_TRUE( check1.attach == 1 );
	ASSERT_TRUE( check1.dettach == 1 );
	ASSERT_TRUE( check1.online == 1 );
	ASSERT_TRUE( check1.offline == 1 );
	ASSERT_TRUE( check1.start == 1 );
	ASSERT_TRUE( check1.stop == 1 );
	ASSERT_TRUE( check1.loadConfig == 1 );
	ASSERT_TRUE( check1.resetConfig == 0 );

	ASSERT_TRUE( check2.attach == 1 );
	ASSERT_TRUE( check2.dettach == 1 );
	ASSERT_TRUE( check2.online == 1 );
	ASSERT_TRUE( check2.offline == 1 );
	ASSERT_TRUE( check2.start == 1 );
	ASSERT_TRUE( check2.stop == 1 );
	ASSERT_TRUE( check2.loadConfig == 1 );
	ASSERT_TRUE( check2.resetConfig == 0 );
}

TEST( PluginManager, start_service_stop_a_service_running ) {
	CheckService check1, check2;
	zapper::plugin::PluginManager mgr( new util::task::DispatcherImpl() );

	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new ServiceWithDependency( check2, "dummyService2", "dummyService1" ) ) );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );
	mgr.start();
	mgr.unregisterService( "dummyService1" );
	ASSERT_TRUE( mgr.registerService( new CountService( check1, "dummyService1" ) ) );
	mgr.unregisterService( "dummyService1" );
	mgr.unregisterService( "dummyService2" );

	ASSERT_TRUE( check1.attach == 2 );
	ASSERT_TRUE( check1.dettach == 2 );
	ASSERT_TRUE( check1.online == 2 );
	ASSERT_TRUE( check1.offline == 2 );
	ASSERT_TRUE( check1.start == 2 );
	ASSERT_TRUE( check1.stop == 2 );
	ASSERT_TRUE( check1.loadConfig == 2 );
	ASSERT_TRUE( check1.resetConfig == 0 );

	ASSERT_TRUE( check2.attach == 1 );
	ASSERT_TRUE( check2.dettach == 1 );
	ASSERT_TRUE( check2.online == 2 );
	ASSERT_TRUE( check2.offline == 2 );
	ASSERT_TRUE( check2.start == 2 );
	ASSERT_TRUE( check2.stop == 2 );
	ASSERT_TRUE( check2.loadConfig == 2 );
	ASSERT_TRUE( check2.resetConfig == 0 );
}

TEST( PluginManager, run_method ) {
	util::task::DispatcherImpl *disp = new util::task::DispatcherImpl();
	zapper::plugin::PluginManager mgr( disp );
	mgr.initialize( "pluginmanager_test" );
	ASSERT_TRUE( mgr.registerService( new MethodService() ) );
	mgr.start();
	int counter=0;
	boost::function<void (MethodService *)> fnc = boost::bind(&MethodService::method1,_1,&counter);
	mgr.runMethod<MethodService>( "MethodService", fnc );
	disp->runAll();
	ASSERT_TRUE( counter == 1 );
	mgr.stop();
}

