/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../../../src/plugin/pluginmanager.h"
#include "../../../src/plugin/plugin.h"
#include "../../../src/plugin/dependency.h"

class CheckService {
public:
	CheckService( void ) { attach = dettach = online = offline = start = stop = loadConfig = resetConfig = 0; }

	int attach;
	int dettach;
	int online;
	int offline;
	int start;
	int stop;
	int loadConfig;
	int resetConfig;
};

class DummyService : public zapper::plugin::ServiceImpl<DummyService> {
public:
	DummyService( const std::string &instance="dummyService" ) : zapper::plugin::ServiceImpl<DummyService>( instance ) {}
	virtual ~DummyService( void ) {}

private:
	std::string _instance;
};

class MethodService : public zapper::plugin::ServiceImpl<MethodService> {
public:
	MethodService( void ) : zapper::plugin::ServiceImpl<MethodService>( "MethodService" ) {}
	void method1( int *x ) {
		(*x)++;
	}
};

class CountService : public DummyService {
public:
	CountService( CheckService &check, const std::string &instance="dummyService" ) : DummyService( instance ), _check( check ) {}

protected:
	//	Aux start/stop
	virtual bool onStart( void ) { _check.start++; return true; }
	virtual void onStop( void ) { _check.stop++; }

	//	Aux config
	virtual void onLoadConfig( void ) { _check.loadConfig++; }
	virtual void onResetConfig( void ) { _check.resetConfig++; }

	//	Aux attach
	virtual void onAttach( void ) { _check.attach++; }
	virtual void onDettach( void ) { _check.dettach++; }

	//	Aux online
	virtual bool onOnline( void ) { _check.online++; return true; }
	virtual void onOffline( void ) { _check.offline++; }

private:
	CheckService &_check;

	CountService &operator=( CountService & /*cs*/ ) { return *this; }
};

class ServiceWithDependency : public CountService {
public:
	ServiceWithDependency( CheckService &check, const std::string &instance, const std::string &dep ) : CountService( check, instance ), _dep(dep) {}
	virtual void onAttach( void ) {
		addDependency( new zapper::plugin::SimpleDependency( _dep ) );
		CountService::onAttach();
	}

private:
	std::string _dep;
};

class UnresolvedService : public CountService {
public:
	UnresolvedService( CheckService &check ) : CountService( check ) {}
	virtual void onAttach( void ) {
		addDependency( new zapper::plugin::SimpleDependency( "nothing" ) );
		CountService::onAttach();
	}
};

class InvalidService : public zapper::plugin::ServiceImpl<InvalidService> {
	InvalidService( void ) : zapper::plugin::ServiceImpl<InvalidService>( "invalidService" ) {}
	virtual ~InvalidService( void ) {}
};

