/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "util.h"
#include <util/mcr.h>
#include <boost/filesystem.hpp>

namespace fs = boost::filesystem;

namespace util {

tuner::Pmt *createPmt( tuner::ID srvID, tuner::ID srvPID, tuner::ID pcrPID, std::vector<tuner::ElementaryInfo> elems ) {
	//	Contruct Service
	tuner::desc::MapOfDescriptors descs;
	return new tuner::Pmt( srvPID, 1, srvID, pcrPID, descs, elems );
}

void addStream( std::vector<tuner::ElementaryInfo> &elems, util::BYTE type, tuner::ID pid ) {
	tuner::ElementaryInfo elem;
	elem.streamType = type;
	elem.pid = pid;
	elems.push_back( elem );
}

void addStreamType( std::vector<tuner::ElementaryInfo> &elems, tuner::ID pid, tuner::ID type, util::BYTE pri ) {
	tuner::ElementaryInfo elem;
	elem.streamType = (util::BYTE) type;
	elem.pid = pid;
	elem.descriptors[tuner::desc::ident::stream_identifier] = pri;
	elems.push_back( elem );
}

tuner::Service *createService( tuner::ID nitID, tuner::ID tsID, tuner::ID srvID, tuner::ID srvPID, tuner::Pmt *pmt ) {
	tuner::Service *srv = new tuner::Service( 8, nitID, tsID, srvID, srvPID );
	boost::shared_ptr<tuner::Pmt> pmtPtr(pmt);
	srv->process( pmtPtr );
	return srv;
}

tuner::Service *createSimpleService() {
	tuner::ID nitID  =0x73a;
	tuner::ID tsID   =0x73a;
	tuner::ID srvID  =0xe741;
	tuner::ID srvPID =0x102;
	tuner::ID pcrPID =0x120;
	
	std::vector<tuner::ElementaryInfo> elems;		
	addStreamType( elems, 0x121, 0x1b, 0x0  );
	addStreamType( elems, 0x122, 0x11, 0x10 );
	addStreamType( elems, 0x214, 0x06, 0x30 );
	tuner::Pmt *pmt = createPmt( srvID, srvPID, pcrPID, elems );
	return createService( nitID, tsID, srvID, srvPID, pmt );
}

tuner::Service *createComplexService() {
	tuner::ID nitID  =0x73a;
	tuner::ID tsID   =0x73a;
	tuner::ID srvID  =0xe741;
	tuner::ID srvPID =0x102;
	tuner::ID pcrPID =0x120;
	
	std::vector<tuner::ElementaryInfo> elems;		
	addStreamType( elems, 0x121, 0x1b, 0x0  );
	addStreamType( elems, 0x122, 0x1b, 0x1  );	
	addStreamType( elems, 0x123, 0x11, 0x10 );
	addStreamType( elems, 0x124, 0x11, 0x11 );	
	addStreamType( elems, 0x125, 0x06, 0x30 );
	addStreamType( elems, 0x126, 0x06, 0x31 );	
	tuner::Pmt *pmt = createPmt( srvID, srvPID, pcrPID, elems );
	return createService( nitID, tsID, srvID, srvPID, pmt );
}

}
